/*
    Your use of this file is governed by the Microsoft Services Agreement http://go.microsoft.com/fwlink/?LinkId=266419.

    This file also contains the following Promise implementation (with a few small modifications):
        * @overview es6-promise - a tiny implementation of Promises/A+.
        * @copyright Copyright (c) 2014 Yehuda Katz, Tom Dale, Stefan Penner and contributors (Conversion to ES6 API by Jake Archibald)
        * @license   Licensed under MIT license
        *            See https://raw.githubusercontent.com/jakearchibald/es6-promise/master/LICENSE
        * @version   2.3.0
*/
var OfficeExt,__extends=this&&this.__extends||function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}();!function(e){var t=function(){function e(){}return e.prototype.isMsAjaxLoaded=function(){return!!("undefined"!=typeof Sys&&"undefined"!=typeof Type&&Sys.StringBuilder&&"function"==typeof Sys.StringBuilder&&Type.registerNamespace&&"function"==typeof Type.registerNamespace&&Type.registerClass&&"function"==typeof Type.registerClass&&"function"==typeof Function._validateParams&&Sys.Serialization&&Sys.Serialization.JavaScriptSerializer&&"function"==typeof Sys.Serialization.JavaScriptSerializer.serialize)},e.prototype.loadMsAjaxFull=function(e){var t=("https:"===window.location.protocol.toLowerCase()?"https:":"http:")+"//ajax.aspnetcdn.com/ajax/3.5/MicrosoftAjax.js";OSF.OUtil.loadScript(t,e)},Object.defineProperty(e.prototype,"msAjaxError",{get:function(){return null==this._msAjaxError&&this.isMsAjaxLoaded()&&(this._msAjaxError=Error),this._msAjaxError},set:function(e){this._msAjaxError=e},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"msAjaxString",{get:function(){return null==this._msAjaxString&&this.isMsAjaxLoaded()&&(this._msAjaxString=String),this._msAjaxString},set:function(e){this._msAjaxString=e},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"msAjaxDebug",{get:function(){return null==this._msAjaxDebug&&this.isMsAjaxLoaded()&&(this._msAjaxDebug=Sys.Debug),this._msAjaxDebug},set:function(e){this._msAjaxDebug=e},enumerable:!0,configurable:!0}),e}();e.MicrosoftAjaxFactory=t}(OfficeExt||(OfficeExt={}));var OsfMsAjaxFactory=new OfficeExt.MicrosoftAjaxFactory,OSF=OSF||{};!function(e){var t=function(){function e(e){this._internalStorage=e}return e.prototype.getItem=function(e){try{return this._internalStorage&&this._internalStorage.getItem(e)}catch(e){return null}},e.prototype.setItem=function(e,t){try{this._internalStorage&&this._internalStorage.setItem(e,t)}catch(e){}},e.prototype.clear=function(){try{this._internalStorage&&this._internalStorage.clear()}catch(e){}},e.prototype.removeItem=function(e){try{this._internalStorage&&this._internalStorage.removeItem(e)}catch(e){}},e.prototype.getKeysWithPrefix=function(e){var t=[];try{for(var n=this._internalStorage&&this._internalStorage.length||0,r=0;r<n;r++){var o=this._internalStorage.key(r);0===o.indexOf(e)&&t.push(o)}}catch(e){}return t},e.prototype.isLocalStorageAvailable=function(){return null!=this._internalStorage},e}();e.SafeStorage=t}(OfficeExt||(OfficeExt={})),OSF.XdmFieldName={ConversationUrl:"ConversationUrl",AppId:"AppId"},OSF.TestFlightStart=1e3,OSF.TestFlightEnd=1009,OSF.FlightNames={UseOriginNotUrl:0,AddinEnforceHttps:2,FirstPartyAnonymousProxyReadyCheckTimeout:6,AddinRibbonIdAllowUnknown:9,ManifestParserDevConsoleLog:15,AddinActionDefinitionHybridMode:18,UseActionIdForUILessCommand:20,RequirementSetRibbonApiOnePointTwo:21,SetFocusToTaskpaneIsEnabled:22,ShortcutInfoArrayInUserPreferenceData:23,OSFTestFlight1000:OSF.TestFlightStart,OSFTestFlight1001:OSF.TestFlightStart+1,OSFTestFlight1002:OSF.TestFlightStart+2,OSFTestFlight1003:OSF.TestFlightStart+3,OSFTestFlight1004:OSF.TestFlightStart+4,OSFTestFlight1005:OSF.TestFlightStart+5,OSFTestFlight1006:OSF.TestFlightStart+6,OSFTestFlight1007:OSF.TestFlightStart+7,OSFTestFlight1008:OSF.TestFlightStart+8,OSFTestFlight1009:OSF.TestFlightEnd},OSF.TrustUXFlightValues={TrustUXControlA:0,TrustUXExperimentB:1,TrustUXExperimentC:2},OSF.FlightTreatmentNames={AddinDialogIFrameContentWindowKillSwitch:"Microsoft.Office.SharedOnline.AddinDialogIFrameContentWindowKillSwitch",AddinTrustUXImprovement:"Microsoft.Office.SharedOnline.AddinTrustUXImprovement",AllowStorageAccessByUserActivationOnIFrameCheck:"Microsoft.Office.SharedOnline.AllowStorageAccessByUserActivationOnIFrameCheck",BlockAutoOpenAddInIfStoreDisabled:"Microsoft.Office.SharedOnline.BlockAutoOpenAddInIfStoreDisabled",CheckProxyIsReadyRetry:"Microsoft.Office.SharedOnline.OEP.CheckProxyIsReadyRetry",InsertionDialogFixesEnabled:"Microsoft.Office.SharedOnline.InsertionDialogFixesEnabled",TeachingUIForPrivateCatelogEnabled:"Microsoft.Office.SharedOnline.TeachingUIForPrivateCatelogEnabled",WopiPreinstalledAddInsEnabled:"Microsoft.Office.SharedOnline.WopiPreinstalledAddInsEnabled",WopiUseNewActivate:"Microsoft.Office.SharedOnline.WopiUseNewActivate",MosManifestEnabled:"Microsoft.Office.SharedOnline.OEP.MosManifest"},OSF.Flights=[],OSF.IntFlights={},OSF.Settings={},OSF.WindowNameItemKeys={BaseFrameName:"baseFrameName",HostInfo:"hostInfo",XdmInfo:"xdmInfo",SerializerVersion:"serializerVersion",AppContext:"appContext",Flights:"flights"},OSF.OUtil=function(){var e=-1,t="&_xdm_Info=",n="&_serializer_version=",r="&_flights=",o="_xdm_",i="#",a="class",s={},l=null,c=null,d=(new Date).getTime();function u(){var e=2147483647*Math.random();return(e^=d^(new Date).getMilliseconds()<<Math.floor(21*Math.random())).toString(16)}function f(){if(!l){try{var e=window.sessionStorage}catch(t){e=null}l=new OfficeExt.SafeStorage(e)}return l}function p(e){var t,n,r=[],o=[],i=e.length;for(t=0;t<i;t++)(n=e[t]).tabIndex?n.tabIndex>0?o.push(n):0===n.tabIndex&&r.push(n):r.push(n);return o=o.sort((function(e,t){var n=e.tabIndex-t.tabIndex;return 0===n&&(n=o.indexOf(e)-o.indexOf(t)),n})),[].concat(o,r)}return{set_entropy:function(e){if("string"==typeof e)for(var t=0;t<e.length;t+=4){for(var n=0,r=0;r<4&&t+r<e.length;r++)n=(n<<8)+e.charCodeAt(t+r);d^=n}else d^="number"==typeof e?e:2147483647*Math.random();d&=2147483647},extend:function(e,t){var n=function(){};n.prototype=t.prototype,e.prototype=new n,e.prototype.constructor=e,e.uber=t.prototype,t.prototype.constructor===Object.prototype.constructor&&(t.prototype.constructor=t)},setNamespace:function(e,t){t&&e&&!t[e]&&(t[e]={})},unsetNamespace:function(e,t){t&&e&&t[e]&&delete t[e]},serializeSettings:function(e){var t={};for(var n in e){var r=e[n];try{r=JSON?JSON.stringify(r,(function(e,t){return OSF.OUtil.isDate(this[e])?OSF.DDA.SettingsManager.DateJSONPrefix+this[e].getTime()+OSF.DDA.SettingsManager.DataJSONSuffix:t})):Sys.Serialization.JavaScriptSerializer.serialize(r),t[n]=r}catch(e){}}return t},deserializeSettings:function(e){var t={};for(var n in e=e||{}){var r=e[n];try{r=JSON?JSON.parse(r,(function(e,t){var n;return"string"==typeof t&&t&&t.length>6&&t.slice(0,5)===OSF.DDA.SettingsManager.DateJSONPrefix&&t.slice(-1)===OSF.DDA.SettingsManager.DataJSONSuffix&&(n=new Date(parseInt(t.slice(5,-1))))?n:t})):Sys.Serialization.JavaScriptSerializer.deserialize(r,!0),t[n]=r}catch(e){}}return t},loadScript:function(e,t,n){if(e&&t){var r=window.document,o=s[e];if(o)o.loaded?t():o.pendingCallbacks.push(t);else{var i=r.createElement("script");i.type="text/javascript",o={loaded:!1,pendingCallbacks:[t],timer:null},s[e]=o;var a=function(){null!=o.timer&&(clearTimeout(o.timer),delete o.timer),o.loaded=!0;for(var e=o.pendingCallbacks.length,t=0;t<e;t++){o.pendingCallbacks.shift()()}},l=function(t){delete s[e],null!=o.timer&&(clearTimeout(o.timer),delete o.timer);for(var n=o.pendingCallbacks.length,r=0;r<n;r++){o.pendingCallbacks.shift()(t)}};i.readyState?i.onreadystatechange=function(){"loaded"!=i.readyState&&"complete"!=i.readyState||(i.onreadystatechange=null,a())}:i.onload=a,i.onerror=l,n=n||3e4,o.timer=setTimeout((function(){window.navigator.userAgent.indexOf("Trident")>0?l(null):l(new Event("Script load timed out"))}),n),i.setAttribute("crossOrigin","anonymous"),i.src=e,r.getElementsByTagName("head")[0].appendChild(i)}}},loadCSS:function(e){if(e){var t=window.document,n=t.createElement("link");n.type="text/css",n.rel="stylesheet",n.href=e,t.getElementsByTagName("head")[0].appendChild(n)}},parseEnum:function(e,t){var n=t[e.trim()];if(void 0===n)throw OsfMsAjaxFactory.msAjaxDebug.trace("invalid enumeration string:"+e),OsfMsAjaxFactory.msAjaxError.argument("str");return n},delayExecutionAndCache:function(){var e={calc:arguments[0]};return function(){return e.calc&&(e.val=e.calc.apply(this,arguments),delete e.calc),e.val}},getUniqueId:function(){return(e+=1).toString()},formatString:function(){var e=arguments,t=e[0];return t.replace(/{(\d+)}/gm,(function(t,n){var r=parseInt(n,10)+1;return void 0===e[r]?"{"+n+"}":e[r]}))},generateConversationId:function(){return[u(),u(),(new Date).getTime().toString()].join("_")},getFrameName:function(e){return o+e+this.generateConversationId()},addXdmInfoAsHash:function(e,n){return OSF.OUtil.addInfoAsHash(e,t,n,!1)},addSerializerVersionAsHash:function(e,t){return OSF.OUtil.addInfoAsHash(e,n,t,!0)},addFlightsAsHash:function(e,t){return OSF.OUtil.addInfoAsHash(e,r,t,!0)},addInfoAsHash:function(e,t,n,r){var o,a=(e=e.trim()||"").split(i),s=a.shift(),l=a.join(i);return o=r?[t,encodeURIComponent(n),l].join(""):[l,t,n].join(""),[s,i,o].join("")},parseHostInfoFromWindowName:function(e,t){return OSF.OUtil.parseInfoFromWindowName(e,t,OSF.WindowNameItemKeys.HostInfo)},parseXdmInfo:function(e){var t=OSF.OUtil.parseXdmInfoWithGivenFragment(e,window.location.hash);return t||(t=OSF.OUtil.parseXdmInfoFromWindowName(e,window.name)),t},parseXdmInfoFromWindowName:function(e,t){return OSF.OUtil.parseInfoFromWindowName(e,t,OSF.WindowNameItemKeys.XdmInfo)},parseXdmInfoWithGivenFragment:function(e,n){return OSF.OUtil.parseInfoWithGivenFragment(t,o,!1,e,n)},parseSerializerVersion:function(e){var t=OSF.OUtil.parseSerializerVersionWithGivenFragment(e,window.location.hash);return isNaN(t)&&(t=OSF.OUtil.parseSerializerVersionFromWindowName(e,window.name)),t},parseSerializerVersionFromWindowName:function(e,t){return parseInt(OSF.OUtil.parseInfoFromWindowName(e,t,OSF.WindowNameItemKeys.SerializerVersion))},parseSerializerVersionWithGivenFragment:function(e,t){return parseInt(OSF.OUtil.parseInfoWithGivenFragment(n,"_serializer_version=",!0,e,t))},parseFlights:function(e){var t=OSF.OUtil.parseFlightsWithGivenFragment(e,window.location.hash);return 0==t.length&&(t=OSF.OUtil.parseFlightsFromWindowName(e,window.name)),t},checkFlight:function(e){return OSF.Flights&&OSF.Flights.indexOf(e)>=0},pushFlight:function(e){return OSF.Flights.indexOf(e)<0&&(OSF.Flights.push(e),!0)},getBooleanSetting:function(e){return OSF.OUtil.getBooleanFromDictionary(OSF.Settings,e)},getBooleanFromDictionary:function(e,t){var n=e&&t&&void 0!==e[t]&&e[t]&&("string"==typeof e[t]&&"TRUE"===e[t].toUpperCase()||"boolean"==typeof e[t]&&e[t]);return void 0!==n&&n},getIntFromDictionary:function(e,t){return e&&t&&void 0!==e[t]&&"string"==typeof e[t]?parseInt(e[t]):NaN},pushIntFlight:function(e,t){return!(e in OSF.IntFlights)&&(OSF.IntFlights[e]=t,!0)},getIntFlight:function(e){return OSF.IntFlights&&e in OSF.IntFlights?OSF.IntFlights[e]:NaN},parseFlightsFromWindowName:function(e,t){return OSF.OUtil.parseArrayWithDefault(OSF.OUtil.parseInfoFromWindowName(e,t,OSF.WindowNameItemKeys.Flights))},parseFlightsWithGivenFragment:function(e,t){return OSF.OUtil.parseArrayWithDefault(OSF.OUtil.parseInfoWithGivenFragment(r,"_flights=",!0,e,t))},parseArrayWithDefault:function(e){var t=[];try{t=JSON.parse(e)}catch(e){}return Array.isArray(t)||(t=[]),t},parseInfoFromWindowName:function(e,t,n){try{var r=JSON.parse(t),o=null!=r?r[n]:null,i=f();if(!e&&i&&null!=r){var a=r[OSF.WindowNameItemKeys.BaseFrameName]+n;o?i.setItem(a,o):o=i.getItem(a)}return o}catch(e){return null}},parseInfoWithGivenFragment:function(e,t,n,r,o){var i=o.split(e),a=i.length>1?i[i.length-1]:null;n&&null!=a&&(a.indexOf("&")>=0&&(a=a.split("&")[0]),a=decodeURIComponent(a));var s=f();if(!r&&s){var l=window.name.indexOf(t);if(l>-1){var c=window.name.indexOf(";",l);-1==c&&(c=window.name.length);var d=window.name.substring(l,c);a?s.setItem(d,a):a=s.getItem(d)}}return a},getConversationId:function(){var e=window.location.search,t=null;if(e){var n=e.indexOf("&");(t=n>0?e.substring(1,n):e.substr(1))&&"="===t.charAt(t.length-1)&&(t=t.substring(0,t.length-1))&&(t=decodeURIComponent(t))}return t},getInfoItems:function(e){var t=e.split("$");return void 0===t[1]&&(t=e.split("|")),void 0===t[1]&&(t=e.split("%7C")),t},getXdmFieldValue:function(e,t){var n="",r=OSF.OUtil.parseXdmInfo(t);if(r){var o=OSF.OUtil.getInfoItems(r);if(null!=o&&o.length>=3)switch(e){case OSF.XdmFieldName.ConversationUrl:n=o[2];break;case OSF.XdmFieldName.AppId:n=o[1]}}return n},validateParamObject:function(e,t,n){var r=Function._validateParams(arguments,[{name:"params",type:Object,mayBeNull:!1},{name:"expectedProperties",type:Object,mayBeNull:!1},{name:"callback",type:Function,mayBeNull:!0}]);if(r)throw r;for(var o in t)if(r=Function._validateParameter(e[o],t[o],o))throw r},writeProfilerMark:function(e){window.msWriteProfilerMark&&(window.msWriteProfilerMark(e),OsfMsAjaxFactory.msAjaxDebug.trace(e))},outputDebug:function(e){void 0!==OsfMsAjaxFactory&&OsfMsAjaxFactory.msAjaxDebug&&OsfMsAjaxFactory.msAjaxDebug.trace&&OsfMsAjaxFactory.msAjaxDebug.trace(e)},defineNondefaultProperty:function(e,t,n,r){for(var o in n=n||{},r){var i=r[o];null==n[i]&&(n[i]=!0)}return Object.defineProperty(e,t,n),e},defineNondefaultProperties:function(e,t,n){for(var r in t=t||{})OSF.OUtil.defineNondefaultProperty(e,r,t[r],n);return e},defineEnumerableProperty:function(e,t,n){return OSF.OUtil.defineNondefaultProperty(e,t,n,["enumerable"])},defineEnumerableProperties:function(e,t){return OSF.OUtil.defineNondefaultProperties(e,t,["enumerable"])},defineMutableProperty:function(e,t,n){return OSF.OUtil.defineNondefaultProperty(e,t,n,["writable","enumerable","configurable"])},defineMutableProperties:function(e,t){return OSF.OUtil.defineNondefaultProperties(e,t,["writable","enumerable","configurable"])},finalizeProperties:function(e,t){t=t||{};for(var n=Object.getOwnPropertyNames(e),r=n.length,o=0;o<r;o++){var i=n[o],a=Object.getOwnPropertyDescriptor(e,i);a.get||a.set||(a.writable=t.writable||!1),a.configurable=t.configurable||!1,a.enumerable=t.enumerable||!0,Object.defineProperty(e,i,a)}return e},mapList:function(e,t){var n=[];if(e)for(var r in e)n.push(t(e[r]));return n},listContainsKey:function(e,t){for(var n in e)if(t==n)return!0;return!1},listContainsValue:function(e,t){for(var n in e)if(t==e[n])return!0;return!1},augmentList:function(e,t){var n=e.push?function(t,n){e.push(n)}:function(t,n){e[t]=n};for(var r in t)n(r,t[r])},redefineList:function(e,t){for(var n in e)delete e[n];for(var r in t)e[r]=t[r]},isArray:function(e){return"[object Array]"===Object.prototype.toString.apply(e)},isFunction:function(e){return"[object Function]"===Object.prototype.toString.apply(e)},isDate:function(e){return"[object Date]"===Object.prototype.toString.apply(e)},addEventListener:function(e,t,n){e.addEventListener?e.addEventListener(t,n,!1):Sys.Browser.agent===Sys.Browser.InternetExplorer&&e.attachEvent?e.attachEvent("on"+t,n):e["on"+t]=n},removeEventListener:function(e,t,n){e.removeEventListener?e.removeEventListener(t,n,!1):Sys.Browser.agent===Sys.Browser.InternetExplorer&&e.detachEvent?e.detachEvent("on"+t,n):e["on"+t]=null},xhrGet:function(e,t,n){var r;try{(r=new XMLHttpRequest).onreadystatechange=function(){4==r.readyState&&(200==r.status?t(r.responseText):n(r.status))},r.open("GET",e,!0),r.send()}catch(e){n(e)}},encodeBase64:function(e){if(!e)return e;var t,n,r,o,i,a,s,l="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",c=[],d=[],u=0,f=e.length;do{t=e.charCodeAt(u++),n=e.charCodeAt(u++),r=e.charCodeAt(u++),s=0,o=255&t,i=t>>8,a=255&n,d[s++]=o>>2,d[s++]=(3&o)<<4|i>>4,d[s++]=(15&i)<<2|a>>6,d[s++]=63&a,isNaN(n)||(o=n>>8,i=255&r,a=r>>8,d[s++]=o>>2,d[s++]=(3&o)<<4|i>>4,d[s++]=(15&i)<<2|a>>6,d[s++]=63&a),isNaN(n)?d[s-1]=64:isNaN(r)&&(d[s-2]=64,d[s-1]=64);for(var p=0;p<s;p++)c.push(l.charAt(d[p]))}while(u<f);return c.join("")},getSessionStorage:function(){return f()},getLocalStorage:function(){if(!c){try{var e=window.localStorage}catch(t){e=null}c=new OfficeExt.SafeStorage(e)}return c},convertIntToCssHexColor:function(e){return"#"+(Number(e)+16777216).toString(16).slice(-6)},attachClickHandler:function(e,t){e.onclick=function(e){t()},e.ontouchend=function(e){t(),e.preventDefault()}},getQueryStringParamValue:function(e,t){var n=Function._validateParams(arguments,[{name:"queryString",type:String,mayBeNull:!1},{name:"paramName",type:String,mayBeNull:!1}]);if(n)return OsfMsAjaxFactory.msAjaxDebug.trace("OSF_Outil_getQueryStringParamValue: Parameters cannot be null."),"";var r=new RegExp("[\\?&]"+t+"=([^&#]*)","i");return r.test(e)?r.exec(e)[1]:(OsfMsAjaxFactory.msAjaxDebug.trace("OSF_Outil_getQueryStringParamValue: The parameter is not found."),"")},getHostnamePortionForLogging:function(e){var t=Function._validateParams(arguments,[{name:"hostname",type:String,mayBeNull:!1}]);if(t)return"";var n=e.split("."),r=n.length;return r>=2?n[r-2]+"."+n[r-1]:1==r?n[0]:void 0},isiOS:function(){return!!window.navigator.userAgent.match(/(iPad|iPhone|iPod)/g)},isChrome:function(){return window.navigator.userAgent.indexOf("Chrome")>0&&!OSF.OUtil.isEdge()},isEdge:function(){return window.navigator.userAgent.indexOf("Edge")>0},isIE:function(){return window.navigator.userAgent.indexOf("Trident")>0},isFirefox:function(){return window.navigator.userAgent.indexOf("Firefox")>0},startsWith:function(e,t,n){return n?e.substr(0,t.length)===t:e.startsWith(t)},containsPort:function(e,t,n,r){return this.startsWith(e,t+"//"+n+":"+r,!0)||this.startsWith(e,n+":"+r,!0)},getRedundandPortString:function(e,t){return e&&t?"https:"==t.protocol&&this.containsPort(e,"https:",t.hostname,"443")?":443":"http:"==t.protocol&&this.containsPort(e,"http:",t.hostname,"80")?":80":"":""},removeChar:function(e,t){return t<e.length-1?e.substring(0,t)+e.substring(t+1):t==e.length-1?e.substring(0,e.length-1):e},cleanUrlOfChar:function(e,t){var n;for(n=0;n<e.length;n++)if(e.charAt(n)===t){if(n+1>=e.length)return this.removeChar(e,n);if("/"===t){if("?"===e.charAt(n+1)||"#"===e.charAt(n+1))return this.removeChar(e,n)}else if("?"===t&&"#"===e.charAt(n+1))return this.removeChar(e,n)}return e},cleanUrl:function(e){if(e=this.cleanUrlOfChar(e,"/"),e=this.cleanUrlOfChar(e,"?"),"https://"==(e=this.cleanUrlOfChar(e,"#")).substr(0,8))-1!=(t=e.indexOf(":443"))&&(t!=e.length-4&&"/"!=e.charAt(t+4)&&"?"!=e.charAt(t+4)&&"#"!=e.charAt(t+4)||(e=e.substring(0,t)+e.substring(t+4)));else if("http://"==e.substr(0,7)){var t;-1!=(t=e.indexOf(":80"))&&(t!=e.length-3&&"/"!=e.charAt(t+3)&&"?"!=e.charAt(t+3)&&"#"!=e.charAt(t+3)||(e=e.substring(0,t)+e.substring(t+3)))}return e},parseUrl:function(e,t){if(void 0===t&&(t=!1),void 0!==e&&e){var n="NotHttps",r=this.isIE(),o={protocol:void 0,hostname:void 0,host:void 0,port:void 0,pathname:void 0,search:void 0,hash:void 0,isPortPartOfUrl:void 0};try{if(r){var i=document.createElement("a");if(i.href=e,!(i&&i.protocol&&i.host&&i.hostname&&i.href&&this.cleanUrl(i.href).toLowerCase()===this.cleanUrl(e).toLowerCase()))throw"InvalidUrl";if(OSF.OUtil.checkFlight(OSF.FlightNames.AddinEnforceHttps)&&t&&"https:"!=i.protocol)throw new Error(n);var a=this.getRedundandPortString(e,i);o.protocol=i.protocol,o.hostname=i.hostname,o.port=""==a?i.port:"",o.host=""!=a?i.hostname:i.host,o.pathname=(r?"/":"")+i.pathname,o.search=i.search,o.hash=i.hash,o.isPortPartOfUrl=this.containsPort(e,i.protocol,i.hostname,i.port)}else{var s=new URL(e);if(s&&s.protocol&&s.host&&s.hostname){if(OSF.OUtil.checkFlight(OSF.FlightNames.AddinEnforceHttps)&&t&&"https:"!=s.protocol)throw new Error(n);o.protocol=s.protocol,o.hostname=s.hostname,o.port=s.port,o.host=s.host,o.pathname=s.pathname,o.search=s.search,o.hash=s.hash,o.isPortPartOfUrl=s.host.lastIndexOf(":"+s.port)==s.host.length-s.port.length-1}}}catch(e){if(e.message===n)throw e}return o}},shallowCopy:function(e){if(null==e)return null;if(e instanceof Object){if(Array.isArray(e)){for(var t=[],n=0;n<e.length;n++)t.push(e[n]);return t}var r=e.constructor();for(var o in e)e.hasOwnProperty(o)&&(r[o]=e[o]);return r}return e},createObject:function(e){var t=null;if(e){t={};for(var n=e.length,r=0;r<n;r++)t[e[r].name]=e[r].value}return t},addClass:function(e,t){if(!OSF.OUtil.hasClass(e,t)){var n=e.getAttribute(a);n?e.setAttribute(a,n+" "+t):e.setAttribute(a,t)}},removeClass:function(e,t){if(OSF.OUtil.hasClass(e,t)){var n=e.getAttribute(a),r=new RegExp("(\\s|^)"+t+"(\\s|$)");n=n.replace(r,""),e.setAttribute(a,n)}},hasClass:function(e,t){var n=e.getAttribute(a);return n&&n.match(new RegExp("(\\s|^)"+t+"(\\s|$)"))},focusToFirstTabbable:function(e,t){var n,r,o=!1,i=function(e){o=!0},a=function(e,t,n){return t<0||t>e||0===t&&n?-1:t!==e-1||n?n?t-1:t+1:-1};if(e=p(e),n=t?e.length-1:0,0===e.length)return null;for(;!o&&n>=0&&n<e.length;)r=e[n],window.focus(),r.addEventListener("focus",i),r.focus(),r.removeEventListener("focus",i),n=a(e.length,n,t),o||r!==document.activeElement||(o=!0);return o?r:null},focusToNextTabbable:function(e,t,n){var r,o,i,a=!1,s=function(e){a=!0},l=function(e,t,n){return t<0||t>e||0===t&&n?-1:t!==e-1||n?n?t-1:t+1:-1};if(r=function(e,t){for(var n=0;n<e.length;n++)if(e[n]===t)return n;return-1}(e=p(e),t),(o=l(e.length,r,n))<0)return null;for(;!a&&o>=0&&o<e.length;)(i=e[o]).addEventListener("focus",s),i.focus(),i.removeEventListener("focus",s),o=l(e.length,o,n),a||i!==document.activeElement||(a=!0);return a?i:null},isNullOrUndefined:function(e){return void 0===e||null===e},stringEndsWith:function(e,t){if(!OSF.OUtil.isNullOrUndefined(e)&&!OSF.OUtil.isNullOrUndefined(t)){if(t.length>e.length)return!1;if(e.substr(e.length-t.length)===t)return!0}return!1},hashCode:function(e){var t=0;if(!OSF.OUtil.isNullOrUndefined(e))for(var n=0,r=e.length;n<r;)t=(t<<5)-t+e.charCodeAt(n++)|0;return t},getValue:function(e,t){return OSF.OUtil.isNullOrUndefined(e)?t:e},externalNativeFunctionExists:function(e){return"unknown"===e||"undefined"!==e}}}(),OSF.OUtil.Guid=function(){var e=["0","1","2","3","4","5","6","7","8","9","a","b","c","d","e","f"];return{generateNewGuid:function(){for(var t="",n=(new Date).getTime(),r=0;r<32&&n>0;r++)8!=r&&12!=r&&16!=r&&20!=r||(t+="-"),t+=e[n%16],n=Math.floor(n/16);for(;r<32;r++)8!=r&&12!=r&&16!=r&&20!=r||(t+="-"),t+=e[Math.floor(16*Math.random())];return t}}}();try{OSF.Flights=OSF.OUtil.parseFlights(!0)}catch(e){}window.OSF=OSF,OSF.OUtil.setNamespace("OSF",window),OSF.MessageIDs={FetchBundleUrl:0,LoadReactBundle:1,LoadBundleSuccess:2,LoadBundleError:3},OSF.AppName={Unsupported:0,Excel:1,Word:2,PowerPoint:4,Outlook:8,ExcelWebApp:16,WordWebApp:32,OutlookWebApp:64,Project:128,AccessWebApp:256,PowerpointWebApp:512,ExcelIOS:1024,Sway:2048,WordIOS:4096,PowerPointIOS:8192,Access:16384,Lync:32768,OutlookIOS:65536,OneNoteWebApp:131072,OneNote:262144,ExcelWinRT:524288,WordWinRT:1048576,PowerpointWinRT:2097152,OutlookAndroid:4194304,OneNoteWinRT:8388608,ExcelAndroid:8388609,VisioWebApp:8388610,OneNoteIOS:8388611,WordAndroid:8388613,PowerpointAndroid:8388614,Visio:8388615,OneNoteAndroid:4194305},OSF.InternalPerfMarker={DataCoercionBegin:"Agave.HostCall.CoerceDataStart",DataCoercionEnd:"Agave.HostCall.CoerceDataEnd"},OSF.HostCallPerfMarker={IssueCall:"Agave.HostCall.IssueCall",ReceiveResponse:"Agave.HostCall.ReceiveResponse",RuntimeExceptionRaised:"Agave.HostCall.RuntimeExecptionRaised"},OSF.AgaveHostAction={Select:0,UnSelect:1,CancelDialog:2,InsertAgave:3,CtrlF6In:4,CtrlF6Exit:5,CtrlF6ExitShift:6,SelectWithError:7,NotifyHostError:8,RefreshAddinCommands:9,PageIsReady:10,TabIn:11,TabInShift:12,TabExit:13,TabExitShift:14,EscExit:15,F2Exit:16,ExitNoFocusable:17,ExitNoFocusableShift:18,MouseEnter:19,MouseLeave:20,UpdateTargetUrl:21,InstallCustomFunctions:22,SendTelemetryEvent:23,UninstallCustomFunctions:24,SendMessage:25,LaunchExtensionComponent:26,StopExtensionComponent:27,RestartExtensionComponent:28,EnableTaskPaneHeaderButton:29,DisableTaskPaneHeaderButton:30,TaskPaneHeaderButtonClicked:31,RemoveAppCommandsAddin:32,RefreshRibbonGallery:33,GetOriginalControlId:34,OfficeJsReady:35,InsertDevManifest:36,InsertDevManifestError:37,SendCustomerContent:38,KeyboardShortcuts:39},OSF.SharedConstants={NotificationConversationIdSuffix:"_ntf"},OSF.DialogMessageType={DialogMessageReceived:0,DialogParentMessageReceived:1,DialogClosed:12006},OSF.OfficeAppContext=function(e,t,n,r,o,i,a,s,l,c,d,u,f,p,g,m,S,O,h,D,v,A,y,F,C,M,E){this._id=e,this._appName=t,this._appVersion=n,this._appUILocale=r,this._dataLocale=o,this._docUrl=i,this._clientMode=a,this._settings=s,this._reason=l,this._osfControlType=c,this._eToken=d,this._correlationId=u,this._appInstanceId=f,this._touchEnabled=p,this._commerceAllowed=g,this._appMinorVersion=m,this._requirementMatrix=S,this._hostCustomMessage=O,this._hostFullVersion=h,this._isDialog=!1,this._clientWindowHeight=D,this._clientWindowWidth=v,this._addinName=A,this._appDomains=y,this._dialogRequirementMatrix=F,this._featureGates=C,this._officeTheme=M,this._initialDisplayMode=E,this.get_id=function(){return this._id},this.get_appName=function(){return this._appName},this.get_appVersion=function(){return this._appVersion},this.get_appUILocale=function(){return this._appUILocale},this.get_dataLocale=function(){return this._dataLocale},this.get_docUrl=function(){return this._docUrl},this.get_clientMode=function(){return this._clientMode},this.get_bindings=function(){return this._bindings},this.get_settings=function(){return this._settings},this.get_reason=function(){return this._reason},this.get_osfControlType=function(){return this._osfControlType},this.get_eToken=function(){return this._eToken},this.get_correlationId=function(){return this._correlationId},this.get_appInstanceId=function(){return this._appInstanceId},this.get_touchEnabled=function(){return this._touchEnabled},this.get_commerceAllowed=function(){return this._commerceAllowed},this.get_appMinorVersion=function(){return this._appMinorVersion},this.get_requirementMatrix=function(){return this._requirementMatrix},this.get_dialogRequirementMatrix=function(){return this._dialogRequirementMatrix},this.get_hostCustomMessage=function(){return this._hostCustomMessage},this.get_hostFullVersion=function(){return this._hostFullVersion},this.get_isDialog=function(){return this._isDialog},this.get_clientWindowHeight=function(){return this._clientWindowHeight},this.get_clientWindowWidth=function(){return this._clientWindowWidth},this.get_addinName=function(){return this._addinName},this.get_appDomains=function(){return this._appDomains},this.get_featureGates=function(){return this._featureGates},this.get_officeTheme=function(){return this._officeTheme},this.get_initialDisplayMode=function(){return this._initialDisplayMode?this._initialDisplayMode:0}},OSF.OsfControlType={DocumentLevel:0,ContainerLevel:1},OSF.ClientMode={ReadOnly:0,ReadWrite:1},OSF.OUtil.setNamespace("Microsoft",window),OSF.OUtil.setNamespace("Office",Microsoft),OSF.OUtil.setNamespace("Client",Microsoft.Office),OSF.OUtil.setNamespace("WebExtension",Microsoft.Office),Microsoft.Office.WebExtension.InitializationReason={Inserted:"inserted",DocumentOpened:"documentOpened",ControlActivation:"controlActivation"},Microsoft.Office.WebExtension.ValueFormat={Unformatted:"unformatted",Formatted:"formatted"},Microsoft.Office.WebExtension.FilterType={All:"all"},Microsoft.Office.WebExtension.Parameters={BindingType:"bindingType",CoercionType:"coercionType",ValueFormat:"valueFormat",FilterType:"filterType",Columns:"columns",SampleData:"sampleData",GoToType:"goToType",SelectionMode:"selectionMode",Id:"id",PromptText:"promptText",ItemName:"itemName",FailOnCollision:"failOnCollision",StartRow:"startRow",StartColumn:"startColumn",RowCount:"rowCount",ColumnCount:"columnCount",Callback:"callback",AsyncContext:"asyncContext",Data:"data",Rows:"rows",OverwriteIfStale:"overwriteIfStale",FileType:"fileType",EventType:"eventType",Handler:"handler",SliceSize:"sliceSize",SliceIndex:"sliceIndex",ActiveView:"activeView",Status:"status",PlatformType:"platformType",HostType:"hostType",ForceConsent:"forceConsent",ForceAddAccount:"forceAddAccount",AuthChallenge:"authChallenge",AllowConsentPrompt:"allowConsentPrompt",ForMSGraphAccess:"forMSGraphAccess",AllowSignInPrompt:"allowSignInPrompt",JsonPayload:"jsonPayload",EnableNewHosts:"enableNewHosts",AccountTypeFilter:"accountTypeFilter",AddinTrustId:"addinTrustId",Reserved:"reserved",Tcid:"tcid",Xml:"xml",Namespace:"namespace",Prefix:"prefix",XPath:"xPath",Text:"text",ImageLeft:"imageLeft",ImageTop:"imageTop",ImageWidth:"imageWidth",ImageHeight:"imageHeight",TaskId:"taskId",FieldId:"fieldId",FieldValue:"fieldValue",ServerUrl:"serverUrl",ListName:"listName",ResourceId:"resourceId",ViewType:"viewType",ViewName:"viewName",GetRawValue:"getRawValue",CellFormat:"cellFormat",TableOptions:"tableOptions",TaskIndex:"taskIndex",ResourceIndex:"resourceIndex",CustomFieldId:"customFieldId",Url:"url",MessageHandler:"messageHandler",Width:"width",Height:"height",RequireHTTPs:"requireHTTPS",MessageToParent:"messageToParent",DisplayInIframe:"displayInIframe",MessageContent:"messageContent",HideTitle:"hideTitle",UseDeviceIndependentPixels:"useDeviceIndependentPixels",PromptBeforeOpen:"promptBeforeOpen",EnforceAppDomain:"enforceAppDomain",UrlNoHostInfo:"urlNoHostInfo",TargetOrigin:"targetOrigin",AppCommandInvocationCompletedData:"appCommandInvocationCompletedData",Base64:"base64",FormId:"formId"},OSF.OUtil.setNamespace("DDA",OSF),OSF.DDA.DocumentMode={ReadOnly:1,ReadWrite:0},OSF.DDA.PropertyDescriptors={AsyncResultStatus:"AsyncResultStatus"},OSF.DDA.EventDescriptors={},OSF.DDA.ListDescriptors={},OSF.DDA.UI={},OSF.DDA.getXdmEventName=function(e,t){return t==Microsoft.Office.WebExtension.EventType.BindingSelectionChanged||t==Microsoft.Office.WebExtension.EventType.BindingDataChanged||t==Microsoft.Office.WebExtension.EventType.DataNodeDeleted||t==Microsoft.Office.WebExtension.EventType.DataNodeInserted||t==Microsoft.Office.WebExtension.EventType.DataNodeReplaced?e+"_"+t:t},OSF.DDA.MethodDispId={dispidMethodMin:64,dispidGetSelectedDataMethod:64,dispidSetSelectedDataMethod:65,dispidAddBindingFromSelectionMethod:66,dispidAddBindingFromPromptMethod:67,dispidGetBindingMethod:68,dispidReleaseBindingMethod:69,dispidGetBindingDataMethod:70,dispidSetBindingDataMethod:71,dispidAddRowsMethod:72,dispidClearAllRowsMethod:73,dispidGetAllBindingsMethod:74,dispidLoadSettingsMethod:75,dispidSaveSettingsMethod:76,dispidGetDocumentCopyMethod:77,dispidAddBindingFromNamedItemMethod:78,dispidAddColumnsMethod:79,dispidGetDocumentCopyChunkMethod:80,dispidReleaseDocumentCopyMethod:81,dispidNavigateToMethod:82,dispidGetActiveViewMethod:83,dispidGetDocumentThemeMethod:84,dispidGetOfficeThemeMethod:85,dispidGetFilePropertiesMethod:86,dispidClearFormatsMethod:87,dispidSetTableOptionsMethod:88,dispidSetFormatsMethod:89,dispidExecuteRichApiRequestMethod:93,dispidAppCommandInvocationCompletedMethod:94,dispidCloseContainerMethod:97,dispidGetAccessTokenMethod:98,dispidGetAuthContextMethod:99,dispidOpenBrowserWindow:102,dispidCreateDocumentMethod:105,dispidInsertFormMethod:106,dispidDisplayRibbonCalloutAsyncMethod:109,dispidGetSelectedTaskMethod:110,dispidGetSelectedResourceMethod:111,dispidGetTaskMethod:112,dispidGetResourceFieldMethod:113,dispidGetWSSUrlMethod:114,dispidGetTaskFieldMethod:115,dispidGetProjectFieldMethod:116,dispidGetSelectedViewMethod:117,dispidGetTaskByIndexMethod:118,dispidGetResourceByIndexMethod:119,dispidSetTaskFieldMethod:120,dispidSetResourceFieldMethod:121,dispidGetMaxTaskIndexMethod:122,dispidGetMaxResourceIndexMethod:123,dispidCreateTaskMethod:124,dispidAddDataPartMethod:128,dispidGetDataPartByIdMethod:129,dispidGetDataPartsByNamespaceMethod:130,dispidGetDataPartXmlMethod:131,dispidGetDataPartNodesMethod:132,dispidDeleteDataPartMethod:133,dispidGetDataNodeValueMethod:134,dispidGetDataNodeXmlMethod:135,dispidGetDataNodesMethod:136,dispidSetDataNodeValueMethod:137,dispidSetDataNodeXmlMethod:138,dispidAddDataNamespaceMethod:139,dispidGetDataUriByPrefixMethod:140,dispidGetDataPrefixByUriMethod:141,dispidGetDataNodeTextMethod:142,dispidSetDataNodeTextMethod:143,dispidMessageParentMethod:144,dispidSendMessageMethod:145,dispidExecuteFeature:146,dispidQueryFeature:147,dispidMethodMax:147},OSF.DDA.EventDispId={dispidEventMin:0,dispidInitializeEvent:0,dispidSettingsChangedEvent:1,dispidDocumentSelectionChangedEvent:2,dispidBindingSelectionChangedEvent:3,dispidBindingDataChangedEvent:4,dispidDocumentOpenEvent:5,dispidDocumentCloseEvent:6,dispidActiveViewChangedEvent:7,dispidDocumentThemeChangedEvent:8,dispidOfficeThemeChangedEvent:9,dispidDialogMessageReceivedEvent:10,dispidDialogNotificationShownInAddinEvent:11,dispidDialogParentMessageReceivedEvent:12,dispidObjectDeletedEvent:13,dispidObjectSelectionChangedEvent:14,dispidObjectDataChangedEvent:15,dispidContentControlAddedEvent:16,dispidActivationStatusChangedEvent:32,dispidRichApiMessageEvent:33,dispidAppCommandInvokedEvent:39,dispidOlkItemSelectedChangedEvent:46,dispidOlkRecipientsChangedEvent:47,dispidOlkAppointmentTimeChangedEvent:48,dispidOlkRecurrenceChangedEvent:49,dispidOlkAttachmentsChangedEvent:50,dispidOlkEnhancedLocationsChangedEvent:51,dispidOlkInfobarClickedEvent:52,dispidTaskSelectionChangedEvent:56,dispidResourceSelectionChangedEvent:57,dispidViewSelectionChangedEvent:58,dispidDataNodeAddedEvent:60,dispidDataNodeReplacedEvent:61,dispidDataNodeDeletedEvent:62,dispidEventMax:63},OSF.DDA.ErrorCodeManager=function(){var e={};return{getErrorArgs:function(t){var n=e[t];return n?(n.name||(n.name=e[this.errorCodes.ooeInternalError].name),n.message||(n.message=e[this.errorCodes.ooeInternalError].message)):n=e[this.errorCodes.ooeInternalError],n},addErrorMessage:function(t,n){e[t]=n},errorCodes:{ooeSuccess:0,ooeChunkResult:1,ooeCoercionTypeNotSupported:1e3,ooeGetSelectionNotMatchDataType:1001,ooeCoercionTypeNotMatchBinding:1002,ooeInvalidGetRowColumnCounts:1003,ooeSelectionNotSupportCoercionType:1004,ooeInvalidGetStartRowColumn:1005,ooeNonUniformPartialGetNotSupported:1006,ooeGetDataIsTooLarge:1008,ooeFileTypeNotSupported:1009,ooeGetDataParametersConflict:1010,ooeInvalidGetColumns:1011,ooeInvalidGetRows:1012,ooeInvalidReadForBlankRow:1013,ooeUnsupportedDataObject:2e3,ooeCannotWriteToSelection:2001,ooeDataNotMatchSelection:2002,ooeOverwriteWorksheetData:2003,ooeDataNotMatchBindingSize:2004,ooeInvalidSetStartRowColumn:2005,ooeInvalidDataFormat:2006,ooeDataNotMatchCoercionType:2007,ooeDataNotMatchBindingType:2008,ooeSetDataIsTooLarge:2009,ooeNonUniformPartialSetNotSupported:2010,ooeInvalidSetColumns:2011,ooeInvalidSetRows:2012,ooeSetDataParametersConflict:2013,ooeCellDataAmountBeyondLimits:2014,ooeSelectionCannotBound:3e3,ooeBindingNotExist:3002,ooeBindingToMultipleSelection:3003,ooeInvalidSelectionForBindingType:3004,ooeOperationNotSupportedOnThisBindingType:3005,ooeNamedItemNotFound:3006,ooeMultipleNamedItemFound:3007,ooeInvalidNamedItemForBindingType:3008,ooeUnknownBindingType:3009,ooeOperationNotSupportedOnMatrixData:3010,ooeInvalidColumnsForBinding:3011,ooeSettingNameNotExist:4e3,ooeSettingsCannotSave:4001,ooeSettingsAreStale:4002,ooeOperationNotSupported:5e3,ooeInternalError:5001,ooeDocumentReadOnly:5002,ooeEventHandlerNotExist:5003,ooeInvalidApiCallInContext:5004,ooeShuttingDown:5005,ooeUnsupportedEnumeration:5007,ooeIndexOutOfRange:5008,ooeBrowserAPINotSupported:5009,ooeInvalidParam:5010,ooeRequestTimeout:5011,ooeInvalidOrTimedOutSession:5012,ooeInvalidApiArguments:5013,ooeOperationCancelled:5014,ooeWorkbookHidden:5015,ooeWriteNotSupportedWhenModalDialogOpen:5016,ooeTooManyIncompleteRequests:5100,ooeRequestTokenUnavailable:5101,ooeActivityLimitReached:5102,ooeRequestPayloadSizeLimitExceeded:5103,ooeResponsePayloadSizeLimitExceeded:5104,ooeCustomXmlNodeNotFound:6e3,ooeCustomXmlError:6100,ooeCustomXmlExceedQuota:6101,ooeCustomXmlOutOfDate:6102,ooeNoCapability:7e3,ooeCannotNavTo:7001,ooeSpecifiedIdNotExist:7002,ooeNavOutOfBound:7004,ooeElementMissing:8e3,ooeProtectedError:8001,ooeInvalidCellsValue:8010,ooeInvalidTableOptionValue:8011,ooeInvalidFormatValue:8012,ooeRowIndexOutOfRange:8020,ooeColIndexOutOfRange:8021,ooeFormatValueOutOfRange:8022,ooeCellFormatAmountBeyondLimits:8023,ooeMemoryFileLimit:11e3,ooeNetworkProblemRetrieveFile:11001,ooeInvalidSliceSize:11002,ooeInvalidCallback:11101,ooeInvalidWidth:12e3,ooeInvalidHeight:12001,ooeNavigationError:12002,ooeInvalidScheme:12003,ooeAppDomains:12004,ooeRequireHTTPS:12005,ooeWebDialogClosed:12006,ooeDialogAlreadyOpened:12007,ooeEndUserAllow:12008,ooeEndUserIgnore:12009,ooeNotUILessDialog:12010,ooeCrossZone:12011,ooeModalDialogOpen:12012,ooeDocumentIsInactive:12013,ooeDialogParentIsMinimized:12014,ooeNotSSOAgave:13e3,ooeSSOUserNotSignedIn:13001,ooeSSOUserAborted:13002,ooeSSOUnsupportedUserIdentity:13003,ooeSSOInvalidResourceUrl:13004,ooeSSOInvalidGrant:13005,ooeSSOClientError:13006,ooeSSOServerError:13007,ooeAddinIsAlreadyRequestingToken:13008,ooeSSOUserConsentNotSupportedByCurrentAddinCategory:13009,ooeSSOConnectionLost:13010,ooeResourceNotAllowed:13011,ooeSSOUnsupportedPlatform:13012,ooeSSOCallThrottled:13013,ooeAccessDenied:13990,ooeGeneralException:13991},initializeErrorMessages:function(t){e[OSF.DDA.ErrorCodeManager.errorCodes.ooeCoercionTypeNotSupported]={name:t.L_InvalidCoercion,message:t.L_CoercionTypeNotSupported},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeGetSelectionNotMatchDataType]={name:t.L_DataReadError,message:t.L_GetSelectionNotSupported},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeCoercionTypeNotMatchBinding]={name:t.L_InvalidCoercion,message:t.L_CoercionTypeNotMatchBinding},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidGetRowColumnCounts]={name:t.L_DataReadError,message:t.L_InvalidGetRowColumnCounts},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSelectionNotSupportCoercionType]={name:t.L_DataReadError,message:t.L_SelectionNotSupportCoercionType},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidGetStartRowColumn]={name:t.L_DataReadError,message:t.L_InvalidGetStartRowColumn},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeNonUniformPartialGetNotSupported]={name:t.L_DataReadError,message:t.L_NonUniformPartialGetNotSupported},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeGetDataIsTooLarge]={name:t.L_DataReadError,message:t.L_GetDataIsTooLarge},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeFileTypeNotSupported]={name:t.L_DataReadError,message:t.L_FileTypeNotSupported},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeGetDataParametersConflict]={name:t.L_DataReadError,message:t.L_GetDataParametersConflict},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidGetColumns]={name:t.L_DataReadError,message:t.L_InvalidGetColumns},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidGetRows]={name:t.L_DataReadError,message:t.L_InvalidGetRows},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidReadForBlankRow]={name:t.L_DataReadError,message:t.L_InvalidReadForBlankRow},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeUnsupportedDataObject]={name:t.L_DataWriteError,message:t.L_UnsupportedDataObject},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeCannotWriteToSelection]={name:t.L_DataWriteError,message:t.L_CannotWriteToSelection},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeDataNotMatchSelection]={name:t.L_DataWriteError,message:t.L_DataNotMatchSelection},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeOverwriteWorksheetData]={name:t.L_DataWriteError,message:t.L_OverwriteWorksheetData},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeDataNotMatchBindingSize]={name:t.L_DataWriteError,message:t.L_DataNotMatchBindingSize},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidSetStartRowColumn]={name:t.L_DataWriteError,message:t.L_InvalidSetStartRowColumn},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidDataFormat]={name:t.L_InvalidFormat,message:t.L_InvalidDataFormat},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeDataNotMatchCoercionType]={name:t.L_InvalidDataObject,message:t.L_DataNotMatchCoercionType},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeDataNotMatchBindingType]={name:t.L_InvalidDataObject,message:t.L_DataNotMatchBindingType},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSetDataIsTooLarge]={name:t.L_DataWriteError,message:t.L_SetDataIsTooLarge},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeNonUniformPartialSetNotSupported]={name:t.L_DataWriteError,message:t.L_NonUniformPartialSetNotSupported},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidSetColumns]={name:t.L_DataWriteError,message:t.L_InvalidSetColumns},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidSetRows]={name:t.L_DataWriteError,message:t.L_InvalidSetRows},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSetDataParametersConflict]={name:t.L_DataWriteError,message:t.L_SetDataParametersConflict},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSelectionCannotBound]={name:t.L_BindingCreationError,message:t.L_SelectionCannotBound},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeBindingNotExist]={name:t.L_InvalidBindingError,message:t.L_BindingNotExist},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeBindingToMultipleSelection]={name:t.L_BindingCreationError,message:t.L_BindingToMultipleSelection},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidSelectionForBindingType]={name:t.L_BindingCreationError,message:t.L_InvalidSelectionForBindingType},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeOperationNotSupportedOnThisBindingType]={name:t.L_InvalidBindingOperation,message:t.L_OperationNotSupportedOnThisBindingType},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeNamedItemNotFound]={name:t.L_BindingCreationError,message:t.L_NamedItemNotFound},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeMultipleNamedItemFound]={name:t.L_BindingCreationError,message:t.L_MultipleNamedItemFound},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidNamedItemForBindingType]={name:t.L_BindingCreationError,message:t.L_InvalidNamedItemForBindingType},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeUnknownBindingType]={name:t.L_InvalidBinding,message:t.L_UnknownBindingType},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeOperationNotSupportedOnMatrixData]={name:t.L_InvalidBindingOperation,message:t.L_OperationNotSupportedOnMatrixData},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidColumnsForBinding]={name:t.L_InvalidBinding,message:t.L_InvalidColumnsForBinding},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSettingNameNotExist]={name:t.L_ReadSettingsError,message:t.L_SettingNameNotExist},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSettingsCannotSave]={name:t.L_SaveSettingsError,message:t.L_SettingsCannotSave},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSettingsAreStale]={name:t.L_SettingsStaleError,message:t.L_SettingsAreStale},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeOperationNotSupported]={name:t.L_HostError,message:t.L_OperationNotSupported},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError]={name:t.L_InternalError,message:t.L_InternalErrorDescription},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeDocumentReadOnly]={name:t.L_PermissionDenied,message:t.L_DocumentReadOnly},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeEventHandlerNotExist]={name:t.L_EventRegistrationError,message:t.L_EventHandlerNotExist},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidApiCallInContext]={name:t.L_InvalidAPICall,message:t.L_InvalidApiCallInContext},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeShuttingDown]={name:t.L_ShuttingDown,message:t.L_ShuttingDown},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeUnsupportedEnumeration]={name:t.L_UnsupportedEnumeration,message:t.L_UnsupportedEnumerationMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeIndexOutOfRange]={name:t.L_IndexOutOfRange,message:t.L_IndexOutOfRange},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeBrowserAPINotSupported]={name:t.L_APINotSupported,message:t.L_BrowserAPINotSupported},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeRequestTimeout]={name:t.L_APICallFailed,message:t.L_RequestTimeout},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidOrTimedOutSession]={name:t.L_InvalidOrTimedOutSession,message:t.L_InvalidOrTimedOutSessionMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidApiArguments]={name:t.L_APICallFailed,message:t.L_InvalidApiArgumentsMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeWorkbookHidden]={name:t.L_APICallFailed,message:t.L_WorkbookHiddenMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeWriteNotSupportedWhenModalDialogOpen]={name:t.L_APICallFailed,message:t.L_WriteNotSupportedWhenModalDialogOpen},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeTooManyIncompleteRequests]={name:t.L_APICallFailed,message:t.L_TooManyIncompleteRequests},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeRequestTokenUnavailable]={name:t.L_APICallFailed,message:t.L_RequestTokenUnavailable},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeActivityLimitReached]={name:t.L_APICallFailed,message:t.L_ActivityLimitReached},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeRequestPayloadSizeLimitExceeded]={name:t.L_APICallFailed,message:t.L_RequestPayloadSizeLimitExceededMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeResponsePayloadSizeLimitExceeded]={name:t.L_APICallFailed,message:t.L_ResponsePayloadSizeLimitExceededMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeCustomXmlNodeNotFound]={name:t.L_InvalidNode,message:t.L_CustomXmlNodeNotFound},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeCustomXmlError]={name:t.L_CustomXmlError,message:t.L_CustomXmlError},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeCustomXmlExceedQuota]={name:t.L_CustomXmlExceedQuotaName,message:t.L_CustomXmlExceedQuotaMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeCustomXmlOutOfDate]={name:t.L_CustomXmlOutOfDateName,message:t.L_CustomXmlOutOfDateMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeNoCapability]={name:t.L_PermissionDenied,message:t.L_NoCapability},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeCannotNavTo]={name:t.L_CannotNavigateTo,message:t.L_CannotNavigateTo},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSpecifiedIdNotExist]={name:t.L_SpecifiedIdNotExist,message:t.L_SpecifiedIdNotExist},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeNavOutOfBound]={name:t.L_NavOutOfBound,message:t.L_NavOutOfBound},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeCellDataAmountBeyondLimits]={name:t.L_DataWriteReminder,message:t.L_CellDataAmountBeyondLimits},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeElementMissing]={name:t.L_MissingParameter,message:t.L_ElementMissing},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeProtectedError]={name:t.L_PermissionDenied,message:t.L_NoCapability},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidCellsValue]={name:t.L_InvalidValue,message:t.L_InvalidCellsValue},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidTableOptionValue]={name:t.L_InvalidValue,message:t.L_InvalidTableOptionValue},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidFormatValue]={name:t.L_InvalidValue,message:t.L_InvalidFormatValue},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeRowIndexOutOfRange]={name:t.L_OutOfRange,message:t.L_RowIndexOutOfRange},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeColIndexOutOfRange]={name:t.L_OutOfRange,message:t.L_ColIndexOutOfRange},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeFormatValueOutOfRange]={name:t.L_OutOfRange,message:t.L_FormatValueOutOfRange},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeCellFormatAmountBeyondLimits]={name:t.L_FormattingReminder,message:t.L_CellFormatAmountBeyondLimits},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeMemoryFileLimit]={name:t.L_MemoryLimit,message:t.L_CloseFileBeforeRetrieve},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeNetworkProblemRetrieveFile]={name:t.L_NetworkProblem,message:t.L_NetworkProblemRetrieveFile},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidSliceSize]={name:t.L_InvalidValue,message:t.L_SliceSizeNotSupported},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeDialogAlreadyOpened]={name:t.L_DisplayDialogError,message:t.L_DialogAlreadyOpened},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidWidth]={name:t.L_IndexOutOfRange,message:t.L_IndexOutOfRange},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidHeight]={name:t.L_IndexOutOfRange,message:t.L_IndexOutOfRange},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeNavigationError]={name:t.L_DisplayDialogError,message:t.L_NetworkProblem},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidScheme]={name:t.L_DialogNavigateError,message:t.L_DialogInvalidScheme},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeAppDomains]={name:t.L_DisplayDialogError,message:t.L_DialogAddressNotTrusted},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeRequireHTTPS]={name:t.L_DisplayDialogError,message:t.L_DialogRequireHTTPS},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeEndUserIgnore]={name:t.L_DisplayDialogError,message:t.L_UserClickIgnore},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeCrossZone]={name:t.L_DisplayDialogError,message:t.L_NewWindowCrossZoneErrorString},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeModalDialogOpen]={name:t.L_DisplayDialogError,message:t.L_ModalDialogOpen},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeDocumentIsInactive]={name:t.L_DisplayDialogError,message:t.L_DocumentIsInactive},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeDialogParentIsMinimized]={name:t.L_DisplayDialogError,message:t.L_DialogParentIsMinimized},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeNotSSOAgave]={name:t.L_APINotSupported,message:t.L_InvalidSSOAddinMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSSOUserNotSignedIn]={name:t.L_UserNotSignedIn,message:t.L_UserNotSignedIn},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSSOUserAborted]={name:t.L_UserAborted,message:t.L_UserAbortedMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSSOUnsupportedUserIdentity]={name:t.L_UnsupportedUserIdentity,message:t.L_UnsupportedUserIdentityMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSSOInvalidResourceUrl]={name:t.L_InvalidResourceUrl,message:t.L_InvalidResourceUrlMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSSOInvalidGrant]={name:t.L_InvalidGrant,message:t.L_InvalidGrantMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSSOClientError]={name:t.L_SSOClientError,message:t.L_SSOClientErrorMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSSOServerError]={name:t.L_SSOServerError,message:t.L_SSOServerErrorMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeAddinIsAlreadyRequestingToken]={name:t.L_AddinIsAlreadyRequestingToken,message:t.L_AddinIsAlreadyRequestingTokenMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSSOUserConsentNotSupportedByCurrentAddinCategory]={name:t.L_SSOUserConsentNotSupportedByCurrentAddinCategory,message:t.L_SSOUserConsentNotSupportedByCurrentAddinCategoryMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSSOConnectionLost]={name:t.L_SSOConnectionLostError,message:t.L_SSOConnectionLostErrorMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSSOUnsupportedPlatform]={name:t.L_APINotSupported,message:t.L_SSOUnsupportedPlatform},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSSOCallThrottled]={name:t.L_APICallFailed,message:t.L_RequestTokenUnavailable},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeOperationCancelled]={name:t.L_OperationCancelledError,message:t.L_OperationCancelledErrorMessage}}}}(),function(e){!function(e){var t=function(){};e.RequirementVersion=t;var n=function(e){this.isSetSupported=function(e,t){if(null==e)return!1;null==t&&(t=0);var n=this._setMap._sets;if(n.hasOwnProperty(e.toLowerCase())){var r=n[e.toLowerCase()];try{var o=this._getVersion(r);t+="";var i=this._getVersion(t);if(o.major>0&&o.major>i.major)return!0;if(o.major>0&&o.minor>=0&&o.major==i.major&&o.minor>=i.minor)return!0}catch(e){return!1}}return!1},this._getVersion=function(e){var t=(e+="").split("."),n=0,r=0;if(t.length<2&&isNaN(Number(e)))throw"version format incorrect";if(n=Number(t[0]),t.length>=2&&(r=Number(t[1])),isNaN(n)||isNaN(r))throw"version format incorrect";return{minor:r,major:n}},this._setMap=e,this.isSetSupported=this.isSetSupported.bind(this)};e.RequirementMatrix=n;var r=function(e){this._addSetMap=function(e){for(var t in e)this._sets[t]=e[t]},this._sets=e};e.DefaultSetRequirement=r;var o=function(e){function t(){return e.call(this,{dialogapi:1.1})||this}return __extends(t,e),t}(r);e.DefaultRequiredDialogSetRequirement=o;var i=function(e){function t(){return e.call(this,{dialogorigin:1.1})||this}return __extends(t,e),t}(r);e.DefaultOptionalDialogSetRequirement=i;var a=function(e){function t(){return e.call(this,{bindingevents:1.1,documentevents:1.1,excelapi:1.1,matrixbindings:1.1,matrixcoercion:1.1,selection:1.1,settings:1.1,tablebindings:1.1,tablecoercion:1.1,textbindings:1.1,textcoercion:1.1})||this}return __extends(t,e),t}(r);e.ExcelClientDefaultSetRequirement=a;var s=function(e){function t(){var t=e.call(this)||this;return t._addSetMap({imagecoercion:1.1}),t}return __extends(t,e),t}(a);e.ExcelClientV1DefaultSetRequirement=s;var l=function(e){function t(){return e.call(this,{mailbox:1.3})||this}return __extends(t,e),t}(r);e.OutlookClientDefaultSetRequirement=l;var c=function(e){function t(){return e.call(this,{bindingevents:1.1,compressedfile:1.1,customxmlparts:1.1,documentevents:1.1,file:1.1,htmlcoercion:1.1,matrixbindings:1.1,matrixcoercion:1.1,ooxmlcoercion:1.1,pdffile:1.1,selection:1.1,settings:1.1,tablebindings:1.1,tablecoercion:1.1,textbindings:1.1,textcoercion:1.1,textfile:1.1,wordapi:1.1})||this}return __extends(t,e),t}(r);e.WordClientDefaultSetRequirement=c;var d=function(e){function t(){var t=e.call(this)||this;return t._addSetMap({customxmlparts:1.2,wordapi:1.2,imagecoercion:1.1}),t}return __extends(t,e),t}(c);e.WordClientV1DefaultSetRequirement=d;var u=function(e){function t(){return e.call(this,{activeview:1.1,compressedfile:1.1,documentevents:1.1,file:1.1,pdffile:1.1,selection:1.1,settings:1.1,textcoercion:1.1})||this}return __extends(t,e),t}(r);e.PowerpointClientDefaultSetRequirement=u;var f=function(e){function t(){var t=e.call(this)||this;return t._addSetMap({imagecoercion:1.1}),t}return __extends(t,e),t}(u);e.PowerpointClientV1DefaultSetRequirement=f;var p=function(e){function t(){return e.call(this,{selection:1.1,textcoercion:1.1})||this}return __extends(t,e),t}(r);e.ProjectClientDefaultSetRequirement=p;var g=function(e){function t(){return e.call(this,{bindingevents:1.1,documentevents:1.1,matrixbindings:1.1,matrixcoercion:1.1,selection:1.1,settings:1.1,tablebindings:1.1,tablecoercion:1.1,textbindings:1.1,textcoercion:1.1,file:1.1})||this}return __extends(t,e),t}(r);e.ExcelWebDefaultSetRequirement=g;var m=function(e){function t(){return e.call(this,{compressedfile:1.1,documentevents:1.1,file:1.1,imagecoercion:1.1,matrixcoercion:1.1,ooxmlcoercion:1.1,pdffile:1.1,selection:1.1,settings:1.1,tablecoercion:1.1,textcoercion:1.1,textfile:1.1})||this}return __extends(t,e),t}(r);e.WordWebDefaultSetRequirement=m;var S=function(e){function t(){return e.call(this,{activeview:1.1,settings:1.1})||this}return __extends(t,e),t}(r);e.PowerpointWebDefaultSetRequirement=S;var O=function(e){function t(){return e.call(this,{mailbox:1.3})||this}return __extends(t,e),t}(r);e.OutlookWebDefaultSetRequirement=O;var h=function(e){function t(){return e.call(this,{activeview:1.1,documentevents:1.1,selection:1.1,settings:1.1,textcoercion:1.1})||this}return __extends(t,e),t}(r);e.SwayWebDefaultSetRequirement=h;var D=function(e){function t(){return e.call(this,{bindingevents:1.1,partialtablebindings:1.1,settings:1.1,tablebindings:1.1,tablecoercion:1.1})||this}return __extends(t,e),t}(r);e.AccessWebDefaultSetRequirement=D;var v=function(e){function t(){return e.call(this,{bindingevents:1.1,documentevents:1.1,matrixbindings:1.1,matrixcoercion:1.1,selection:1.1,settings:1.1,tablebindings:1.1,tablecoercion:1.1,textbindings:1.1,textcoercion:1.1})||this}return __extends(t,e),t}(r);e.ExcelIOSDefaultSetRequirement=v;var A=function(e){function t(){return e.call(this,{bindingevents:1.1,compressedfile:1.1,customxmlparts:1.1,documentevents:1.1,file:1.1,htmlcoercion:1.1,matrixbindings:1.1,matrixcoercion:1.1,ooxmlcoercion:1.1,pdffile:1.1,selection:1.1,settings:1.1,tablebindings:1.1,tablecoercion:1.1,textbindings:1.1,textcoercion:1.1,textfile:1.1})||this}return __extends(t,e),t}(r);e.WordIOSDefaultSetRequirement=A;var y=function(e){function t(){var t=e.call(this)||this;return t._addSetMap({customxmlparts:1.2,wordapi:1.2}),t}return __extends(t,e),t}(A);e.WordIOSV1DefaultSetRequirement=y;var F=function(e){function t(){return e.call(this,{activeview:1.1,compressedfile:1.1,documentevents:1.1,file:1.1,pdffile:1.1,selection:1.1,settings:1.1,textcoercion:1.1})||this}return __extends(t,e),t}(r);e.PowerpointIOSDefaultSetRequirement=F;var C=function(e){function t(){return e.call(this,{mailbox:1.1})||this}return __extends(t,e),t}(r);e.OutlookIOSDefaultSetRequirement=C;var M=function(){function e(){}return e.initializeOsfDda=function(){OSF.OUtil.setNamespace("Requirement",OSF.DDA)},e.getDefaultRequirementMatrix=function(t){this.initializeDefaultSetMatrix();var o=void 0,i=t.get_requirementMatrix();if(null!=i&&i.length>0&&"undefined"!=typeof JSON){var a=JSON.parse(t.get_requirementMatrix().toLowerCase());o=new n(new r(a))}else{var s=e.getClientFullVersionString(t);o=null!=e.DefaultSetArrayMatrix&&null!=e.DefaultSetArrayMatrix[s]?new n(e.DefaultSetArrayMatrix[s]):new n(new r({}))}return o},e.getDefaultDialogRequirementMatrix=function(e){var t=void 0,a=e.get_dialogRequirementMatrix();if(null!=a&&a.length>0&&"undefined"!=typeof JSON){var s=JSON.parse(e.get_requirementMatrix().toLowerCase());t=new r(s)}else{t=new o;var l=e.get_requirementMatrix();if(null!=l&&l.length>0&&"undefined"!=typeof JSON){s=JSON.parse(l.toLowerCase());for(var c in t._sets)s.hasOwnProperty(c)&&(t._sets[c]=s[c]);var d=new i;for(var c in d._sets)s.hasOwnProperty(c)&&(t._sets[c]=s[c])}}return new n(t)},e.getClientFullVersionString=function(e){var t=e.get_appMinorVersion(),n="",r="",o=e.get_appName();return(1024==o||4096==o||8192==o||65536==o)&&1==e.get_appVersion()?r=4096==o&&t>=15?"16.00.01":"16.00":64==e.get_appName()?r=e.get_appVersion():(n=t<10?"0"+t:""+t,r=e.get_appVersion()+"."+n),e.get_appName()+"-"+r},e.initializeDefaultSetMatrix=function(){e.DefaultSetArrayMatrix[e.Excel_RCLIENT_1600]=new a,e.DefaultSetArrayMatrix[e.Word_RCLIENT_1600]=new c,e.DefaultSetArrayMatrix[e.PowerPoint_RCLIENT_1600]=new u,e.DefaultSetArrayMatrix[e.Excel_RCLIENT_1601]=new s,e.DefaultSetArrayMatrix[e.Word_RCLIENT_1601]=new d,e.DefaultSetArrayMatrix[e.PowerPoint_RCLIENT_1601]=new f,e.DefaultSetArrayMatrix[e.Outlook_RCLIENT_1600]=new l,e.DefaultSetArrayMatrix[e.Excel_WAC_1600]=new g,e.DefaultSetArrayMatrix[e.Word_WAC_1600]=new m,e.DefaultSetArrayMatrix[e.Outlook_WAC_1600]=new O,e.DefaultSetArrayMatrix[e.Outlook_WAC_1601]=new O,e.DefaultSetArrayMatrix[e.Project_RCLIENT_1600]=new p,e.DefaultSetArrayMatrix[e.Access_WAC_1600]=new D,e.DefaultSetArrayMatrix[e.PowerPoint_WAC_1600]=new S,e.DefaultSetArrayMatrix[e.Excel_IOS_1600]=new v,e.DefaultSetArrayMatrix[e.SWAY_WAC_1600]=new h,e.DefaultSetArrayMatrix[e.Word_IOS_1600]=new A,e.DefaultSetArrayMatrix[e.Word_IOS_16001]=new y,e.DefaultSetArrayMatrix[e.PowerPoint_IOS_1600]=new F,e.DefaultSetArrayMatrix[e.Outlook_IOS_1600]=new C},e.Excel_RCLIENT_1600="1-16.00",e.Excel_RCLIENT_1601="1-16.01",e.Word_RCLIENT_1600="2-16.00",e.Word_RCLIENT_1601="2-16.01",e.PowerPoint_RCLIENT_1600="4-16.00",e.PowerPoint_RCLIENT_1601="4-16.01",e.Outlook_RCLIENT_1600="8-16.00",e.Excel_WAC_1600="16-16.00",e.Word_WAC_1600="32-16.00",e.Outlook_WAC_1600="64-16.00",e.Outlook_WAC_1601="64-16.01",e.Project_RCLIENT_1600="128-16.00",e.Access_WAC_1600="256-16.00",e.PowerPoint_WAC_1600="512-16.00",e.Excel_IOS_1600="1024-16.00",e.SWAY_WAC_1600="2048-16.00",e.Word_IOS_1600="4096-16.00",e.Word_IOS_16001="4096-16.00.01",e.PowerPoint_IOS_1600="8192-16.00",e.Outlook_IOS_1600="65536-16.00",e.DefaultSetArrayMatrix={},e}();e.RequirementsMatrixFactory=M}(e.Requirement||(e.Requirement={}))}(OfficeExt||(OfficeExt={})),OfficeExt.Requirement.RequirementsMatrixFactory.initializeOsfDda(),Microsoft.Office.WebExtension.ApplicationMode={WebEditor:"webEditor",WebViewer:"webViewer",Client:"client"},Microsoft.Office.WebExtension.DocumentMode={ReadOnly:"readOnly",ReadWrite:"readWrite"},OSF.NamespaceManager=function(){var e,t=!1;return{enableShortcut:function(){t||(window.Office?e=window.Office:OSF.OUtil.setNamespace("Office",window),window.Office=Microsoft.Office.WebExtension,t=!0)},disableShortcut:function(){t&&(e?window.Office=e:OSF.OUtil.unsetNamespace("Office",window),t=!1)}}}(),OSF.NamespaceManager.enableShortcut(),Microsoft.Office.WebExtension.useShortNamespace=function(e){e?OSF.NamespaceManager.enableShortcut():OSF.NamespaceManager.disableShortcut()},Microsoft.Office.WebExtension.select=function(e,t){var n;if(e&&"string"==typeof e){var r=e.indexOf("#");if(-1!=r){var o=e.substring(0,r),i=e.substring(r+1);switch(o){case"binding":case"bindings":i&&(n=new OSF.DDA.BindingPromise(i))}}}if(n)return n.onFail=t,n;if(t){var a=typeof t;if("function"!=a)throw OSF.OUtil.formatString(Strings.OfficeOM.L_CallbackNotAFunction,a);var s={};s[Microsoft.Office.WebExtension.Parameters.Callback]=t,OSF.DDA.issueAsyncResult(s,OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidApiCallInContext,OSF.DDA.ErrorCodeManager.getErrorArgs(OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidApiCallInContext))}},OSF.DDA.Context=function(e,t,n,r,o){if(OSF.OUtil.defineEnumerableProperties(this,{contentLanguage:{value:e.get_dataLocale()},displayLanguage:{value:e.get_appUILocale()},touchEnabled:{value:e.get_touchEnabled()},commerceAllowed:{value:e.get_commerceAllowed()},host:{value:OfficeExt.HostName.Host.getInstance().getHost()},platform:{value:OfficeExt.HostName.Host.getInstance().getPlatform()},isDialog:{value:OSF._OfficeAppFactory.getHostInfo().isDialog},diagnostics:{value:OfficeExt.HostName.Host.getInstance().getDiagnostics(e.get_hostFullVersion())}}),n&&OSF.OUtil.defineEnumerableProperty(this,"license",{value:n}),e.ui&&OSF.OUtil.defineEnumerableProperty(this,"ui",{value:e.ui}),e.auth&&OSF.OUtil.defineEnumerableProperty(this,"auth",{value:e.auth}),e.webAuth&&OSF.OUtil.defineEnumerableProperty(this,"webAuth",{value:e.webAuth}),e.application&&OSF.OUtil.defineEnumerableProperty(this,"application",{value:e.application}),e.extensionLifeCycle&&OSF.OUtil.defineEnumerableProperty(this,"extensionLifeCycle",{value:e.extensionLifeCycle}),e.messaging&&OSF.OUtil.defineEnumerableProperty(this,"messaging",{value:e.messaging}),e.ui&&e.ui.taskPaneAction&&OSF.OUtil.defineEnumerableProperty(this,"taskPaneAction",{value:e.ui.taskPaneAction}),e.ui&&e.ui.ribbonGallery&&OSF.OUtil.defineEnumerableProperty(this,"ribbonGallery",{value:e.ui.ribbonGallery}),e.get_isDialog()){var i=OfficeExt.Requirement.RequirementsMatrixFactory.getDefaultDialogRequirementMatrix(e);OSF.OUtil.defineEnumerableProperty(this,"requirements",{value:i})}else{if(t&&OSF.OUtil.defineEnumerableProperty(this,"document",{value:t}),r){var a=r.displayName||"appOM";delete r.displayName,OSF.OUtil.defineEnumerableProperty(this,a,{value:r})}e.get_officeTheme()?OSF.OUtil.defineEnumerableProperty(this,"officeTheme",{get:function(){return e.get_officeTheme()}}):o&&OSF.OUtil.defineEnumerableProperty(this,"officeTheme",{get:function(){return o()}});i=OfficeExt.Requirement.RequirementsMatrixFactory.getDefaultRequirementMatrix(e);OSF.OUtil.defineEnumerableProperty(this,"requirements",{value:i})}},OSF.DDA.OutlookContext=function(e,t,n,r,o){OSF.DDA.OutlookContext.uber.constructor.call(this,e,null,n,r,o),t&&OSF.OUtil.defineEnumerableProperty(this,"roamingSettings",{value:t})},OSF.OUtil.extend(OSF.DDA.OutlookContext,OSF.DDA.Context),OSF.DDA.OutlookAppOm=function(e,t,n){},OSF.DDA.Application=function(e){},OSF.DDA.Document=function(e,t){var n;switch(e.get_clientMode()){case OSF.ClientMode.ReadOnly:n=Microsoft.Office.WebExtension.DocumentMode.ReadOnly;break;case OSF.ClientMode.ReadWrite:n=Microsoft.Office.WebExtension.DocumentMode.ReadWrite}t&&OSF.OUtil.defineEnumerableProperty(this,"settings",{value:t}),OSF.OUtil.defineMutableProperties(this,{mode:{value:n},url:{value:e.get_docUrl()}})},OSF.DDA.JsomDocument=function(e,t,n){OSF.DDA.JsomDocument.uber.constructor.call(this,e,n),t&&OSF.OUtil.defineEnumerableProperty(this,"bindings",{get:function(){return t}});var r=OSF.DDA.AsyncMethodNames;OSF.DDA.DispIdHost.addAsyncMethods(this,[r.GetSelectedDataAsync,r.SetSelectedDataAsync]),OSF.DDA.DispIdHost.addEventSupport(this,new OSF.EventDispatch([Microsoft.Office.WebExtension.EventType.DocumentSelectionChanged]))},OSF.OUtil.extend(OSF.DDA.JsomDocument,OSF.DDA.Document),OSF.OUtil.defineEnumerableProperty(Microsoft.Office.WebExtension,"context",{get:function(){var e;return OSF&&OSF._OfficeAppFactory&&(e=OSF._OfficeAppFactory.getContext()),e}}),OSF.DDA.License=function(e){OSF.OUtil.defineEnumerableProperty(this,"value",{value:e})},OSF.DDA.ApiMethodCall=function(e,t,n,r,o){var i=e.length,a=OSF.OUtil.delayExecutionAndCache((function(){return OSF.OUtil.formatString(Strings.OfficeOM.L_InvalidParameters,o)}));this.verifyArguments=function(e,t){for(var n in e){var r=e[n],o=t[n];if(r.enum)switch(typeof o){case"string":if(OSF.OUtil.listContainsValue(r.enum,o))break;case"undefined":throw OSF.DDA.ErrorCodeManager.errorCodes.ooeUnsupportedEnumeration;default:throw a()}if(r.types&&!OSF.OUtil.listContainsValue(r.types,typeof o))throw a()}},this.extractRequiredArguments=function(t,n,r){if(t.length<i)throw OsfMsAjaxFactory.msAjaxError.parameterCount(Strings.OfficeOM.L_MissingRequiredArguments);var o,s=[];for(o=0;o<i;o++)s.push(t[o]);this.verifyArguments(e,s);var l={};for(o=0;o<i;o++){var c=e[o],d=s[o];if(c.verify)if(!c.verify(d,n,r))throw a();l[c.name]=d}return l},this.fillOptions=function(e,n,r,o){for(var i in e=e||{},t)if(!OSF.OUtil.listContainsKey(e,i)){var a=void 0,s=t[i];s.calculate&&n&&(a=s.calculate(n,r,o)),a||void 0===s.defaultValue||(a=s.defaultValue),e[i]=a}return e},this.constructCallArgs=function(e,t,o,i){var a={};for(var s in e)a[s]=e[s];for(var l in t)a[l]=t[l];for(var c in n)a[c]=n[c](o,i);return r&&(a=r(a,o,i)),a}},OSF.OUtil.setNamespace("AsyncResultEnum",OSF.DDA),OSF.DDA.AsyncResultEnum.Properties={Context:"Context",Value:"Value",Status:"Status",Error:"Error"},Microsoft.Office.WebExtension.AsyncResultStatus={Succeeded:"succeeded",Failed:"failed"},OSF.DDA.AsyncResultEnum.ErrorCode={Success:0,Failed:1},OSF.DDA.AsyncResultEnum.ErrorProperties={Name:"Name",Message:"Message",Code:"Code"},OSF.DDA.AsyncMethodNames={},OSF.DDA.AsyncMethodNames.addNames=function(e){for(var t in e){var n={};OSF.OUtil.defineEnumerableProperties(n,{id:{value:t},displayName:{value:e[t]}}),OSF.DDA.AsyncMethodNames[t]=n}},OSF.DDA.AsyncMethodCall=function(e,t,n,r,o,i,a){var s=e.length,l=new OSF.DDA.ApiMethodCall(e,t,n,i,a);this.verifyAndExtractCall=function(e,n,r){var o=l.extractRequiredArguments(e,n,r),i=function(e,n,r,o){if(e.length>s+2)throw OsfMsAjaxFactory.msAjaxError.parameterCount(Strings.OfficeOM.L_TooManyArguments);for(var i,a,c=e.length-1;c>=s;c--){var d=e[c];switch(typeof d){case"object":if(i)throw OsfMsAjaxFactory.msAjaxError.parameterCount(Strings.OfficeOM.L_TooManyOptionalObjects);i=d;break;case"function":if(a)throw OsfMsAjaxFactory.msAjaxError.parameterCount(Strings.OfficeOM.L_TooManyOptionalFunction);a=d;break;default:throw OsfMsAjaxFactory.msAjaxError.argument(Strings.OfficeOM.L_InValidOptionalArgument)}}if(i=l.fillOptions(i,n,r,o),a){if(i[Microsoft.Office.WebExtension.Parameters.Callback])throw Strings.OfficeOM.L_RedundantCallbackSpecification;i[Microsoft.Office.WebExtension.Parameters.Callback]=a}return l.verifyArguments(t,i),i}(e,o,n,r);return l.constructCallArgs(o,i,n,r)},this.processResponse=function(e,t,n,i){return e==OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess?r?r(t,n,i):t:o?o(e,t):OSF.DDA.ErrorCodeManager.getErrorArgs(e)},this.getCallArgs=function(e){for(var t,n,r=e.length-1;r>=s;r--){var o=e[r];switch(typeof o){case"object":t=o;break;case"function":n=o}}return t=t||{},n&&(t[Microsoft.Office.WebExtension.Parameters.Callback]=n),t}},OSF.DDA.AsyncMethodCallFactory={manufacture:function(e){var t=e.supportedOptions?OSF.OUtil.createObject(e.supportedOptions):[],n=e.privateStateCallbacks?OSF.OUtil.createObject(e.privateStateCallbacks):[];return new OSF.DDA.AsyncMethodCall(e.requiredArguments||[],t,n,e.onSucceeded,e.onFailed,e.checkCallArgs,e.method.displayName)}},OSF.DDA.AsyncMethodCalls={},OSF.DDA.AsyncMethodCalls.define=function(e){OSF.DDA.AsyncMethodCalls[e.method.id]=OSF.DDA.AsyncMethodCallFactory.manufacture(e)},OSF.DDA.Error=function(e,t,n){OSF.OUtil.defineEnumerableProperties(this,{name:{value:e},message:{value:t},code:{value:n}})},OSF.DDA.AsyncResult=function(e,t){OSF.OUtil.defineEnumerableProperties(this,{value:{value:e[OSF.DDA.AsyncResultEnum.Properties.Value]},status:{value:t?Microsoft.Office.WebExtension.AsyncResultStatus.Failed:Microsoft.Office.WebExtension.AsyncResultStatus.Succeeded}}),e[OSF.DDA.AsyncResultEnum.Properties.Context]&&OSF.OUtil.defineEnumerableProperty(this,"asyncContext",{value:e[OSF.DDA.AsyncResultEnum.Properties.Context]}),t&&OSF.OUtil.defineEnumerableProperty(this,"error",{value:new OSF.DDA.Error(t[OSF.DDA.AsyncResultEnum.ErrorProperties.Name],t[OSF.DDA.AsyncResultEnum.ErrorProperties.Message],t[OSF.DDA.AsyncResultEnum.ErrorProperties.Code])})},OSF.DDA.issueAsyncResult=function(e,t,n){var r=e[Microsoft.Office.WebExtension.Parameters.Callback];if(r){var o,i={};i[OSF.DDA.AsyncResultEnum.Properties.Context]=e[Microsoft.Office.WebExtension.Parameters.AsyncContext],t==OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess?i[OSF.DDA.AsyncResultEnum.Properties.Value]=n:(o={},n=n||OSF.DDA.ErrorCodeManager.getErrorArgs(OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError),o[OSF.DDA.AsyncResultEnum.ErrorProperties.Code]=t||OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError,o[OSF.DDA.AsyncResultEnum.ErrorProperties.Name]=n.name||n,o[OSF.DDA.AsyncResultEnum.ErrorProperties.Message]=n.message||n),r(new OSF.DDA.AsyncResult(i,o))}},OSF.DDA.SyncMethodNames={},OSF.DDA.SyncMethodNames.addNames=function(e){for(var t in e){var n={};OSF.OUtil.defineEnumerableProperties(n,{id:{value:t},displayName:{value:e[t]}}),OSF.DDA.SyncMethodNames[t]=n}},OSF.DDA.SyncMethodCall=function(e,t,n,r,o){var i=e.length,a=new OSF.DDA.ApiMethodCall(e,t,n,r,o);this.verifyAndExtractCall=function(e,n,r){var o=a.extractRequiredArguments(e,n,r),s=function(e,n,r,o){if(e.length>i+1)throw OsfMsAjaxFactory.msAjaxError.parameterCount(Strings.OfficeOM.L_TooManyArguments);for(var s,l=e.length-1;l>=i;l--){var c=e[l];switch(typeof c){case"object":if(s)throw OsfMsAjaxFactory.msAjaxError.parameterCount(Strings.OfficeOM.L_TooManyOptionalObjects);s=c;break;default:throw OsfMsAjaxFactory.msAjaxError.argument(Strings.OfficeOM.L_InValidOptionalArgument)}}return s=a.fillOptions(s,n,r,o),a.verifyArguments(t,s),s}(e,o,n,r);return a.constructCallArgs(o,s,n,r)}},OSF.DDA.SyncMethodCallFactory={manufacture:function(e){var t=e.supportedOptions?OSF.OUtil.createObject(e.supportedOptions):[];return new OSF.DDA.SyncMethodCall(e.requiredArguments||[],t,e.privateStateCallbacks,e.checkCallArgs,e.method.displayName)}},OSF.DDA.SyncMethodCalls={},OSF.DDA.SyncMethodCalls.define=function(e){OSF.DDA.SyncMethodCalls[e.method.id]=OSF.DDA.SyncMethodCallFactory.manufacture(e)},OSF.DDA.ListType=function(){var e={};return{setListType:function(t,n){e[t]=n},isListType:function(t){return OSF.OUtil.listContainsKey(e,t)},getDescriptor:function(t){return e[t]}}}(),OSF.DDA.HostParameterMap=function(e,t){var n="toHost",r="fromHost",o="sourceData",i="self",a={};function s(n,o){var i=n?{}:void 0;for(var l in n){var c,d=n[l];if(OSF.DDA.ListType.isListType(l))for(var u in c=[],d)c.push(s(d[u],o));else if(OSF.OUtil.listContainsKey(a,l))c=a[l][o](d);else if(o==r&&e.preserveNesting(l))c=s(d,o);else{var f=t[l];if(f){var p=f[o];p&&void 0===(c=p[d])&&(c=d)}else c=d}i[l]=c}return i}function l(n,r,a){var l,c=t[n][a];if("toHost"==a)l=function n(r,o){var a;for(var s in o){var l;if(null!=(l=e.isComplexType(s)?n(r,t[s].toHost):r[s])){a||(a={});var c=o[s];c==i&&(c=s),a[c]=e.pack(s,l)}}return a}(s(r,a),c);else if("fromHost"==a){l=s(function n(r,a,s){for(var l in s||(s={}),a){var c,d,u=a[l];if(u==i)c=r;else{if(u==o){s[l]=r.toArray();continue}c=r[u]}if(null==c)s[l]=void 0;else if(c=e.unpack(l,c),e.isComplexType(l))d=t[l].fromHost,e.preserveNesting(l)?s[l]=n(c,d):n(c,d,s);else if(OSF.DDA.ListType.isListType(l)){(d={})[OSF.DDA.ListType.getDescriptor(l)]=i;var f=new Array(c.length);for(var p in c)f[p]=n(c[p],d);s[l]=f}else s[l]=c}return s}(r,c),a)}return l}a[Microsoft.Office.WebExtension.Parameters.Data]={toHost:function(e){if(null!=e&&void 0!==e.rows){var t={};t[OSF.DDA.TableDataProperties.TableRows]=e.rows,t[OSF.DDA.TableDataProperties.TableHeaders]=e.headers,e=t}return e},fromHost:function(e){return e}},a[Microsoft.Office.WebExtension.Parameters.SampleData]=a[Microsoft.Office.WebExtension.Parameters.Data],t||(t={}),this.addMapping=function(e,n){var r,o;if(n.map)for(var a in o={},r=n.map){var s=r[a];s==i&&(s=a),o[s]=a}else r=n.toHost,o=n.fromHost;var l=t[e];if(l){var c=l.toHost;for(var d in c)r[d]=c[d];for(var u in c=l.fromHost)o[u]=c[u]}else l=t[e]={};l.toHost=r,l.fromHost=o},this.toHost=function(e,t){return l(e,t,n)},this.fromHost=function(e,t){return l(e,t,r)},this.self=i,this.sourceData=o,this.addComplexType=function(t){e.addComplexType(t)},this.getDynamicType=function(t){return e.getDynamicType(t)},this.setDynamicType=function(t,n){e.setDynamicType(t,n)},this.dynamicTypes=a,this.doMapValues=function(e,t){return s(e,t)}},OSF.DDA.SpecialProcessor=function(e,t){this.addComplexType=function(t){e.push(t)},this.getDynamicType=function(e){return t[e]},this.setDynamicType=function(e,n){t[e]=n},this.isComplexType=function(t){return OSF.OUtil.listContainsValue(e,t)},this.isDynamicType=function(e){return OSF.OUtil.listContainsKey(t,e)},this.preserveNesting=function(e){var t=[];return OSF.DDA.PropertyDescriptors&&t.push(OSF.DDA.PropertyDescriptors.Subset),OSF.DDA.DataNodeEventProperties&&(t=t.concat([OSF.DDA.DataNodeEventProperties.OldNode,OSF.DDA.DataNodeEventProperties.NewNode,OSF.DDA.DataNodeEventProperties.NextSiblingNode])),OSF.OUtil.listContainsValue(t,e)},this.pack=function(e,n){return this.isDynamicType(e)?t[e].toHost(n):n},this.unpack=function(e,n){return this.isDynamicType(e)?t[e].fromHost(n):n}},OSF.DDA.getDecoratedParameterMap=function(e,t){var n=new OSF.DDA.HostParameterMap(e);n.self;function r(e){var t=null;if(e){t={};for(var n=e.length,r=0;r<n;r++)t[e[r].name]=e[r].value}return t}for(var o in n.define=function(e){var t={},o=r(e.toHost);e.invertible?t.map=o:e.canonical?t.toHost=t.fromHost=o:(t.toHost=o,t.fromHost=r(e.fromHost)),n.addMapping(e.type,t),e.isComplexType&&n.addComplexType(e.type)},t)n.define(t[o]);return n},OSF.OUtil.setNamespace("DispIdHost",OSF.DDA),OSF.DDA.DispIdHost.Methods={InvokeMethod:"invokeMethod",AddEventHandler:"addEventHandler",RemoveEventHandler:"removeEventHandler",OpenDialog:"openDialog",CloseDialog:"closeDialog",MessageParent:"messageParent",SendMessage:"sendMessage"},OSF.DDA.DispIdHost.Delegates={ExecuteAsync:"executeAsync",RegisterEventAsync:"registerEventAsync",UnregisterEventAsync:"unregisterEventAsync",ParameterMap:"parameterMap",OpenDialog:"openDialog",CloseDialog:"closeDialog",MessageParent:"messageParent",SendMessage:"sendMessage"},OSF.DDA.DispIdHost.Facade=function(e,t){var n={},r=OSF.DDA.AsyncMethodNames,o=OSF.DDA.MethodDispId,i={GoToByIdAsync:o.dispidNavigateToMethod,GetSelectedDataAsync:o.dispidGetSelectedDataMethod,SetSelectedDataAsync:o.dispidSetSelectedDataMethod,GetDocumentCopyChunkAsync:o.dispidGetDocumentCopyChunkMethod,ReleaseDocumentCopyAsync:o.dispidReleaseDocumentCopyMethod,GetDocumentCopyAsync:o.dispidGetDocumentCopyMethod,AddFromSelectionAsync:o.dispidAddBindingFromSelectionMethod,AddFromPromptAsync:o.dispidAddBindingFromPromptMethod,AddFromNamedItemAsync:o.dispidAddBindingFromNamedItemMethod,GetAllAsync:o.dispidGetAllBindingsMethod,GetByIdAsync:o.dispidGetBindingMethod,ReleaseByIdAsync:o.dispidReleaseBindingMethod,GetDataAsync:o.dispidGetBindingDataMethod,SetDataAsync:o.dispidSetBindingDataMethod,AddRowsAsync:o.dispidAddRowsMethod,AddColumnsAsync:o.dispidAddColumnsMethod,DeleteAllDataValuesAsync:o.dispidClearAllRowsMethod,RefreshAsync:o.dispidLoadSettingsMethod,SaveAsync:o.dispidSaveSettingsMethod,GetActiveViewAsync:o.dispidGetActiveViewMethod,GetFilePropertiesAsync:o.dispidGetFilePropertiesMethod,GetOfficeThemeAsync:o.dispidGetOfficeThemeMethod,GetDocumentThemeAsync:o.dispidGetDocumentThemeMethod,ClearFormatsAsync:o.dispidClearFormatsMethod,SetTableOptionsAsync:o.dispidSetTableOptionsMethod,SetFormatsAsync:o.dispidSetFormatsMethod,GetAccessTokenAsync:o.dispidGetAccessTokenMethod,GetAuthContextAsync:o.dispidGetAuthContextMethod,ExecuteRichApiRequestAsync:o.dispidExecuteRichApiRequestMethod,AppCommandInvocationCompletedAsync:o.dispidAppCommandInvocationCompletedMethod,CloseContainerAsync:o.dispidCloseContainerMethod,OpenBrowserWindow:o.dispidOpenBrowserWindow,CreateDocumentAsync:o.dispidCreateDocumentMethod,InsertFormAsync:o.dispidInsertFormMethod,ExecuteFeature:o.dispidExecuteFeature,QueryFeature:o.dispidQueryFeature,AddDataPartAsync:o.dispidAddDataPartMethod,GetDataPartByIdAsync:o.dispidGetDataPartByIdMethod,GetDataPartsByNameSpaceAsync:o.dispidGetDataPartsByNamespaceMethod,GetPartXmlAsync:o.dispidGetDataPartXmlMethod,GetPartNodesAsync:o.dispidGetDataPartNodesMethod,DeleteDataPartAsync:o.dispidDeleteDataPartMethod,GetNodeValueAsync:o.dispidGetDataNodeValueMethod,GetNodeXmlAsync:o.dispidGetDataNodeXmlMethod,GetRelativeNodesAsync:o.dispidGetDataNodesMethod,SetNodeValueAsync:o.dispidSetDataNodeValueMethod,SetNodeXmlAsync:o.dispidSetDataNodeXmlMethod,AddDataPartNamespaceAsync:o.dispidAddDataNamespaceMethod,GetDataPartNamespaceAsync:o.dispidGetDataUriByPrefixMethod,GetDataPartPrefixAsync:o.dispidGetDataPrefixByUriMethod,GetNodeTextAsync:o.dispidGetDataNodeTextMethod,SetNodeTextAsync:o.dispidSetDataNodeTextMethod,GetSelectedTask:o.dispidGetSelectedTaskMethod,GetTask:o.dispidGetTaskMethod,GetWSSUrl:o.dispidGetWSSUrlMethod,GetTaskField:o.dispidGetTaskFieldMethod,GetSelectedResource:o.dispidGetSelectedResourceMethod,GetResourceField:o.dispidGetResourceFieldMethod,GetProjectField:o.dispidGetProjectFieldMethod,GetSelectedView:o.dispidGetSelectedViewMethod,GetTaskByIndex:o.dispidGetTaskByIndexMethod,GetResourceByIndex:o.dispidGetResourceByIndexMethod,SetTaskField:o.dispidSetTaskFieldMethod,SetResourceField:o.dispidSetResourceFieldMethod,GetMaxTaskIndex:o.dispidGetMaxTaskIndexMethod,GetMaxResourceIndex:o.dispidGetMaxResourceIndexMethod,CreateTask:o.dispidCreateTaskMethod};for(var a in i)r[a]&&(n[r[a].id]=i[a]);r=OSF.DDA.SyncMethodNames;var s={MessageParent:(o=OSF.DDA.MethodDispId).dispidMessageParentMethod,SendMessage:o.dispidSendMessageMethod};for(var a in s)r[a]&&(n[r[a].id]=s[a]);r=Microsoft.Office.WebExtension.EventType;var l={SettingsChanged:(o=OSF.DDA.EventDispId).dispidSettingsChangedEvent,DocumentSelectionChanged:o.dispidDocumentSelectionChangedEvent,BindingSelectionChanged:o.dispidBindingSelectionChangedEvent,BindingDataChanged:o.dispidBindingDataChangedEvent,ActiveViewChanged:o.dispidActiveViewChangedEvent,OfficeThemeChanged:o.dispidOfficeThemeChangedEvent,DocumentThemeChanged:o.dispidDocumentThemeChangedEvent,AppCommandInvoked:o.dispidAppCommandInvokedEvent,DialogMessageReceived:o.dispidDialogMessageReceivedEvent,DialogParentMessageReceived:o.dispidDialogParentMessageReceivedEvent,ObjectDeleted:o.dispidObjectDeletedEvent,ObjectSelectionChanged:o.dispidObjectSelectionChangedEvent,ObjectDataChanged:o.dispidObjectDataChangedEvent,ContentControlAdded:o.dispidContentControlAddedEvent,RichApiMessage:o.dispidRichApiMessageEvent,ItemChanged:o.dispidOlkItemSelectedChangedEvent,RecipientsChanged:o.dispidOlkRecipientsChangedEvent,AppointmentTimeChanged:o.dispidOlkAppointmentTimeChangedEvent,RecurrenceChanged:o.dispidOlkRecurrenceChangedEvent,AttachmentsChanged:o.dispidOlkAttachmentsChangedEvent,EnhancedLocationsChanged:o.dispidOlkEnhancedLocationsChangedEvent,InfobarClicked:o.dispidOlkInfobarClickedEvent,TaskSelectionChanged:o.dispidTaskSelectionChangedEvent,ResourceSelectionChanged:o.dispidResourceSelectionChangedEvent,ViewSelectionChanged:o.dispidViewSelectionChangedEvent,DataNodeInserted:o.dispidDataNodeAddedEvent,DataNodeReplaced:o.dispidDataNodeReplacedEvent,DataNodeDeleted:o.dispidDataNodeDeletedEvent};for(var c in l)r[c]&&(n[r[c]]=l[c]);function d(e){return e==OSF.DDA.EventDispId.dispidObjectDeletedEvent||e==OSF.DDA.EventDispId.dispidObjectSelectionChangedEvent||e==OSF.DDA.EventDispId.dispidObjectDataChangedEvent||e==OSF.DDA.EventDispId.dispidContentControlAddedEvent}function u(e,t,n,r){if("number"!=typeof e)throw e;r||(r=t.getCallArgs(n)),OSF.DDA.issueAsyncResult(r,e,OSF.DDA.ErrorCodeManager.getErrorArgs(e))}this[OSF.DDA.DispIdHost.Methods.InvokeMethod]=function(r,o,i,a){var s;try{var l=r.id,c=OSF.DDA.AsyncMethodCalls[l];s=c.verifyAndExtractCall(o,i,a);var d=n[l],f=e(l),p=null;if(window.Excel&&window.Office.context.requirements.isSetSupported("RedirectV1Api")&&(window.Excel._RedirectV1APIs=!0),window.Excel&&window.Excel._RedirectV1APIs&&(p=window.Excel._V1APIMap[l])){var g=OSF.OUtil.shallowCopy(s);delete g[Microsoft.Office.WebExtension.Parameters.AsyncContext],p.preprocess&&(g=p.preprocess(g));var m=new window.Excel.RequestContext,S=p.call(m,g);m.sync().then((function(){var e=S.value,t=e.status;delete e.status,delete e["@odata.type"],p.postprocess&&(e=p.postprocess(e,g)),0!=t&&(e=OSF.DDA.ErrorCodeManager.getErrorArgs(t)),OSF.DDA.issueAsyncResult(s,t,e)})).catch((function(e){OSF.DDA.issueAsyncResult(s,OSF.DDA.ErrorCodeManager.errorCodes.ooeFailure,null)}))}else{var O;O=t.toHost?t.toHost(d,s):s;var h=(new Date).getTime();f[OSF.DDA.DispIdHost.Delegates.ExecuteAsync]({dispId:d,hostCallArgs:O,onCalling:function(){},onReceiving:function(){},onComplete:function(e,n){var r;r=e==OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess&&t.fromHost?t.fromHost(d,n):n;var o=c.processResponse(e,r,i,s);OSF.DDA.issueAsyncResult(s,e,o),!OSF.AppTelemetry||OSF.ConstantNames&&OSF.ConstantNames.IsCustomFunctionsRuntime||OSF.AppTelemetry.onMethodDone(d,O,Math.abs((new Date).getTime()-h),e)}})}}catch(e){u(e,c,o,s)}},this[OSF.DDA.DispIdHost.Methods.AddEventHandler]=function(r,o,i,a){var s,l,c,f=!1;function p(e){var t;e==OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess&&((f?o.addObjectEventHandler(l,s[Microsoft.Office.WebExtension.Parameters.Id],c):o.addEventHandler(l,c))||(e=OSF.DDA.ErrorCodeManager.errorCodes.ooeEventHandlerAdditionFailed));e!=OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess&&(t=OSF.DDA.ErrorCodeManager.getErrorArgs(e)),OSF.DDA.issueAsyncResult(s,e,t)}try{var g=OSF.DDA.AsyncMethodCalls[OSF.DDA.AsyncMethodNames.AddHandlerAsync.id];if(s=g.verifyAndExtractCall(r,i,o),l=s[Microsoft.Office.WebExtension.Parameters.EventType],c=s[Microsoft.Office.WebExtension.Parameters.Handler],a)return void p(OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess);var m=n[l],S=(f=d(m))?s[Microsoft.Office.WebExtension.Parameters.Id]:i.id||"";if(0==(f?o.getObjectEventHandlerCount(l,S):o.getEventHandlerCount(l)))(0,e(l)[OSF.DDA.DispIdHost.Delegates.RegisterEventAsync])({eventType:l,dispId:m,targetId:S,onCalling:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.IssueCall)},onReceiving:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.ReceiveResponse)},onComplete:p,onEvent:function(e){var n=t.fromHost(m,e);f?o.fireObjectEvent(S,OSF.DDA.OMFactory.manufactureEventArgs(l,S,n)):o.fireEvent(OSF.DDA.OMFactory.manufactureEventArgs(l,i,n))}});else p(OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess)}catch(e){u(e,g,r,s)}},this[OSF.DDA.DispIdHost.Methods.RemoveEventHandler]=function(t,r,o){var i,a,s,l=!1;function c(e){var t;e!=OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess&&(t=OSF.DDA.ErrorCodeManager.getErrorArgs(e)),OSF.DDA.issueAsyncResult(i,e,t)}try{var f=OSF.DDA.AsyncMethodCalls[OSF.DDA.AsyncMethodNames.RemoveHandlerAsync.id];a=(i=f.verifyAndExtractCall(t,o,r))[Microsoft.Office.WebExtension.Parameters.EventType],s=i[Microsoft.Office.WebExtension.Parameters.Handler];var p,g,m=n[a],S=(l=d(m))?i[Microsoft.Office.WebExtension.Parameters.Id]:o.id||"";null===s?(g=l?r.clearObjectEventHandlers(a,S):r.clearEventHandlers(a),p=OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess):p=(g=l?r.removeObjectEventHandler(a,S,s):r.removeEventHandler(a,s))?OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess:OSF.DDA.ErrorCodeManager.errorCodes.ooeEventHandlerNotExist;var O=l?r.getObjectEventHandlerCount(a,S):r.getEventHandlerCount(a);if(g&&0==O)(0,e(a)[OSF.DDA.DispIdHost.Delegates.UnregisterEventAsync])({eventType:a,dispId:m,targetId:S,onCalling:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.IssueCall)},onReceiving:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.ReceiveResponse)},onComplete:c});else c(p)}catch(e){u(e,f,t,i)}},this[OSF.DDA.DispIdHost.Methods.OpenDialog]=function(r,o,i,a){var s,l,c=null,d=Microsoft.Office.WebExtension.EventType.DialogMessageReceived,f=Microsoft.Office.WebExtension.EventType.DialogEventReceived;function p(e){if(e!=OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess)n=OSF.DDA.ErrorCodeManager.getErrorArgs(e);else{var t={};t[Microsoft.Office.WebExtension.Parameters.Id]=l,t[Microsoft.Office.WebExtension.Parameters.Data]=o;var n=c.processResponse(e,t,i,s);OSF.DialogShownStatus.hasDialogShown=!0,o.clearEventHandlers(d),o.clearEventHandlers(f)}OSF.DDA.issueAsyncResult(s,e,n)}try{if(null!=d&&null!=f||p(OSF.DDA.ErrorCodeManager.ooeOperationNotSupported),a){if(null==OSF.DDA.AsyncMethodNames.DisplayModalDialogAsync)return void p(OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError);c=OSF.DDA.AsyncMethodCalls[OSF.DDA.AsyncMethodNames.DisplayModalDialogAsync.id]}else{if(null==OSF.DDA.AsyncMethodNames.DisplayDialogAsync)return void p(OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError);c=OSF.DDA.AsyncMethodCalls[OSF.DDA.AsyncMethodNames.DisplayDialogAsync.id]}s=c.verifyAndExtractCall(r,i,o);var g=n[d],m=e(d),S=null!=m[OSF.DDA.DispIdHost.Delegates.OpenDialog]?m[OSF.DDA.DispIdHost.Delegates.OpenDialog]:m[OSF.DDA.DispIdHost.Delegates.RegisterEventAsync];s.isModal=a,l=JSON.stringify(s),OSF.DialogShownStatus.hasDialogShown||(o.clearQueuedEvent(d),o.clearQueuedEvent(f),o.clearQueuedEvent(Microsoft.Office.WebExtension.EventType.DialogParentMessageReceived)),S({eventType:d,dispId:g,targetId:l,onCalling:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.IssueCall)},onReceiving:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.ReceiveResponse)},onComplete:p,onEvent:function(e){var n=t.fromHost(g,e),r=OSF.DDA.OMFactory.manufactureEventArgs(d,i,n);if(r.type==f){var a=OSF.DDA.ErrorCodeManager.getErrorArgs(r.error),s={};s[OSF.DDA.AsyncResultEnum.ErrorProperties.Code]=status||OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError,s[OSF.DDA.AsyncResultEnum.ErrorProperties.Name]=a.name||a,s[OSF.DDA.AsyncResultEnum.ErrorProperties.Message]=a.message||a,r.error=new OSF.DDA.Error(s[OSF.DDA.AsyncResultEnum.ErrorProperties.Name],s[OSF.DDA.AsyncResultEnum.ErrorProperties.Message],s[OSF.DDA.AsyncResultEnum.ErrorProperties.Code])}o.fireOrQueueEvent(r),n[OSF.DDA.PropertyDescriptors.MessageType]==OSF.DialogMessageType.DialogClosed&&(o.clearEventHandlers(d),o.clearEventHandlers(f),o.clearEventHandlers(Microsoft.Office.WebExtension.EventType.DialogParentMessageReceived),OSF.DialogShownStatus.hasDialogShown=!1)}})}catch(e){u(e,c,r,s)}},this[OSF.DDA.DispIdHost.Methods.CloseDialog]=function(t,r,o,i){var a,s,l,c=OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess;try{var d=OSF.DDA.AsyncMethodCalls[OSF.DDA.AsyncMethodNames.CloseAsync.id];a=d.verifyAndExtractCall(t,i,o),s=Microsoft.Office.WebExtension.EventType.DialogMessageReceived,l=Microsoft.Office.WebExtension.EventType.DialogEventReceived,o.clearEventHandlers(s),o.clearEventHandlers(l);var f=n[s],p=e(s);(null!=p[OSF.DDA.DispIdHost.Delegates.CloseDialog]?p[OSF.DDA.DispIdHost.Delegates.CloseDialog]:p[OSF.DDA.DispIdHost.Delegates.UnregisterEventAsync])({eventType:s,dispId:f,targetId:r,onCalling:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.IssueCall)},onReceiving:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.ReceiveResponse)},onComplete:function(e){c=e,OSF.DialogShownStatus.hasDialogShown=!1}})}catch(e){u(e,d,t,a)}if(c!=OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess)throw OSF.OUtil.formatString(Strings.OfficeOM.L_FunctionCallFailed,OSF.DDA.AsyncMethodNames.CloseAsync.displayName,c)},this[OSF.DDA.DispIdHost.Methods.MessageParent]=function(t,r){var o=OSF.DDA.SyncMethodCalls[OSF.DDA.SyncMethodNames.MessageParent.id].verifyAndExtractCall(t,r,{});return(0,e(OSF.DDA.SyncMethodNames.MessageParent.id)[OSF.DDA.DispIdHost.Delegates.MessageParent])({dispId:n[OSF.DDA.SyncMethodNames.MessageParent.id],hostCallArgs:o,onCalling:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.IssueCall)},onReceiving:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.ReceiveResponse)}})},this[OSF.DDA.DispIdHost.Methods.SendMessage]=function(t,r,o){var i=OSF.DDA.SyncMethodCalls[OSF.DDA.SyncMethodNames.SendMessage.id].verifyAndExtractCall(t,o,{});return(0,e(OSF.DDA.SyncMethodNames.SendMessage.id)[OSF.DDA.DispIdHost.Delegates.SendMessage])({dispId:n[OSF.DDA.SyncMethodNames.SendMessage.id],hostCallArgs:i,onCalling:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.IssueCall)},onReceiving:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.ReceiveResponse)}})}},OSF.DDA.DispIdHost.addAsyncMethods=function(e,t,n){for(var r in t){var o=t[r],i=o.displayName;e[i]||OSF.OUtil.defineEnumerableProperty(e,i,{value:function(t){return function(){var r=OSF._OfficeAppFactory.getHostFacade()[OSF.DDA.DispIdHost.Methods.InvokeMethod];r(t,arguments,e,n)}}(o)})}},OSF.DDA.DispIdHost.addEventSupport=function(e,t,n){var r=OSF.DDA.AsyncMethodNames.AddHandlerAsync.displayName,o=OSF.DDA.AsyncMethodNames.RemoveHandlerAsync.displayName;e[r]||OSF.OUtil.defineEnumerableProperty(e,r,{value:function(){var r=OSF._OfficeAppFactory.getHostFacade()[OSF.DDA.DispIdHost.Methods.AddEventHandler];r(arguments,t,e,n)}}),e[o]||OSF.OUtil.defineEnumerableProperty(e,o,{value:function(){var n=OSF._OfficeAppFactory.getHostFacade()[OSF.DDA.DispIdHost.Methods.RemoveEventHandler];n(arguments,t,e)}})},function(e){var t=function(){function e(){}return e.isInstanceOfType=function(e,t){if(null==t)return!1;if(t instanceof e)return!0;var n=t.constructor;return n&&"function"==typeof n&&n.__typeName&&"Object"!==n.__typeName||(n=Object),!(n!==e)||n.__typeName&&e.__typeName&&n.__typeName===e.__typeName},e}();e.MsAjaxTypeHelper=t;var n=function(){function e(){}return e.create=function(e,t){var n=new Error(e);if(n.message=e,t)for(var r in t)n[r]=t[r];return n.popStackFrame(),n},e.parameterCount=function(t){var n="Sys.ParameterCountException: "+(t||"Parameter count mismatch."),r=e.create(n,{name:"Sys.ParameterCountException"});return r.popStackFrame(),r},e.argument=function(t,n){var o="Sys.ArgumentException: "+(n||"Value does not fall within the expected range.");t&&(o+="\n"+r.format("Parameter name: {0}",t));var i=e.create(o,{name:"Sys.ArgumentException",paramName:t});return i.popStackFrame(),i},e.argumentNull=function(t,n){var o="Sys.ArgumentNullException: "+(n||"Value cannot be null.");t&&(o+="\n"+r.format("Parameter name: {0}",t));var i=e.create(o,{name:"Sys.ArgumentNullException",paramName:t});return i.popStackFrame(),i},e.argumentOutOfRange=function(t,n,o){var i="Sys.ArgumentOutOfRangeException: "+(o||"Specified argument was out of the range of valid values.");t&&(i+="\n"+r.format("Parameter name: {0}",t)),null!=n&&(i+="\n"+r.format("Actual value was {0}.",n));var a=e.create(i,{name:"Sys.ArgumentOutOfRangeException",paramName:t,actualValue:n});return a.popStackFrame(),a},e.argumentType=function(t,n,o,i){var a="Sys.ArgumentTypeException: ";a+=i||(n&&o?r.format("Object of type '{0}' cannot be converted to type '{1}'.",n.getName?n.getName():n,o.getName?o.getName():o):"Object cannot be converted to the required type."),t&&(a+="\n"+r.format("Parameter name: {0}",t));var s=e.create(a,{name:"Sys.ArgumentTypeException",paramName:t,actualType:n,expectedType:o});return s.popStackFrame(),s},e.argumentUndefined=function(t,n){var o="Sys.ArgumentUndefinedException: "+(n||"Value cannot be undefined.");t&&(o+="\n"+r.format("Parameter name: {0}",t));var i=e.create(o,{name:"Sys.ArgumentUndefinedException",paramName:t});return i.popStackFrame(),i},e.invalidOperation=function(t){var n="Sys.InvalidOperationException: "+(t||"Operation is not valid due to the current state of the object."),r=e.create(n,{name:"Sys.InvalidOperationException"});return r.popStackFrame(),r},e}();e.MsAjaxError=n;var r=function(){function e(){}return e.format=function(e){for(var t=[],n=1;n<arguments.length;n++)t[n-1]=arguments[n];var r=e;return r.replace(/{(\d+)}/gm,(function(e,n){var r=parseInt(n,10);return void 0===t[r]?"{"+n+"}":t[r]}))},e.startsWith=function(e,t){return e.substr(0,t.length)===t},e}();e.MsAjaxString=r;var o=function(){function e(){}return e.trace=function(e){"undefined"!=typeof Debug&&Debug.writeln&&Debug.writeln(e),window.console&&window.console.log&&window.console.log(e),window.opera&&window.opera.postError&&window.opera.postError(e),window.debugService&&window.debugService.trace&&window.debugService.trace(e);var t=document.getElementById("TraceConsole");t&&"TEXTAREA"===t.tagName.toUpperCase()&&(t.innerHTML+=e+"\n")},e}();if(e.MsAjaxDebug=o,!OsfMsAjaxFactory.isMsAjaxLoaded()){var i=function(e,t,n){void 0!==e.__typeName&&null!==e.__typeName||(e.__typeName=t),void 0!==e.__class&&null!==e.__class||(e.__class=n)};i(Function,"Function",!0),i(Error,"Error",!0),i(Object,"Object",!0),i(String,"String",!0),i(Boolean,"Boolean",!0),i(Date,"Date",!0),i(Number,"Number",!0),i(RegExp,"RegExp",!0),i(Array,"Array",!0),Function.createCallback||(Function.createCallback=function(e,t){var n=Function._validateParams(arguments,[{name:"method",type:Function},{name:"context",mayBeNull:!0}]);if(n)throw n;return function(){var n=arguments.length;if(n>0){for(var r=[],o=0;o<n;o++)r[o]=arguments[o];return r[n]=t,e.apply(this,r)}return e.call(this,t)}}),Function.createDelegate||(Function.createDelegate=function(e,t){var n=Function._validateParams(arguments,[{name:"instance",mayBeNull:!0},{name:"method",type:Function}]);if(n)throw n;return function(){return t.apply(e,arguments)}}),Function._validateParams||(Function._validateParams=function(e,t,n){var r,o=t.length;if(n=n||void 0===n,r=Function._validateParameterCount(e,t,n))return r.popStackFrame(),r;for(var i=0,a=e.length;i<a;i++){var s=t[Math.min(i,o-1)],l=s.name;if(s.parameterArray)l+="["+(i-o+1)+"]";else if(!n&&i>=o)break;if(r=Function._validateParameter(e[i],s,l))return r.popStackFrame(),r}return null}),Function._validateParameterCount||(Function._validateParameterCount=function(e,t,r){var o,i,a=t.length,s=e.length;if(s<a){var l=a;for(o=0;o<a;o++){var c=t[o];(c.optional||c.parameterArray)&&l--}s<l&&(i=!0)}else if(r&&s>a)for(i=!0,o=0;o<a;o++)if(t[o].parameterArray){i=!1;break}if(i){var d=n.parameterCount();return d.popStackFrame(),d}return null}),Function._validateParameter||(Function._validateParameter=function(e,t,n){var r,o=t.type,i=!!t.integer,a=!!t.domElement,s=!!t.mayBeNull;if(r=Function._validateParameterType(e,o,i,a,s,n))return r.popStackFrame(),r;var l=t.elementType,c=!!t.elementMayBeNull;if(o===Array&&null!=e&&(l||!c))for(var d=!!t.elementInteger,u=!!t.elementDomElement,f=0;f<e.length;f++){var p=e[f];if(r=Function._validateParameterType(p,l,d,u,c,n+"["+f+"]"))return r.popStackFrame(),r}return null}),Function._validateParameterType||(Function._validateParameterType=function(t,n,r,o,i,a){var s;return void 0===t?i?null:((s=e.MsAjaxError.argumentUndefined(a)).popStackFrame(),s):null===t?i?null:((s=e.MsAjaxError.argumentNull(a)).popStackFrame(),s):n&&!e.MsAjaxTypeHelper.isInstanceOfType(n,t)?((s=e.MsAjaxError.argumentType(a,typeof t,n)).popStackFrame(),s):null}),window.Type||(window.Type=Function),Type.registerNamespace||(Type.registerNamespace=function(e){for(var t=e.split("."),n=window,r=0;r<t.length;r++)n[t[r]]=n[t[r]]||{},n=n[t[r]]}),Type.prototype.registerClass||(Type.prototype.registerClass=function(e){({})}),"undefined"==typeof Sys&&Type.registerNamespace("Sys"),Error.prototype.popStackFrame||(Error.prototype.popStackFrame=function(){if(0!==arguments.length)throw n.parameterCount();if(void 0!==this.stack&&null!==this.stack&&void 0!==this.fileName&&null!==this.fileName&&void 0!==this.lineNumber&&null!==this.lineNumber){for(var e=this.stack.split("\n"),t=e[0],r=this.fileName+":"+this.lineNumber;null!=t&&-1===t.indexOf(r);)e.shift(),t=e[0];var o=e[1];if(null!=o){var i=o.match(/@(.*):(\d+)$/);null!=i&&(this.fileName=i[1],this.lineNumber=parseInt(i[2]),e.shift(),this.stack=e.join("\n"))}}}),OsfMsAjaxFactory.msAjaxError=n,OsfMsAjaxFactory.msAjaxString=r,OsfMsAjaxFactory.msAjaxDebug=o}}(OfficeExt||(OfficeExt={})),OSF.OUtil.setNamespace("SafeArray",OSF.DDA),OSF.DDA.SafeArray.Response={Status:0,Payload:1},OSF.DDA.SafeArray.UniqueArguments={Offset:"offset",Run:"run",BindingSpecificData:"bindingSpecificData",MergedCellGuid:"{66e7831f-81b2-42e2-823c-89e872d541b3}"},OSF.OUtil.setNamespace("Delegate",OSF.DDA.SafeArray),OSF.DDA.SafeArray.Delegate._onException=function(e,t){var n,r=e.number;if(r)switch(r){case-2146828218:n=OSF.DDA.ErrorCodeManager.errorCodes.ooeNoCapability;break;case-2147467259:n=t.dispId==OSF.DDA.EventDispId.dispidDialogMessageReceivedEvent?OSF.DDA.ErrorCodeManager.errorCodes.ooeDialogAlreadyOpened:OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError;break;case-2146828283:case-2147209089:n=OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidParam;break;case-2147208704:n=OSF.DDA.ErrorCodeManager.errorCodes.ooeTooManyIncompleteRequests;break;case-2146827850:default:n=OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError}t.onComplete&&t.onComplete(n||OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError)},OSF.DDA.SafeArray.Delegate._onExceptionSyncMethod=function(e,t){var n,r=e.number;if(r)switch(r){case-2146828218:n=OSF.DDA.ErrorCodeManager.errorCodes.ooeNoCapability;break;case-2146827850:default:n=OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError}return n||OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError},OSF.DDA.SafeArray.Delegate.SpecialProcessor=function(){function e(e){var t;try{var n=e.ubound(1),r=e.ubound(2);if(e=e.toArray(),1==n&&1==r)t=[e];else{t=[];for(var o=0;o<n;o++){for(var i=[],a=0;a<r;a++){var s=e[o*r+a];s!=OSF.DDA.SafeArray.UniqueArguments.MergedCellGuid&&i.push(s)}i.length>0&&t.push(i)}}}catch(e){}return t}var t={};t[Microsoft.Office.WebExtension.Parameters.Data]={toHost:function(e){if(OSF.DDA.TableDataProperties&&"string"!=typeof e&&void 0!==e[OSF.DDA.TableDataProperties.TableRows]){var t=[];t[0]=e[OSF.DDA.TableDataProperties.TableRows],t[1]=e[OSF.DDA.TableDataProperties.TableHeaders],e=t}return e},fromHost:function(t){var n;if(t.toArray)if(2===t.dimensions())n=e(t);else{var r=t.toArray();2===r.length&&(null!=r[0]&&r[0].toArray||null!=r[1]&&r[1].toArray)?((n={})[OSF.DDA.TableDataProperties.TableRows]=e(r[0]),n[OSF.DDA.TableDataProperties.TableHeaders]=e(r[1])):n=r}else n=t;return n}},OSF.DDA.SafeArray.Delegate.SpecialProcessor.uber.constructor.call(this,[],t),this.unpack=function(e,n){var r;this.isComplexType(e)||OSF.DDA.ListType.isListType(e)?r=void 0!==n&&void 0!==n.toArray?n.toArray():n||{}:r=this.isDynamicType(e)?t[e].fromHost(n):n;return r}},OSF.OUtil.extend(OSF.DDA.SafeArray.Delegate.SpecialProcessor,OSF.DDA.SpecialProcessor),OSF.DDA.SafeArray.Delegate.ParameterMap=OSF.DDA.getDecoratedParameterMap(new OSF.DDA.SafeArray.Delegate.SpecialProcessor,[{type:Microsoft.Office.WebExtension.Parameters.ValueFormat,toHost:[{name:Microsoft.Office.WebExtension.ValueFormat.Unformatted,value:0},{name:Microsoft.Office.WebExtension.ValueFormat.Formatted,value:1}]},{type:Microsoft.Office.WebExtension.Parameters.FilterType,toHost:[{name:Microsoft.Office.WebExtension.FilterType.All,value:0}]}]),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.PropertyDescriptors.AsyncResultStatus,fromHost:[{name:Microsoft.Office.WebExtension.AsyncResultStatus.Succeeded,value:0},{name:Microsoft.Office.WebExtension.AsyncResultStatus.Failed,value:1}]}),OSF.DDA.SafeArray.Delegate.executeAsync=function(e){function t(e){var n=e;if(null!=e&&e.toArray){var r=e.toArray();n=new Array(r.length);for(var o=0;o<r.length;o++)n[o]=t(r[o])}return n}try{e.onCalling&&e.onCalling(),OSF.ClientHostController.execute(e.dispId,function e(t){var n=t;if(OSF.OUtil.isArray(t))for(var r=n.length,o=0;o<r;o++)n[o]=e(n[o]);else if(OSF.OUtil.isDate(t))n=t.getVarDate();else if("object"==typeof t&&!OSF.OUtil.isArray(t))for(var i in n=[],t)OSF.OUtil.isFunction(t[i])||(n[i]=e(t[i]));return n}(e.hostCallArgs),(function(n,r){var o,i;if("number"==typeof n?(o=[],i=n):i=(o=n.toArray())[OSF.DDA.SafeArray.Response.Status],i==OSF.DDA.ErrorCodeManager.errorCodes.ooeChunkResult)return null!=(a=t(a=o[OSF.DDA.SafeArray.Response.Payload]))&&(e._chunkResultData||(e._chunkResultData=new Array),e._chunkResultData[a[0]]=a[1]),!1;if(e.onReceiving&&e.onReceiving(),e.onComplete){var a;if(i==OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess){if(o.length>2){a=[];for(var s=1;s<o.length;s++)a[s-1]=o[s]}else a=o[OSF.DDA.SafeArray.Response.Payload];if(e._chunkResultData&&null!=(a=t(a))){var l=a[a.length-1];e._chunkResultData.length==l?a[a.length-1]=e._chunkResultData:i=OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError}}else a=o[OSF.DDA.SafeArray.Response.Payload];e.onComplete(i,a)}return!0}))}catch(t){OSF.DDA.SafeArray.Delegate._onException(t,e)}},OSF.DDA.SafeArray.Delegate._getOnAfterRegisterEvent=function(e,t){var n=(new Date).getTime();return function(r){t.onReceiving&&t.onReceiving();var o=r.toArray?r.toArray()[OSF.DDA.SafeArray.Response.Status]:r;t.onComplete&&t.onComplete(o),OSF.AppTelemetry&&OSF.AppTelemetry.onRegisterDone(e,t.dispId,Math.abs((new Date).getTime()-n),o)}},OSF.DDA.SafeArray.Delegate.registerEventAsync=function(e){e.onCalling&&e.onCalling();var t=OSF.DDA.SafeArray.Delegate._getOnAfterRegisterEvent(!0,e);try{OSF.ClientHostController.registerEvent(e.dispId,e.targetId,(function(t,n){e.onEvent&&e.onEvent(n),OSF.AppTelemetry&&OSF.AppTelemetry.onEventDone(e.dispId)}),t)}catch(t){OSF.DDA.SafeArray.Delegate._onException(t,e)}},OSF.DDA.SafeArray.Delegate.unregisterEventAsync=function(e){e.onCalling&&e.onCalling();var t=OSF.DDA.SafeArray.Delegate._getOnAfterRegisterEvent(!1,e);try{OSF.ClientHostController.unregisterEvent(e.dispId,e.targetId,t)}catch(t){OSF.DDA.SafeArray.Delegate._onException(t,e)}},OSF.ClientMode={ReadWrite:0,ReadOnly:1},OSF.DDA.RichInitializationReason={1:Microsoft.Office.WebExtension.InitializationReason.Inserted,2:Microsoft.Office.WebExtension.InitializationReason.DocumentOpened},OSF.InitializationHelper=function(e,t,n,r,o){this._hostInfo=e,this._webAppState=t,this._context=n,this._settings=r,this._hostFacade=o,this._initializeSettings=this.initializeSettings},OSF.InitializationHelper.prototype.deserializeSettings=function(e,t){var n=OSF.OUtil.getSessionStorage();if(n){var r=n.getItem(OSF._OfficeAppFactory.getCachedSessionSettingsKey());r?e=JSON.parse(r):(r=JSON.stringify(e),n.setItem(OSF._OfficeAppFactory.getCachedSessionSettingsKey(),r))}var o=OSF.DDA.SettingsManager.deserializeSettings(e);return t?new OSF.DDA.RefreshableSettings(o):new OSF.DDA.Settings(o)},OSF.InitializationHelper.prototype.saveAndSetDialogInfo=function(e){},OSF.InitializationHelper.prototype.setAgaveHostCommunication=function(){},OSF.InitializationHelper.prototype.prepareRightBeforeWebExtensionInitialize=function(e){this.prepareApiSurface(e),Microsoft.Office.WebExtension.initialize(this.getInitializationReason(e))},OSF.InitializationHelper.prototype.prepareApiSurface=function(e){var t,n,r=new OSF.DDA.License(e.get_eToken()),o=OSF.DDA.OfficeTheme&&OSF.DDA.OfficeTheme.getOfficeTheme?OSF.DDA.OfficeTheme.getOfficeTheme:null;e.get_isDialog()?OSF.DDA.UI.ChildUI&&(e.ui=new OSF.DDA.UI.ChildUI):OSF.DDA.UI.ParentUI&&(e.ui=new OSF.DDA.UI.ParentUI,OfficeExt.Container&&OSF.DDA.DispIdHost.addAsyncMethods(e.ui,[OSF.DDA.AsyncMethodNames.CloseContainerAsync])),OSF.DDA.OpenBrowser&&OSF.DDA.DispIdHost.addAsyncMethods(e.ui,[OSF.DDA.AsyncMethodNames.OpenBrowserWindow]),OSF.DDA.ExecuteFeature&&OSF.DDA.DispIdHost.addAsyncMethods(e.ui,[OSF.DDA.AsyncMethodNames.ExecuteFeature]),OSF.DDA.QueryFeature&&OSF.DDA.DispIdHost.addAsyncMethods(e.ui,[OSF.DDA.AsyncMethodNames.QueryFeature]),OSF.DDA.Auth&&(e.auth=new OSF.DDA.Auth,OSF.DDA.DispIdHost.addAsyncMethods(e.auth,[OSF.DDA.AsyncMethodNames.GetAccessTokenAsync])),OSF._OfficeAppFactory.setContext(new OSF.DDA.Context(e,e.doc,r,null,o)),t=OSF.DDA.DispIdHost.getClientDelegateMethods,n=OSF.DDA.SafeArray.Delegate.ParameterMap,OSF._OfficeAppFactory.setHostFacade(new OSF.DDA.DispIdHost.Facade(t,n))},OSF.InitializationHelper.prototype.getInitializationReason=function(e){return OSF.DDA.RichInitializationReason[e.get_reason()]},OSF.DDA.DispIdHost.getClientDelegateMethods=function(e){var t={};if(t[OSF.DDA.DispIdHost.Delegates.ExecuteAsync]=OSF.DDA.SafeArray.Delegate.executeAsync,t[OSF.DDA.DispIdHost.Delegates.RegisterEventAsync]=OSF.DDA.SafeArray.Delegate.registerEventAsync,t[OSF.DDA.DispIdHost.Delegates.UnregisterEventAsync]=OSF.DDA.SafeArray.Delegate.unregisterEventAsync,t[OSF.DDA.DispIdHost.Delegates.OpenDialog]=OSF.DDA.SafeArray.Delegate.openDialog,t[OSF.DDA.DispIdHost.Delegates.CloseDialog]=OSF.DDA.SafeArray.Delegate.closeDialog,t[OSF.DDA.DispIdHost.Delegates.MessageParent]=OSF.DDA.SafeArray.Delegate.messageParent,t[OSF.DDA.DispIdHost.Delegates.SendMessage]=OSF.DDA.SafeArray.Delegate.sendMessage,OSF.DDA.AsyncMethodNames.RefreshAsync&&e==OSF.DDA.AsyncMethodNames.RefreshAsync.id){t[OSF.DDA.DispIdHost.Delegates.ExecuteAsync]=OSF.DDA.ClientSettingsManager.getSettingsExecuteMethod((function(e,t,n){return"function"==typeof OSF.DDA.ClientSettingsManager.refresh?OSF.DDA.ClientSettingsManager.refresh(t,n):OSF.DDA.ClientSettingsManager.read(t,n)}))}if(OSF.DDA.AsyncMethodNames.SaveAsync&&e==OSF.DDA.AsyncMethodNames.SaveAsync.id){t[OSF.DDA.DispIdHost.Delegates.ExecuteAsync]=OSF.DDA.ClientSettingsManager.getSettingsExecuteMethod((function(e,t,n){return OSF.DDA.ClientSettingsManager.write(e[OSF.DDA.SettingsManager.SerializedSettings],e[Microsoft.Office.WebExtension.Parameters.OverwriteIfStale],t,n)}))}return t},function(e){var t=function(){function e(){}return e.prototype.execute=function(e,t,n){setTimeout((function(){window.external.Execute(e,t,n)}),0)},e.prototype.registerEvent=function(e,t,n,r){setTimeout((function(){window.external.RegisterEvent(e,t,n,r)}),0)},e.prototype.unregisterEvent=function(e,t,n){setTimeout((function(){window.external.UnregisterEvent(e,t,n)}),0)},e.prototype.openDialog=function(t,n,r,o){if(!e.popup||e.popup.closed){window.dialogAPIErrorCode=void 0;var i=JSON.parse(n),a=i.url;if(a){var s=a.split("#"),l="?";s[0].indexOf("?")>-1&&(l="&");var c="width="+screen.width*i.width/100+", height="+screen.height*i.height/100;s[0]=s[0].concat(l).concat("action=displayDialog");var d=s.join("#");if(e.popup=window.open(d,"",c),e.DialogEventListener=function(n){if(n.source==e.popup)try{var o=JSON.parse(n.data);o.dialogMessage&&r(t,[OSF.DialogMessageType.DialogMessageReceived,o.dialogMessage.messageContent])}catch(e){OsfMsAjaxFactory.msAjaxDebug.trace("messages received cannot be handlered. Message:"+n.data)}},null!=e.popup&&null==window.dialogAPIErrorCode)window.addEventListener("message",e.DialogEventListener),e.interval=window.setInterval((function(){try{(null==e.popup||e.popup.closed)&&(window.clearInterval(e.interval),window.removeEventListener("message",e.DialogEventListener),e.NotifyError=null,r(t,[OSF.DialogMessageType.DialogClosed]))}catch(e){OsfMsAjaxFactory.msAjaxDebug.trace("Error happened when popup window closed.")}}),500),e.NotifyError=function(e){r(t,[e])},o(OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess);else{var u=OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError;window.dialogAPIErrorCode&&(u=window.dialogAPIErrorCode),o(u)}}else o(OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidScheme)}else o(OSF.DDA.ErrorCodeManager.errorCodes.ooeDialogAlreadyOpened)},e.prototype.messageParent=function(e){var t=e[Microsoft.Office.WebExtension.Parameters.MessageToParent],n={dialogMessage:{messageType:OSF.DialogMessageType.DialogMessageReceived,messageContent:t}};window.opener.postMessage(JSON.stringify(n),window.location.origin)},e.prototype.closeDialog=function(t,n,r){e.popup?(e.interval&&window.clearInterval(e.interval),e.popup.close(),e.popup=null,window.removeEventListener("message",e.DialogEventListener),e.NotifyError=null,r(OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess)):r(OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError)},e.prototype.sendMessage=function(e){},e}();e.MacRichClientHostController=t}(OfficeExt||(OfficeExt={}));var OSFWebkit,OSFLog,Logger,OSFAriaLogger,OSFAppTelemetry;OSF=OSF||{};!function(e){var t=function(){function e(e){this.data=e,this.safeArrayFlag=this.isSafeArray(e)}return e.prototype.dimensions=function(){var e=0;return this.safeArrayFlag?e=this.data[0][0]:this.isArray()&&(e=2),e},e.prototype.getItem=function(){var e=null;e=this.safeArrayFlag?this.toArray():this.data;for(var t=0;t<arguments.length;t++)e=e[arguments[t]];return e},e.prototype.lbound=function(e){return 0},e.prototype.ubound=function(e){var t=0;if(this.safeArrayFlag)t=this.data[0][e];else if(this.isArray()){if(1==e)return this.data.length;if(2==e){if(OSF.OUtil.isArray(this.data[0]))return this.data[0].length;if(null!=this.data[0])return 1}}return t},e.prototype.toArray=function(){if(0==this.isArray())return this.data;for(var t=[],n=this.safeArrayFlag?1:0;n<this.data.length;n++){var r=this.data[n];this.isSafeArray(r)?t.push(new e(r)):t.push(r)}return t},e.prototype.isArray=function(){return OSF.OUtil.isArray(this.data)},e.prototype.isSafeArray=function(e){var t=!1;if(OSF.OUtil.isArray(e)&&OSF.OUtil.isArray(e[0])){var n=e[0],r=n[0];if(n.length!=r+1)return!1;for(var o=1,i=1;i<n.length;i++){var a=n[i];if(0==isFinite(a))return!1;o*=a}t=++o==e.length}return t},e}();e.WebkitSafeArray=t}(OSFWebkit||(OSFWebkit={})),function(e){!function(e){var t=null;e.agaveHostCallback=function(e,n){t.agaveHostCallback(e,n)},e.agaveHostEventCallback=function(e,n){t.agaveHostEventCallback(e,n)},e.GetScriptMessenger=function(){return null==t&&(t=r.IsSupportedByHost()?new r:new o("OSF.ScriptMessaging.agaveHostCallback","OSF.ScriptMessaging.agaveHostEventCallback")),t};var n=function(e,t,n){this.id=e,this.targetId=t,this.handler=n},r=function(){function e(){this.eventHandlers=[]}return e.IsSupportedByHost=function(){return!!window._osfHostAPI},e.prototype.invokeMethod=function(e,t,n,r){window._osfHostAPI(e,this.constructMessagingArgs(t,n),(function(e){var t=r(e);return void 0===t||!0===t}))},e.prototype.registerEvent=function(e,t,n,r,o,i){var a={id:n,targetId:r},s=window._osfHostRegisterEvent(e,this.constructMessagingArgs(t,a),(function(e){return o(e),!1}),(function(e){return i(e),!0}));this.eventHandlers.push({targetId:r,dispId:n,callbackId:s.eventCallbackId})},e.prototype.unregisterEvent=function(e,t,n,r,o){var i={id:n,targetId:r};window._osfHostAPI(e,this.constructMessagingArgs(t,i),(function(e){return o(e),!0}));for(var a=this.eventHandlers.length-1;a>=0;a--){var s=this.eventHandlers[a];s.dispId===n&&s.targetId===r&&(this.eventHandlers.splice(a,1),window._osfHostUnregisterCallback(s.callbackId))}},e.prototype.agaveHostCallback=function(e,t){},e.prototype.agaveHostEventCallback=function(e,t){},e.prototype.constructMessagingArgs=function(e,t){return{params:t,methodId:e}},e}(),o=function(){function e(t,n){this.callingIndex=0,this.callbackList={},this.eventHandlerList={},this.asyncMethodCallbackFunctionName=t,this.eventCallbackFunctionName=n,this.conversationId=e.getCurrentTimeMS().toString()}return e.prototype.invokeMethod=function(e,t,n,r){this.postWebkitMessage({},e,t,n,r)},e.prototype.registerEvent=function(e,t,r,o,i,a){var s={eventCallbackFunction:this.eventCallbackFunctionName},l={id:r,targetId:o},c=this.postWebkitMessage(s,e,t,l,a);this.eventHandlerList[c]=new n(r,o,i)},e.prototype.unregisterEvent=function(e,t,n,r,o){var i={id:n,targetId:r};for(var a in this.eventHandlerList)if(this.eventHandlerList.hasOwnProperty(a)){var s=this.eventHandlerList[a];s.id==n&&s.targetId==r&&delete this.eventHandlerList[a]}this.invokeMethod(e,t,i,o)},e.prototype.agaveHostCallback=function(e,t){var n=this.callbackList[e];if(n){var r=n(t);void 0!==r&&!0!==r||delete this.callbackList[e]}},e.prototype.agaveHostEventCallback=function(e,t){var n=this.eventHandlerList[e];n&&n.handler(t)},e.prototype.postWebkitMessage=function(t,n,r,o,i){t.methodId=r,t.params=o;var a="";i&&(a=this.generateCorrelationId(),this.callbackList[a]=i,t.callbackId=a,t.callbackFunction=this.asyncMethodCallbackFunctionName);var s=function(){window.webkit.messageHandlers[n].postMessage(JSON.stringify(t))},l=e.getCurrentTimeMS();return null==this.lastMessageTimestamp||l-this.lastMessageTimestamp>=e.MESSAGE_TIME_DELTA?(s(),this.lastMessageTimestamp=l):(this.lastMessageTimestamp+=e.MESSAGE_TIME_DELTA,setTimeout((function(){s()}),this.lastMessageTimestamp-l)),a},e.prototype.generateCorrelationId=function(){return++this.callingIndex,this.conversationId+this.callingIndex},e.getCurrentTimeMS=function(){return(new Date).getTime()},e.MESSAGE_TIME_DELTA=10,e}()}(e.ScriptMessaging||(e.ScriptMessaging={}))}(OSFWebkit||(OSFWebkit={})),OSF.ScriptMessaging=OSFWebkit.ScriptMessaging,function(e){e.MessageHandlerName="Agave",e.PopupMessageHandlerName="WefPopupHandler",function(e){e[e.Settings=0]="Settings",e[e.SolutionReferenceId=1]="SolutionReferenceId",e[e.AppType=2]="AppType",e[e.MajorVersion=3]="MajorVersion",e[e.MinorVersion=4]="MinorVersion",e[e.RevisionVersion=5]="RevisionVersion",e[e.APIVersionSequence=6]="APIVersionSequence",e[e.AppCapabilities=7]="AppCapabilities",e[e.APPUILocale=8]="APPUILocale",e[e.AppDataLocale=9]="AppDataLocale",e[e.BindingCount=10]="BindingCount",e[e.DocumentUrl=11]="DocumentUrl",e[e.ActivationMode=12]="ActivationMode",e[e.ControlIntegrationLevel=13]="ControlIntegrationLevel",e[e.SolutionToken=14]="SolutionToken",e[e.APISetVersion=15]="APISetVersion",e[e.CorrelationId=16]="CorrelationId",e[e.InstanceId=17]="InstanceId",e[e.TouchEnabled=18]="TouchEnabled",e[e.CommerceAllowed=19]="CommerceAllowed",e[e.RequirementMatrix=20]="RequirementMatrix",e[e.HostCustomMessage=21]="HostCustomMessage",e[e.HostFullVersion=22]="HostFullVersion",e[e.InitialDisplayMode=23]="InitialDisplayMode"}(e.AppContextProperties||(e.AppContextProperties={})),function(e){e[e.Execute=1]="Execute",e[e.RegisterEvent=2]="RegisterEvent",e[e.UnregisterEvent=3]="UnregisterEvent",e[e.WriteSettings=4]="WriteSettings",e[e.GetContext=5]="GetContext",e[e.SendMessage=6]="SendMessage",e[e.MessageParent=7]="MessageParent",e[e.SendMessage2=8]="SendMessage2",e[e.MessageParent2=9]="MessageParent2"}(e.MethodId||(e.MethodId={}));var t=function(){function t(e){this.hostScriptProxy=e,this.useFullDialogAPI=!!window._enableFullDialogAPI,this.osfSupportsDialogOrigin=!!window._osfEnableDialogOrigin}return t.prototype.execute=function(t,n,r){var o={id:t,apiArgs:n};this.hostScriptProxy.invokeMethod(OSF.Webkit.MessageHandlerName,OSF.Webkit.MethodId.Execute,o,(function(t){if(r){var n=[];if(OSF.OUtil.isArray(t))for(var o=0;o<t.length;o++){var i=t[o];OSF.OUtil.isArray(i)&&(i=new e.WebkitSafeArray(i)),n.unshift(i)}return r.apply(null,n)}}))},t.prototype.registerEvent=function(t,n,r,o){this.hostScriptProxy.registerEvent(OSF.Webkit.MessageHandlerName,OSF.Webkit.MethodId.RegisterEvent,t,n,(function(t){var n=t,o=0;OSF.OUtil.isArray(t)&&t.length>=2&&(n=t[0],o=t[1]),r&&r(o,new e.WebkitSafeArray(n))}),(function(t){if(o)return o(new e.WebkitSafeArray(t))}))},t.prototype.unregisterEvent=function(t,n,r){this.hostScriptProxy.unregisterEvent(OSF.Webkit.MessageHandlerName,OSF.Webkit.MethodId.UnregisterEvent,t,n,(function(t){return r(new e.WebkitSafeArray(t))}))},t.prototype.messageParent=function(e){if(this.useFullDialogAPI&&e){var t=e[Microsoft.Office.WebExtension.Parameters.MessageToParent];"boolean"==typeof t&&(!0===t?e[Microsoft.Office.WebExtension.Parameters.MessageToParent]="true":!1===t&&(e[Microsoft.Office.WebExtension.Parameters.MessageToParent]=""))}var n,r;if(this.osfSupportsDialogOrigin&&e[Microsoft.Office.WebExtension.Parameters.TargetOrigin]?(n=this.useFullDialogAPI?JSON.stringify(e):e[Microsoft.Office.WebExtension.Parameters.MessageToParent],r=OSF.Webkit.MethodId.MessageParent2):(n=e[Microsoft.Office.WebExtension.Parameters.MessageToParent],r=OSF.Webkit.MethodId.MessageParent),this.useFullDialogAPI)this.hostScriptProxy.invokeMethod(OSF.Webkit.MessageHandlerName,r,n,null);else{var o={dialogMessage:{messageType:OSF.DialogMessageType.DialogMessageReceived,messageContent:n}},i=r==OSF.Webkit.MethodId.MessageParent2?e[Microsoft.Office.WebExtension.Parameters.TargetOrigin]:window.location.origin;window.opener.postMessage(JSON.stringify(o),i)}},t.prototype.openDialog=function(e,n,r,o){if(this.useFullDialogAPI)this.registerEvent(e,n,r,o);else if(!t.popup||t.popup.closed){t.OpenDialogCallback=void 0;var i=JSON.parse(n),a=i.url;if(a){var s=a.split("#"),l="?";s[0].indexOf("?")>-1&&(l="&");var c="width="+screen.width*i.width/100+", height="+screen.height*i.height/100;s[0]=s[0].concat(l).concat("action=displayDialog");var d=s.join("#");t.popup=window.open(d,"",c),t.DialogEventListener=function(t){if(t.origin==window.location.origin)try{var n=JSON.parse(t.data);n.dialogMessage&&r(e,[OSF.DialogMessageType.DialogMessageReceived,n.dialogMessage.messageContent,t.origin])}catch(e){OsfMsAjaxFactory.msAjaxDebug.trace("messages received cannot be handlered. Message:"+t.data)}},t.OpenDialogCallback=function(e){e==OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess&&(window.addEventListener("message",t.DialogEventListener),t.interval=window.setInterval(u,1e3),t.NotifyError=f),o(e)}}else o(OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidScheme)}else o(OSF.DDA.ErrorCodeManager.errorCodes.ooeDialogAlreadyOpened);function u(){try{(null==t.popup||t.popup.closed)&&(window.clearInterval(t.interval),window.removeEventListener("message",t.DialogEventListener),t.NotifyError=null,t.popup=null,r(e,[OSF.DialogMessageType.DialogClosed]))}catch(e){OsfMsAjaxFactory.msAjaxDebug.trace("Error happened when popup window closed.")}}function f(t){r(e,[t])}},t.prototype.closeDialog=function(e,n,r){this.useFullDialogAPI?this.unregisterEvent(e,n,r):t.popup?(t.interval&&window.clearInterval(t.interval),t.popup.close(),t.popup=null,window.removeEventListener("message",t.DialogEventListener),t.NotifyError=null,r(OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess)):r(OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError)},t.prototype.sendMessage=function(e){var t,n;this.osfSupportsDialogOrigin&&e[Microsoft.Office.WebExtension.Parameters.TargetOrigin]?(t=JSON.stringify(e),n=OSF.Webkit.MethodId.SendMessage2):(t=e[Microsoft.Office.WebExtension.Parameters.MessageContent],n=OSF.Webkit.MethodId.SendMessage,!isNaN(parseFloat(t))&&isFinite(t)&&(t=t.toString())),this.hostScriptProxy.invokeMethod(OSF.Webkit.MessageHandlerName,n,t,null)},t}();e.WebkitHostController=t}(OSFWebkit||(OSFWebkit={})),OSF.Webkit=OSFWebkit,OSF.initializeWebkitCommon=function(){OSF.DDA.ClientSettingsManager={getSettingsExecuteMethod:function(e){return function(t){var n=function(e,n){t.onReceiving&&t.onReceiving(),t.onComplete&&t.onComplete(e,n)};try{e(t.hostCallArgs,t.onCalling,n)}catch(e){n(OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError,{name:Strings.OfficeOM.L_InternalError,message:e})}}},read:function(e,t){e&&e();OSF._OfficeAppFactory.getInitializationHelper().getAppContext(null,(function(e){t&&t(OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess,e.get_settings())}))},write:function(e,t,n,r){var o={},i=[],a=[];for(var s in e)i.push(s),a.push(e[s]);o.keys=i,o.values=a,n&&n();OSF.ScriptMessaging.GetScriptMessenger().invokeMethod(OSF.Webkit.MessageHandlerName,OSF.Webkit.MethodId.WriteSettings,o,(function(e){r&&r(e[0],null)}))}},OSF.InitializationHelper.prototype.initializeSettings=function(e,t){var n=e.get_settings();return this.deserializeSettings(n,t)},OSF.InitializationHelper.prototype.getAppContext=function(e,t){var n;n=this._hostInfo.isDialog&&window.webkit.messageHandlers[OSF.Webkit.PopupMessageHandlerName]?OSF.Webkit.PopupMessageHandlerName:OSF.Webkit.MessageHandlerName,OSF.ScriptMessaging.GetScriptMessenger().invokeMethod(n,OSF.Webkit.MethodId.GetContext,[],(function(e){for(var n,r=OSF.Webkit.AppContextProperties,o=e[r.AppType],i=e[r.Settings],a={},s=i[0],l=i[1],c=0;c<s.length;c++)a[s[c]]=l[c];var d=e[r.SolutionReferenceId],u=e[r.MajorVersion],f=e[r.MinorVersion],p=e[r.AppCapabilities],g=e[r.APPUILocale],m=e[r.AppDataLocale],S=e[r.DocumentUrl],O=e[r.ActivationMode],h=e[r.ControlIntegrationLevel],D=e[r.SolutionToken];D=D?D.toString():"";var v=e[r.CorrelationId],A=e[r.InstanceId],y=e[r.TouchEnabled],F=e[r.CommerceAllowed],C=e[r.RequirementMatrix],M=e[r.HostCustomMessage],E=e[r.HostFullVersion],b=e[r.InitialDisplayMode];n=new OSF.OfficeAppContext(d,o,u,g,m,S,p,a,O,h,D,v,A,y,F,f,C,M,E,void 0,void 0,void 0,void 0,void 0,void 0,void 0,b),OSF.AppTelemetry&&OSF.AppTelemetry.initialize(n),t(n)}))}},function(e){!function(t){var n=function(){function e(){this._osfOfficeTheme=null,this._osfOfficeThemeTimeStamp=null}return e.prototype.getOfficeTheme=function(){if(OSF.DDA._OsfControlContext){if(this._osfOfficeTheme&&this._osfOfficeThemeTimeStamp&&(new Date).getTime()-this._osfOfficeThemeTimeStamp<e._osfOfficeThemeCacheValidPeriod)OSF.AppTelemetry&&OSF.AppTelemetry.onPropertyDone("GetOfficeThemeInfo",0);else{var t=(new Date).getTime(),n=OSF.DDA._OsfControlContext.GetOfficeThemeInfo(),r=(new Date).getTime();for(var o in OSF.AppTelemetry&&OSF.AppTelemetry.onPropertyDone("GetOfficeThemeInfo",Math.abs(r-t)),this._osfOfficeTheme=JSON.parse(n),this._osfOfficeTheme)this._osfOfficeTheme[o]=OSF.OUtil.convertIntToCssHexColor(this._osfOfficeTheme[o]);this._osfOfficeThemeTimeStamp=r}return this._osfOfficeTheme}},e.instance=function(){return null==e._instance&&(e._instance=new e),e._instance},e._osfOfficeThemeCacheValidPeriod=5e3,e._instance=null,e}();t.OfficeThemeManager=n,OSF.OUtil.setNamespace("OfficeTheme",OSF.DDA),OSF.DDA.OfficeTheme.getOfficeTheme=e.OfficeTheme.OfficeThemeManager.instance().getOfficeTheme}(e.OfficeTheme||(e.OfficeTheme={}))}(OfficeExt||(OfficeExt={})),OSF.initializeRichCommon=function(){OSF.DDA.ClientSettingsManager={getSettingsExecuteMethod:function(e){return function(t){var n;try{n=e(t.hostCallArgs,t.onCalling,(function(e,n){t.onReceiving&&t.onReceiving(),t.onComplete&&t.onComplete(e,n)}))}catch(e){var r=OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError;n={name:Strings.OfficeOM.L_InternalError,message:e},t.onComplete&&t.onComplete(r,n)}}},read:function(e,t){var n=[],r=[];e&&e(),"undefined"!=typeof OsfOMToken&&OsfOMToken?OSF.DDA._OsfControlContext.GetSettings(OsfOMToken).Read(n,r):OSF.DDA._OsfControlContext.GetSettings().Read(n,r);for(var o={},i=0;i<n.length;i++)o[n[i]]=r[i];return t&&t(OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess,o),o},write:function(e,t,n,r){var o,i=[],a=[];for(var s in e)i.push(s),a.push(e[s]);n&&n(),void 0!==(o="undefined"!=typeof OsfOMToken&&OsfOMToken?OSF.DDA._OsfControlContext.GetSettings(OsfOMToken):OSF.DDA._OsfControlContext.GetSettings()).WriteAsync?o.WriteAsync(i,a,r):(o.Write(i,a),r&&r(OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess))},refresh:function(e,t){var n,r=[],o=[];e&&e(),n="undefined"!=typeof OsfOMToken&&OsfOMToken?OSF.DDA._OsfControlContext.GetSettings(OsfOMToken):OSF.DDA._OsfControlContext.GetSettings();var i=function(){n.Read(r,o);for(var e={},i=0;i<r.length;i++)e[r[i]]=o[i];t&&t(OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess,e)};n.RefreshAsync?n.RefreshAsync((function(){i()})):i()}},OSF.InitializationHelper.prototype.initializeSettings=function(e){var t=OSF.DDA.ClientSettingsManager.read();return this.deserializeSettings(t,e)},OSF.InitializationHelper.prototype.getAppContext=function(e,t){var n,r,o="Warning: Office.js is loaded outside of Office client";try{if(!window.external||!OSF.OUtil.externalNativeFunctionExists(typeof window.external.GetContext))return void OsfMsAjaxFactory.msAjaxDebug.trace(o);r=OSF.DDA._OsfControlContext=window.external.GetContext()}catch(e){return void OsfMsAjaxFactory.msAjaxDebug.trace(o)}var i,a,s,l,c,d,u,f,p,g,m=r.GetAppType(),S=r.GetSolutionRef(),O=r.GetAppVersionMajor(),h=r.GetAppVersionMinor(),D=r.GetAppUILocale(),v=r.GetAppDataLocale(),A=r.GetDocUrl(),y=r.GetAppCapabilities(),F=r.GetActivationMode(),C=r.GetControlIntegrationLevel();try{i=r.GetSolutionToken()}catch(e){}if(OSF.OUtil.externalNativeFunctionExists(typeof r.GetCorrelationId)&&(a=r.GetCorrelationId()),OSF.OUtil.externalNativeFunctionExists(typeof r.GetInstanceId)&&(s=r.GetInstanceId()),OSF.OUtil.externalNativeFunctionExists(typeof r.GetTouchEnabled)&&(l=r.GetTouchEnabled()),OSF.OUtil.externalNativeFunctionExists(typeof r.GetCommerceAllowed)&&(c=r.GetCommerceAllowed()),OSF.OUtil.externalNativeFunctionExists(typeof r.GetSupportedMatrix)&&(d=r.GetSupportedMatrix()),OSF.OUtil.externalNativeFunctionExists(typeof r.GetHostCustomMessage)&&(u=r.GetHostCustomMessage()),OSF.OUtil.externalNativeFunctionExists(typeof r.GetHostFullVersion)&&(f=r.GetHostFullVersion()),OSF.OUtil.externalNativeFunctionExists(typeof r.GetDialogRequirementMatrix)&&(p=r.GetDialogRequirementMatrix()),OSF.OUtil.externalNativeFunctionExists(typeof r.GetFeaturesForSolution))try{var M=r.GetFeaturesForSolution();M&&(g=JSON.parse(M))}catch(e){OsfMsAjaxFactory.msAjaxDebug.trace("Exception while creating the SDX FeatureGates object. Details: "+e)}var E=0;OSF.OUtil.externalNativeFunctionExists(typeof r.GetInitialDisplayMode)&&(E=r.GetInitialDisplayMode()),i=i?i.toString():"",n=new OSF.OfficeAppContext(S,m,O,D,v,A,y,[],F,C,i,a,s,l,c,h,d,u,f,void 0,void 0,void 0,void 0,p,g,void 0,E),OSF.AppTelemetry&&OSF.AppTelemetry.initialize(n),t(n)}},window.webkit&&window.webkit.messageHandlers&&window.webkit.messageHandlers.Agave?(OSF.ClientHostController=new OSFWebkit.WebkitHostController(OSF.ScriptMessaging.GetScriptMessenger()),OSF.initializeWebkitCommon(),OSF.InitializationHelper.prototype.initializeMacSettings=function(e,t){return this._initializeSettings(e,t)}):(OSF.ClientHostController=new OfficeExt.MacRichClientHostController,OSF.initializeRichCommon(),OSF.InitializationHelper.prototype.initializeMacSettings=function(e,t){return this._initializeSettings(t)}),function(e){var t=function(){function e(e){this._table=e,this._fields={}}return Object.defineProperty(e.prototype,"Fields",{get:function(){return this._fields},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"Table",{get:function(){return this._table},enumerable:!0,configurable:!0}),e.prototype.SerializeFields=function(){},e.prototype.SetSerializedField=function(e,t){null!=t&&(this._serializedFields[e]=t.toString())},e.prototype.SerializeRow=function(){return this._serializedFields={},this.SetSerializedField("Table",this._table),this.SerializeFields(),JSON.stringify(this._serializedFields)},e}();e.BaseUsageData=t;var n=function(e){function t(){return e.call(this,"AppActivated")||this}return __extends(t,e),Object.defineProperty(t.prototype,"CorrelationId",{get:function(){return this.Fields.CorrelationId},set:function(e){this.Fields.CorrelationId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"SessionId",{get:function(){return this.Fields.SessionId},set:function(e){this.Fields.SessionId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"AppId",{get:function(){return this.Fields.AppId},set:function(e){this.Fields.AppId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"AppInstanceId",{get:function(){return this.Fields.AppInstanceId},set:function(e){this.Fields.AppInstanceId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"AppURL",{get:function(){return this.Fields.AppURL},set:function(e){this.Fields.AppURL=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"AssetId",{get:function(){return this.Fields.AssetId},set:function(e){this.Fields.AssetId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"Browser",{get:function(){return this.Fields.Browser},set:function(e){this.Fields.Browser=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"UserId",{get:function(){return this.Fields.UserId},set:function(e){this.Fields.UserId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"Host",{get:function(){return this.Fields.Host},set:function(e){this.Fields.Host=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"HostVersion",{get:function(){return this.Fields.HostVersion},set:function(e){this.Fields.HostVersion=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"ClientId",{get:function(){return this.Fields.ClientId},set:function(e){this.Fields.ClientId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"AppSizeWidth",{get:function(){return this.Fields.AppSizeWidth},set:function(e){this.Fields.AppSizeWidth=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"AppSizeHeight",{get:function(){return this.Fields.AppSizeHeight},set:function(e){this.Fields.AppSizeHeight=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"Message",{get:function(){return this.Fields.Message},set:function(e){this.Fields.Message=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"DocUrl",{get:function(){return this.Fields.DocUrl},set:function(e){this.Fields.DocUrl=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"OfficeJSVersion",{get:function(){return this.Fields.OfficeJSVersion},set:function(e){this.Fields.OfficeJSVersion=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"HostJSVersion",{get:function(){return this.Fields.HostJSVersion},set:function(e){this.Fields.HostJSVersion=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"WacHostEnvironment",{get:function(){return this.Fields.WacHostEnvironment},set:function(e){this.Fields.WacHostEnvironment=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"IsFromWacAutomation",{get:function(){return this.Fields.IsFromWacAutomation},set:function(e){this.Fields.IsFromWacAutomation=e},enumerable:!0,configurable:!0}),t.prototype.SerializeFields=function(){this.SetSerializedField("CorrelationId",this.CorrelationId),this.SetSerializedField("SessionId",this.SessionId),this.SetSerializedField("AppId",this.AppId),this.SetSerializedField("AppInstanceId",this.AppInstanceId),this.SetSerializedField("AppURL",this.AppURL),this.SetSerializedField("AssetId",this.AssetId),this.SetSerializedField("Browser",this.Browser),this.SetSerializedField("UserId",this.UserId),this.SetSerializedField("Host",this.Host),this.SetSerializedField("HostVersion",this.HostVersion),this.SetSerializedField("ClientId",this.ClientId),this.SetSerializedField("AppSizeWidth",this.AppSizeWidth),this.SetSerializedField("AppSizeHeight",this.AppSizeHeight),this.SetSerializedField("Message",this.Message),this.SetSerializedField("DocUrl",this.DocUrl),this.SetSerializedField("OfficeJSVersion",this.OfficeJSVersion),this.SetSerializedField("HostJSVersion",this.HostJSVersion),this.SetSerializedField("WacHostEnvironment",this.WacHostEnvironment),this.SetSerializedField("IsFromWacAutomation",this.IsFromWacAutomation)},t}(t);e.AppActivatedUsageData=n;var r=function(e){function t(){return e.call(this,"ScriptLoad")||this}return __extends(t,e),Object.defineProperty(t.prototype,"CorrelationId",{get:function(){return this.Fields.CorrelationId},set:function(e){this.Fields.CorrelationId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"SessionId",{get:function(){return this.Fields.SessionId},set:function(e){this.Fields.SessionId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"ScriptId",{get:function(){return this.Fields.ScriptId},set:function(e){this.Fields.ScriptId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"StartTime",{get:function(){return this.Fields.StartTime},set:function(e){this.Fields.StartTime=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"ResponseTime",{get:function(){return this.Fields.ResponseTime},set:function(e){this.Fields.ResponseTime=e},enumerable:!0,configurable:!0}),t.prototype.SerializeFields=function(){this.SetSerializedField("CorrelationId",this.CorrelationId),this.SetSerializedField("SessionId",this.SessionId),this.SetSerializedField("ScriptId",this.ScriptId),this.SetSerializedField("StartTime",this.StartTime),this.SetSerializedField("ResponseTime",this.ResponseTime)},t}(t);e.ScriptLoadUsageData=r;var o=function(e){function t(){return e.call(this,"AppClosed")||this}return __extends(t,e),Object.defineProperty(t.prototype,"CorrelationId",{get:function(){return this.Fields.CorrelationId},set:function(e){this.Fields.CorrelationId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"SessionId",{get:function(){return this.Fields.SessionId},set:function(e){this.Fields.SessionId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"FocusTime",{get:function(){return this.Fields.FocusTime},set:function(e){this.Fields.FocusTime=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"AppSizeFinalWidth",{get:function(){return this.Fields.AppSizeFinalWidth},set:function(e){this.Fields.AppSizeFinalWidth=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"AppSizeFinalHeight",{get:function(){return this.Fields.AppSizeFinalHeight},set:function(e){this.Fields.AppSizeFinalHeight=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"OpenTime",{get:function(){return this.Fields.OpenTime},set:function(e){this.Fields.OpenTime=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"CloseMethod",{get:function(){return this.Fields.CloseMethod},set:function(e){this.Fields.CloseMethod=e},enumerable:!0,configurable:!0}),t.prototype.SerializeFields=function(){this.SetSerializedField("CorrelationId",this.CorrelationId),this.SetSerializedField("SessionId",this.SessionId),this.SetSerializedField("FocusTime",this.FocusTime),this.SetSerializedField("AppSizeFinalWidth",this.AppSizeFinalWidth),this.SetSerializedField("AppSizeFinalHeight",this.AppSizeFinalHeight),this.SetSerializedField("OpenTime",this.OpenTime),this.SetSerializedField("CloseMethod",this.CloseMethod)},t}(t);e.AppClosedUsageData=o;var i=function(e){function t(){return e.call(this,"APIUsage")||this}return __extends(t,e),Object.defineProperty(t.prototype,"CorrelationId",{get:function(){return this.Fields.CorrelationId},set:function(e){this.Fields.CorrelationId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"SessionId",{get:function(){return this.Fields.SessionId},set:function(e){this.Fields.SessionId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"APIType",{get:function(){return this.Fields.APIType},set:function(e){this.Fields.APIType=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"APIID",{get:function(){return this.Fields.APIID},set:function(e){this.Fields.APIID=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"Parameters",{get:function(){return this.Fields.Parameters},set:function(e){this.Fields.Parameters=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"ResponseTime",{get:function(){return this.Fields.ResponseTime},set:function(e){this.Fields.ResponseTime=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"ErrorType",{get:function(){return this.Fields.ErrorType},set:function(e){this.Fields.ErrorType=e},enumerable:!0,configurable:!0}),t.prototype.SerializeFields=function(){this.SetSerializedField("CorrelationId",this.CorrelationId),this.SetSerializedField("SessionId",this.SessionId),this.SetSerializedField("APIType",this.APIType),this.SetSerializedField("APIID",this.APIID),this.SetSerializedField("Parameters",this.Parameters),this.SetSerializedField("ResponseTime",this.ResponseTime),this.SetSerializedField("ErrorType",this.ErrorType)},t}(t);e.APIUsageUsageData=i;var a=function(e){function t(){return e.call(this,"AppInitialization")||this}return __extends(t,e),Object.defineProperty(t.prototype,"CorrelationId",{get:function(){return this.Fields.CorrelationId},set:function(e){this.Fields.CorrelationId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"SessionId",{get:function(){return this.Fields.SessionId},set:function(e){this.Fields.SessionId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"SuccessCode",{get:function(){return this.Fields.SuccessCode},set:function(e){this.Fields.SuccessCode=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"Message",{get:function(){return this.Fields.Message},set:function(e){this.Fields.Message=e},enumerable:!0,configurable:!0}),t.prototype.SerializeFields=function(){this.SetSerializedField("CorrelationId",this.CorrelationId),this.SetSerializedField("SessionId",this.SessionId),this.SetSerializedField("SuccessCode",this.SuccessCode),this.SetSerializedField("Message",this.Message)},t}(t);e.AppInitializationUsageData=a;var s=function(e){function t(){return e.call(this,"CheckWACHost")||this}return __extends(t,e),Object.defineProperty(t.prototype,"isWacKnownHost",{get:function(){return this.Fields.isWacKnownHost},set:function(e){this.Fields.isWacKnownHost=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"instanceId",{get:function(){return this.Fields.instanceId},set:function(e){this.Fields.instanceId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"hostType",{get:function(){return this.Fields.hostType},set:function(e){this.Fields.hostType=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"hostPlatform",{get:function(){return this.Fields.hostPlatform},set:function(e){this.Fields.hostPlatform=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"wacDomain",{get:function(){return this.Fields.wacDomain},set:function(e){this.Fields.wacDomain=e},enumerable:!0,configurable:!0}),t.prototype.SerializeFields=function(){this.SetSerializedField("isWacKnownHost",this.isWacKnownHost),this.SetSerializedField("instanceId",this.instanceId),this.SetSerializedField("hostType",this.hostType),this.SetSerializedField("hostPlatform",this.hostPlatform),this.SetSerializedField("wacDomain",this.wacDomain)},t}(t);e.CheckWACHostUsageData=s}(OSFLog||(OSFLog={})),function(e){"use strict";!function(e){e[e.info=0]="info",e[e.warning=1]="warning",e[e.error=2]="error"}(e.TraceLevel||(e.TraceLevel={})),function(e){e[e.none=0]="none",e[e.flush=1]="flush"}(e.SendFlag||(e.SendFlag={})),e.allowUploadingData=function(){},e.sendLog=function(e,t,n){};var t=function(){function e(){}return e.prototype.writeLog=function(e){},e.prototype.loadProxyFrame=function(){},e}();OSF.Logger||(OSF.Logger=e),e.ulsEndpoint=function(){try{return new t}catch(e){return null}}()}(Logger||(Logger={})),function(e){var t=[{name:"AppActivated",enabled:!0,critical:!0,points:[{name:"Browser",type:"string"},{name:"Message",type:"string"},{name:"Host",type:"string"},{name:"AppSizeWidth",type:"int64"},{name:"AppSizeHeight",type:"int64"},{name:"IsFromWacAutomation",type:"string"}]},{name:"ScriptLoad",enabled:!0,critical:!1,points:[{name:"ScriptId",type:"string"},{name:"StartTime",type:"double"},{name:"ResponseTime",type:"double"}]},{name:"APIUsage",enabled:function(){if(!OSF._OfficeAppFactory||!OSF._OfficeAppFactory.getHostInfo)return!1;var e=OSF._OfficeAppFactory.getHostInfo();if(!e)return!1;switch(e.hostType){case"outlook":switch(e.hostPlatform){case"mac":case"web":return!0;default:return!1}default:return!1}}(),critical:!1,points:[{name:"APIType",type:"string"},{name:"APIID",type:"int64"},{name:"Parameters",type:"string"},{name:"ResponseTime",type:"int64"},{name:"ErrorType",type:"int64"}]},{name:"AppInitialization",enabled:!0,critical:!1,points:[{name:"SuccessCode",type:"int64"},{name:"Message",type:"string"}]},{name:"AppClosed",enabled:!0,critical:!1,points:[{name:"FocusTime",type:"int64"},{name:"AppSizeFinalWidth",type:"int64"},{name:"AppSizeFinalHeight",type:"int64"},{name:"OpenTime",type:"int64"}]},{name:"CheckWACHost",enabled:!0,critical:!1,points:[{name:"isWacKnownHost",type:"int64"},{name:"solutionId",type:"string"},{name:"hostType",type:"string"},{name:"hostPlatform",type:"string"},{name:"correlationId",type:"string"}]}];function n(e,t){var n=void 0===t.rename?t.name:t.rename,r=void 0;switch(t.type){case"string":r=oteljs.makeStringDataField(n,e);break;case"double":"string"==typeof e&&(e=parseFloat(e)),r=oteljs.makeDoubleDataField(n,e);break;case"int64":"string"==typeof e&&(e=parseInt(e)),r=oteljs.makeInt64DataField(n,e);break;case"boolean":"string"==typeof e&&(e="true"===e),r=oteljs.makeBooleanDataField(n,e)}return r}function r(e){for(var n=0,r=t;n<r.length;n++){var o=r[n];if(o.name===e)return o}}function o(e,t){(function(e){var t=r(e);return void 0!==t&&t.enabled})(e)&&"undefined"!=typeof OTel&&OTel.OTelLogger.onTelemetryLoaded((function(){var o=function(e,t){var o=r(e);if(void 0!==o){for(var i=[],a=0,s=o.points;a<s.length;a++){var l=s[a],c=t[l.name];if(void 0!==c){var d=n(c,l);void 0!==d&&i.push(d)}}var u={dataCategories:oteljs.DataCategories.ProductServiceUsage};return o.critical&&(u.samplingPolicy=oteljs.SamplingPolicy.CriticalBusinessImpact),u.diagnosticLevel=oteljs.DiagnosticLevel.NecessaryServiceDataEvent,{eventName:"Office.Extensibility.OfficeJs."+e+"X",dataFields:i,eventFlags:u}}}(e,t);void 0!==o&&Microsoft.Office.WebExtension.sendTelemetryEvent(o)}))}var i=function(){function e(){}return e.prototype.getAriaCDNLocation=function(){return OSF._OfficeAppFactory.getLoadScriptHelper().getOfficeJsBasePath()+"ariatelemetry/aria-web-telemetry.js"},e.getInstance=function(){return void 0===e.AriaLoggerObj&&(e.AriaLoggerObj=new e),e.AriaLoggerObj},e.prototype.isIUsageData=function(e){return void 0!==e.Fields},e.prototype.shouldSendDirectToAria=function(e,t){var n;if(!e)return!1;if("win32"===e.toLowerCase())n=[16,0,11601];else{if("mac"!==e.toLowerCase())return!0;n=[16,28]}if(!t)return!1;for(var r=t.split("."),o=0;o<n.length&&o<r.length;o++){var i=parseInt(r[o],10);if(isNaN(i))return!1;if(i<n[o])return!0;if(i>n[o])return!1}return!1},e.prototype.isDirectToAriaEnabled=function(){if(void 0===this.EnableDirectToAria||null===this.EnableDirectToAria){var e=void 0,t=void 0;OSF._OfficeAppFactory&&OSF._OfficeAppFactory.getHostInfo&&(e=OSF._OfficeAppFactory.getHostInfo().hostPlatform),window.external&&void 0!==window.external.GetContext&&void 0!==window.external.GetContext().GetHostFullVersion&&(t=window.external.GetContext().GetHostFullVersion()),this.EnableDirectToAria=this.shouldSendDirectToAria(e,t)}return this.EnableDirectToAria},e.prototype.sendTelemetry=function(t,n){e.EnableSendingTelemetryWithLegacyAria&&this.isDirectToAriaEnabled()&&OSF.OUtil.loadScript(this.getAriaCDNLocation(),(function(){try{if(!this.ALogger){this.ALogger=AWTLogManager.initialize("db334b301e7b474db5e0f02f07c51a47-a1b5bc36-1bbe-482f-a64a-c2d9cb606706-7439")}var e=new AWTEventProperties;for(var r in e.setName("Office.Extensibility.OfficeJS."+t),n)"table"!==r.toLowerCase()&&e.setProperty(r,n[r]);var o=new Date;e.setProperty("Date",o.toISOString()),this.ALogger.logEvent(e)}catch(e){}}),1e3),e.EnableSendingTelemetryWithOTel&&o(t,n)},e.prototype.logData=function(e){this.isIUsageData(e)?this.sendTelemetry(e.Table,e.Fields):this.sendTelemetry(e.Table,e)},e.EnableSendingTelemetryWithOTel=!0,e.EnableSendingTelemetryWithLegacyAria=!1,e}();e.AriaLogger=i}(OSFAriaLogger||(OSFAriaLogger={})),function(e){"use strict";var t,n=OSF.OUtil.Guid.generateNewGuid(),r="",o=new RegExp("^https?://store\\.office(ppe|-int)?\\.com/","i"),i="PRIVATE";e.enableTelemetry=!0;var a=function(){};e.AppInfo=a;var s=function(e,t){this.name=e,this.handler=t},l=function(){function e(){this.clientIDKey="Office API client",this.logIdSetKey="Office App Log Id Set"}return e.prototype.getClientId=function(){var e=this.getValue(this.clientIDKey);return(!e||e.length<=0||e.length>40)&&(e=OSF.OUtil.Guid.generateNewGuid(),this.setValue(this.clientIDKey,e)),e},e.prototype.saveLog=function(e,t){var n=this.getValue(this.logIdSetKey);n=(n&&n.length>0?n+";":"")+e,this.setValue(this.logIdSetKey,n),this.setValue(e,t)},e.prototype.enumerateLog=function(e,t){var n=this.getValue(this.logIdSetKey);if(n){var r=n.split(";");for(var o in r){var i=r[o],a=this.getValue(i);a&&(e&&e(i,a),t&&this.remove(i))}t&&this.remove(this.logIdSetKey)}},e.prototype.getValue=function(e){var t=OSF.OUtil.getLocalStorage(),n="";return t&&(n=t.getItem(e)),n},e.prototype.setValue=function(e,t){var n=OSF.OUtil.getLocalStorage();n&&n.setItem(e,t)},e.prototype.remove=function(e){var t=OSF.OUtil.getLocalStorage();if(t)try{t.removeItem(e)}catch(e){}},e}(),c=function(){function t(){}return t.prototype.LogData=function(t){if(e.enableTelemetry)try{OSFAriaLogger.AriaLogger.getInstance().logData(t)}catch(e){}},t.prototype.LogRawData=function(t){if(e.enableTelemetry)try{OSFAriaLogger.AriaLogger.getInstance().logData(JSON.parse(t))}catch(e){}},t}();function d(e){return e&&(e=e.replace(/[{}]/g,"").toLowerCase()),e||""}function u(e,t){var o=new OSFLog.AppInitializationUsageData;o.CorrelationId=d(r),o.SessionId=n,o.SuccessCode=e?1:0,o.Message=t,(new c).LogData(o)}function f(){var e=0!=(OSF._OfficeAppFactory.getHostInfo().flags&OSF.HostInfoFlags.PublicAddin);return e||!!t&&g(OSF._OfficeAppFactory.getHostInfo().hostPlatform,t.hostVersion)}function p(e,t){return f()||t!==e?t:i}function g(e,t){var n=!1,r=/^(\d+)\.(\d+)\.(\d+)\.(\d+)$/.exec(t);if(r){var o=parseInt(r[1]),i=parseInt(r[2]),a=parseInt(r[3]);"win32"==e?(o<16||16==o&&a<14225)&&(n=!0):"mac"==e&&(o<16||16==o&&(i<52||52==i&&a<808))&&(n=!0)}return n}e.initialize=function(r){if(e.enableTelemetry&&!t){t=new a,r.get_hostFullVersion()?t.hostVersion=r.get_hostFullVersion():t.hostVersion=r.get_appVersion(),t.appId=f()?r.get_id():i,t.marketplaceType=r._marketplaceType,t.browser=window.navigator.userAgent,t.correlationId=d(r.get_correlationId()),t.clientId=(new l).getClientId(),t.appInstanceId=r.get_appInstanceId(),t.appInstanceId&&(t.appInstanceId=d(t.appInstanceId),t.appInstanceId=p(r.get_id(),t.appInstanceId)),t.message=r.get_hostCustomMessage(),t.officeJSVersion=OSF.ConstantNames.FileVersion,t.hostJSVersion="16.0.15407.10000",r._wacHostEnvironment&&(t.wacHostEnvironment=r._wacHostEnvironment),void 0!==r._isFromWacAutomation&&null!==r._isFromWacAutomation&&(t.isFromWacAutomation=r._isFromWacAutomation.toString().toLowerCase());var c=r.get_docUrl();t.docUrl=o.test(c)?c:"";var u=location.href;u&&(u=u.split("?")[0].split("#")[0]),t.appURL="",function(e,t){var n,r;t.assetId="",t.userId="";try{n=decodeURIComponent(e);var o=(r=(new DOMParser).parseFromString(n,"text/xml")).getElementsByTagName("t")[0].attributes.getNamedItem("cid"),i=r.getElementsByTagName("t")[0].attributes.getNamedItem("oid");o&&o.nodeValue?t.userId=o.nodeValue:i&&i.nodeValue&&(t.userId=i.nodeValue),t.assetId=r.getElementsByTagName("t")[0].attributes.getNamedItem("aid").nodeValue}catch(e){}finally{n=null,r=null,null}}(r.get_eToken(),t),t.sessionId=n,"undefined"!=typeof OTel&&OTel.OTelLogger.initialize(t),function(){var t=new Date,n=null,r=0,o=!1,i=function(){document.hasFocus()?null==n&&(n=new Date):n&&(r+=Math.abs((new Date).getTime()-n.getTime()),n=null)},a=[];a.push(new s("focus",i)),a.push(new s("blur",i)),a.push(new s("focusout",i)),a.push(new s("focusin",i));var l=function(){for(var i=0;i<a.length;i++)OSF.OUtil.removeEventListener(window,a[i].name,a[i].handler);a.length=0,o||(document.hasFocus()&&n&&(r+=Math.abs((new Date).getTime()-n.getTime()),n=null),e.onAppClosed(Math.abs((new Date).getTime()-t.getTime()),r),o=!0)};a.push(new s("beforeunload",l)),a.push(new s("unload",l));for(var c=0;c<a.length;c++)OSF.OUtil.addEventListener(window,a[c].name,a[c].handler);i()}(),e.onAppActivated()}},e.onAppActivated=function(){if(t){(new l).enumerateLog((function(e,t){return(new c).LogRawData(t)}),!0);var e=new OSFLog.AppActivatedUsageData;e.SessionId=n,e.AppId=t.appId,e.AssetId=t.assetId,e.AppURL="",e.UserId="",e.ClientId=t.clientId,e.Browser=t.browser,e.HostVersion=t.hostVersion,e.CorrelationId=d(t.correlationId),e.AppSizeWidth=window.innerWidth,e.AppSizeHeight=window.innerHeight,e.AppInstanceId=t.appInstanceId,e.Message=t.message,e.DocUrl=t.docUrl,e.OfficeJSVersion=t.officeJSVersion,e.HostJSVersion=t.hostJSVersion,t.wacHostEnvironment&&(e.WacHostEnvironment=t.wacHostEnvironment),void 0!==t.isFromWacAutomation&&null!==t.isFromWacAutomation&&(e.IsFromWacAutomation=t.isFromWacAutomation),(new c).LogData(e)}},e.onScriptDone=function(e,t,r,o){var i=new OSFLog.ScriptLoadUsageData;i.CorrelationId=d(o),i.SessionId=n,i.ScriptId=e,i.StartTime=t,i.ResponseTime=r,(new c).LogData(i)},e.onCallDone=function(e,o,i,a,s){if(t&&function(){if(!OSF._OfficeAppFactory||!OSF._OfficeAppFactory.getHostInfo)return!1;var e=OSF._OfficeAppFactory.getHostInfo();if(!e)return!1;switch(e.hostType){case"outlook":switch(e.hostPlatform){case"mac":case"web":return!0;default:return!1}default:return!1}}()&&function(e,t){if("method"===t)switch(e){case 3:case 4:case 38:case 37:case 10:case 12:return!0;default:return!1}return!1}(o,e)){var l=new OSFLog.APIUsageUsageData;l.CorrelationId=d(r),l.SessionId=n,l.APIType=e,l.APIID=o,l.Parameters=i,l.ResponseTime=a,l.ErrorType=s,(new c).LogData(l)}},e.onMethodDone=function(e,t,n,r){var o=null;if(t)if("number"==typeof t)o=String(t);else if("object"==typeof t)for(var i in t)null!==o?o+=",":o="","number"==typeof t[i]&&(o+=String(t[i]));else o="";OSF.AppTelemetry.onCallDone("method",e,o,n,r)},e.onPropertyDone=function(e,t){OSF.AppTelemetry.onCallDone("property",-1,e,t)},e.onCheckWACHost=function(e,t,n,r,o){var i=new OSFLog.CheckWACHostUsageData;i.isWacKnownHost=e,i.instanceId=t,i.hostType=n,i.hostPlatform=r,i.wacDomain="",(new c).LogData(i)},e.onEventDone=function(e,t){OSF.AppTelemetry.onCallDone("event",e,null,0,t)},e.onRegisterDone=function(e,t,n,r){OSF.AppTelemetry.onCallDone(e?"registerevent":"unregisterevent",t,null,n,r)},e.onAppClosed=function(e,o){if(t){var i=new OSFLog.AppClosedUsageData;i.CorrelationId=d(r),i.SessionId=n,i.FocusTime=o,i.OpenTime=e,i.AppSizeFinalWidth=window.innerWidth,i.AppSizeFinalHeight=window.innerHeight,(new l).saveLog(n,i.SerializeRow())}},e.setOsfControlAppCorrelationId=function(e){r=d(e)},e.doAppInitializationLogging=u,e.logAppCommonMessage=function(e){u(!1,e)},e.logAppException=function(e){u(!0,e)},e.canSendAddinId=f,e.getCompliantAppInstanceId=p,e._isComplianceExceptedHost=g,OSF.AppTelemetry=e}(OSFAppTelemetry||(OSFAppTelemetry={})),OSF.InitializationHelper.prototype.loadAppSpecificScriptAndCreateOM=function(e,t,n){OSF.DDA.ErrorCodeManager.initializeErrorMessages(Strings.OfficeOM),OSF.DDA.DispIdHost.addAsyncMethods(OSF.DDA.RichApi,[OSF.DDA.AsyncMethodNames.ExecuteRichApiRequestAsync]),OSF.DDA.RichApi.richApiMessageManager=new OfficeExt.RichApiMessageManager,t()},OSF.DDA.AsyncMethodNames.addNames({ExecuteRichApiRequestAsync:"executeRichApiRequestAsync"}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.ExecuteRichApiRequestAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Data,types:["object"]}],supportedOptions:[]}),OSF.OUtil.setNamespace("RichApi",OSF.DDA),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidExecuteRichApiRequestMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Data,value:0}],fromHost:[{name:Microsoft.Office.WebExtension.Parameters.Data,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}]}),Microsoft.Office.WebExtension.EventType={},OSF.EventDispatch=function(e){if(this._eventHandlers={},this._objectEventHandlers={},this._queuedEventsArgs={},null!=e)for(var t=0;t<e.length;t++){var n=e[t];"objectDeleted"==n||"objectSelectionChanged"==n||"objectDataChanged"==n||"contentControlAdded"==n?this._objectEventHandlers[n]={}:this._eventHandlers[n]=[],this._queuedEventsArgs[n]=[]}},OSF.EventDispatch.prototype={getSupportedEvents:function(){var e=[];for(var t in this._eventHandlers)e.push(t);for(var t in this._objectEventHandlers)e.push(t);return e},supportsEvent:function(e){for(var t in this._eventHandlers)if(e==t)return!0;for(var t in this._objectEventHandlers)if(e==t)return!0;return!1},hasEventHandler:function(e,t){var n=this._eventHandlers[e];if(n&&n.length>0)for(var r=0;r<n.length;r++)if(n[r]===t)return!0;return!1},hasObjectEventHandler:function(e,t,n){var r=this._objectEventHandlers[e];if(null!=r)for(var o=r[t],i=0;null!=o&&i<o.length;i++)if(o[i]===n)return!0;return!1},addEventHandler:function(e,t){if("function"!=typeof t)return!1;var n=this._eventHandlers[e];return!(!n||this.hasEventHandler(e,t))&&(n.push(t),!0)},addObjectEventHandler:function(e,t,n){if("function"!=typeof n)return!1;var r=this._objectEventHandlers[e];return!(!r||this.hasObjectEventHandler(e,t,n))&&(null==r[t]&&(r[t]=[]),r[t].push(n),!0)},addEventHandlerAndFireQueuedEvent:function(e,t){var n=0==this._eventHandlers[e].length,r=this.addEventHandler(e,t);return n&&r&&this.fireQueuedEvent(e),r},removeEventHandler:function(e,t){var n=this._eventHandlers[e];if(n&&n.length>0)for(var r=0;r<n.length;r++)if(n[r]===t)return n.splice(r,1),!0;return!1},removeObjectEventHandler:function(e,t,n){var r=this._objectEventHandlers[e];if(null!=r)for(var o=r[t],i=0;null!=o&&i<o.length;i++)if(o[i]===n)return o.splice(i,1),!0;return!1},clearEventHandlers:function(e){return void 0!==this._eventHandlers[e]&&this._eventHandlers[e].length>0&&(this._eventHandlers[e]=[],!0)},clearObjectEventHandlers:function(e,t){return null!=this._objectEventHandlers[e]&&null!=this._objectEventHandlers[e][t]&&(this._objectEventHandlers[e][t]=[],!0)},getEventHandlerCount:function(e){return null!=this._eventHandlers[e]?this._eventHandlers[e].length:-1},getObjectEventHandlerCount:function(e,t){return null==this._objectEventHandlers[e]||null==this._objectEventHandlers[e][t]?0:this._objectEventHandlers[e][t].length},fireEvent:function(e){if(null==e.type)return!1;var t=e.type;if(t&&this._eventHandlers[t]){for(var n=this._eventHandlers[t],r=0;r<n.length;r++)n[r](e);return!0}return!1},fireObjectEvent:function(e,t){if(null==t.type)return!1;var n=t.type;if(n&&this._objectEventHandlers[n]){var r=this._objectEventHandlers[n][e];if(null!=r){for(var o=0;o<r.length;o++)r[o](t);return!0}}return!1},fireOrQueueEvent:function(e){var t=e.type;if(t&&this._eventHandlers[t]){var n=this._eventHandlers[t],r=this._queuedEventsArgs[t];return 0==n.length?r.push(e):this.fireEvent(e),!0}return!1},fireQueuedEvent:function(e){if(e&&this._eventHandlers[e]){var t=this._eventHandlers[e],n=this._queuedEventsArgs[e];if(t.length>0){for(var r=t[0];n.length>0;){r(n.shift())}return!0}}return!1},clearQueuedEvent:function(e){e&&this._eventHandlers[e]&&(this._queuedEventsArgs[e]&&(this._queuedEventsArgs[e]=[]))}},OSF.DDA.OMFactory=OSF.DDA.OMFactory||{},OSF.DDA.OMFactory.manufactureEventArgs=function(e,t,n){var r;switch(e){case Microsoft.Office.WebExtension.EventType.DocumentSelectionChanged:r=new OSF.DDA.DocumentSelectionChangedEventArgs(t);break;case Microsoft.Office.WebExtension.EventType.BindingSelectionChanged:r=new OSF.DDA.BindingSelectionChangedEventArgs(this.manufactureBinding(n,t.document),n[OSF.DDA.PropertyDescriptors.Subset]);break;case Microsoft.Office.WebExtension.EventType.BindingDataChanged:r=new OSF.DDA.BindingDataChangedEventArgs(this.manufactureBinding(n,t.document));break;case Microsoft.Office.WebExtension.EventType.SettingsChanged:r=new OSF.DDA.SettingsChangedEventArgs(t);break;case Microsoft.Office.WebExtension.EventType.ActiveViewChanged:r=new OSF.DDA.ActiveViewChangedEventArgs(n);break;case Microsoft.Office.WebExtension.EventType.OfficeThemeChanged:r=new OSF.DDA.Theming.OfficeThemeChangedEventArgs(n);break;case Microsoft.Office.WebExtension.EventType.DocumentThemeChanged:r=new OSF.DDA.Theming.DocumentThemeChangedEventArgs(n);break;case Microsoft.Office.WebExtension.EventType.AppCommandInvoked:r=OSF.DDA.AppCommand.AppCommandInvokedEventArgs.create(n),"outlook"==OSF._OfficeAppFactory.getHostInfo().hostType&&"mac"==OSF._OfficeAppFactory.getHostInfo().hostPlatform&&OSF.DDA.convertOlkAppointmentTimeToDateFormat(r);break;case Microsoft.Office.WebExtension.EventType.ObjectDeleted:case Microsoft.Office.WebExtension.EventType.ObjectSelectionChanged:case Microsoft.Office.WebExtension.EventType.ObjectDataChanged:case Microsoft.Office.WebExtension.EventType.ContentControlAdded:r=new OSF.DDA.ObjectEventArgs(e,n[Microsoft.Office.WebExtension.Parameters.Id]);break;case Microsoft.Office.WebExtension.EventType.RichApiMessage:r=new OSF.DDA.RichApiMessageEventArgs(e,n);break;case Microsoft.Office.WebExtension.EventType.DataNodeInserted:r=new OSF.DDA.NodeInsertedEventArgs(this.manufactureDataNode(n[OSF.DDA.DataNodeEventProperties.NewNode]),n[OSF.DDA.DataNodeEventProperties.InUndoRedo]);break;case Microsoft.Office.WebExtension.EventType.DataNodeReplaced:r=new OSF.DDA.NodeReplacedEventArgs(this.manufactureDataNode(n[OSF.DDA.DataNodeEventProperties.OldNode]),this.manufactureDataNode(n[OSF.DDA.DataNodeEventProperties.NewNode]),n[OSF.DDA.DataNodeEventProperties.InUndoRedo]);break;case Microsoft.Office.WebExtension.EventType.DataNodeDeleted:r=new OSF.DDA.NodeDeletedEventArgs(this.manufactureDataNode(n[OSF.DDA.DataNodeEventProperties.OldNode]),this.manufactureDataNode(n[OSF.DDA.DataNodeEventProperties.NextSiblingNode]),n[OSF.DDA.DataNodeEventProperties.InUndoRedo]);break;case Microsoft.Office.WebExtension.EventType.TaskSelectionChanged:r=new OSF.DDA.TaskSelectionChangedEventArgs(t);break;case Microsoft.Office.WebExtension.EventType.ResourceSelectionChanged:r=new OSF.DDA.ResourceSelectionChangedEventArgs(t);break;case Microsoft.Office.WebExtension.EventType.ViewSelectionChanged:r=new OSF.DDA.ViewSelectionChangedEventArgs(t);break;case Microsoft.Office.WebExtension.EventType.DialogMessageReceived:r=new OSF.DDA.DialogEventArgs(n);break;case Microsoft.Office.WebExtension.EventType.DialogParentMessageReceived:r=new OSF.DDA.DialogParentEventArgs(n);break;case Microsoft.Office.WebExtension.EventType.ItemChanged:if("outlook"!=OSF._OfficeAppFactory.getHostInfo().hostType)throw OsfMsAjaxFactory.msAjaxError.argument(Microsoft.Office.WebExtension.Parameters.EventType,OSF.OUtil.formatString(Strings.OfficeOM.L_NotSupportedEventType,e));r=new OSF.DDA.OlkItemSelectedChangedEventArgs(n),t.initialize(r.initialData),"win32"!=OSF._OfficeAppFactory.getHostInfo().hostPlatform&&"mac"!=OSF._OfficeAppFactory.getHostInfo().hostPlatform||t.setCurrentItemNumber(r.itemNumber.itemNumber);break;case Microsoft.Office.WebExtension.EventType.RecipientsChanged:if("outlook"!=OSF._OfficeAppFactory.getHostInfo().hostType)throw OsfMsAjaxFactory.msAjaxError.argument(Microsoft.Office.WebExtension.Parameters.EventType,OSF.OUtil.formatString(Strings.OfficeOM.L_NotSupportedEventType,e));r=new OSF.DDA.OlkRecipientsChangedEventArgs(n);break;case Microsoft.Office.WebExtension.EventType.AppointmentTimeChanged:if("outlook"!=OSF._OfficeAppFactory.getHostInfo().hostType)throw OsfMsAjaxFactory.msAjaxError.argument(Microsoft.Office.WebExtension.Parameters.EventType,OSF.OUtil.formatString(Strings.OfficeOM.L_NotSupportedEventType,e));r=new OSF.DDA.OlkAppointmentTimeChangedEventArgs(n);break;case Microsoft.Office.WebExtension.EventType.RecurrenceChanged:if("outlook"!=OSF._OfficeAppFactory.getHostInfo().hostType)throw OsfMsAjaxFactory.msAjaxError.argument(Microsoft.Office.WebExtension.Parameters.EventType,OSF.OUtil.formatString(Strings.OfficeOM.L_NotSupportedEventType,e));r=new OSF.DDA.OlkRecurrenceChangedEventArgs(n);break;case Microsoft.Office.WebExtension.EventType.AttachmentsChanged:if("outlook"!=OSF._OfficeAppFactory.getHostInfo().hostType)throw OsfMsAjaxFactory.msAjaxError.argument(Microsoft.Office.WebExtension.Parameters.EventType,OSF.OUtil.formatString(Strings.OfficeOM.L_NotSupportedEventType,e));r=new OSF.DDA.OlkAttachmentsChangedEventArgs(n);break;case Microsoft.Office.WebExtension.EventType.EnhancedLocationsChanged:if("outlook"!=OSF._OfficeAppFactory.getHostInfo().hostType)throw OsfMsAjaxFactory.msAjaxError.argument(Microsoft.Office.WebExtension.Parameters.EventType,OSF.OUtil.formatString(Strings.OfficeOM.L_NotSupportedEventType,e));r=new OSF.DDA.OlkEnhancedLocationsChangedEventArgs(n);break;case Microsoft.Office.WebExtension.EventType.InfobarClicked:if("outlook"!=OSF._OfficeAppFactory.getHostInfo().hostType)throw OsfMsAjaxFactory.msAjaxError.argument(Microsoft.Office.WebExtension.Parameters.EventType,OSF.OUtil.formatString(Strings.OfficeOM.L_NotSupportedEventType,e));r=new OSF.DDA.OlkInfobarClickedEventArgs(n);break;default:throw OsfMsAjaxFactory.msAjaxError.argument(Microsoft.Office.WebExtension.Parameters.EventType,OSF.OUtil.formatString(Strings.OfficeOM.L_NotSupportedEventType,e))}return r},OSF.DDA.AsyncMethodNames.addNames({AddHandlerAsync:"addHandlerAsync",RemoveHandlerAsync:"removeHandlerAsync"}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.AddHandlerAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.EventType,enum:Microsoft.Office.WebExtension.EventType,verify:function(e,t,n){return n.supportsEvent(e)}},{name:Microsoft.Office.WebExtension.Parameters.Handler,types:["function"]}],supportedOptions:[],privateStateCallbacks:[]}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.RemoveHandlerAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.EventType,enum:Microsoft.Office.WebExtension.EventType,verify:function(e,t,n){return n.supportsEvent(e)}}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.Handler,value:{types:["function","object"],defaultValue:null}}],privateStateCallbacks:[]}),OSF.OUtil.augmentList(Microsoft.Office.WebExtension.EventType,{RichApiMessage:"richApiMessage"}),OSF.DDA.RichApiMessageEventArgs=function(e,t){var n=t[Microsoft.Office.WebExtension.Parameters.Data],r=[];if(n)for(var o=0;o<n.length;o++){var i=n[o];i.toArray&&(i=i.toArray()),r.push({messageCategory:i[0],messageType:i[1],targetId:i[2],message:i[3],id:i[4],isRemoteOverride:i[5]})}OSF.OUtil.defineEnumerableProperties(this,{type:{value:Microsoft.Office.WebExtension.EventType.RichApiMessage},entries:{value:r}})},function(e){var t=function(){function e(){this._eventDispatch=null,this._registerHandlers=[],this._eventDispatch=new OSF.EventDispatch([Microsoft.Office.WebExtension.EventType.RichApiMessage]),OSF.DDA.DispIdHost.addEventSupport(this,this._eventDispatch)}return e.prototype.register=function(e){var t=this;return this._registerWithHostPromise||(this._registerWithHostPromise=new Office.Promise((function(e,n){t.addHandlerAsync(Microsoft.Office.WebExtension.EventType.RichApiMessage,(function(e){t._registerHandlers.forEach((function(t){t&&t(e)}))}),(function(t){"failed"==t.status?n(t.error):e()}))}))),this._registerWithHostPromise.then((function(){t._registerHandlers.push(e)}))},e}();e.RichApiMessageManager=t}(OfficeExt||(OfficeExt={})),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.EventDispId.dispidRichApiMessageEvent,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Data,value:0}],fromHost:[{name:Microsoft.Office.WebExtension.Parameters.Data,value:OSF.DDA.SafeArray.Delegate.ParameterMap.sourceData}]}),OSF.DialogShownStatus={hasDialogShown:!1,isWindowDialog:!1},OSF.OUtil.augmentList(OSF.DDA.EventDescriptors,{DialogMessageReceivedEvent:"DialogMessageReceivedEvent"}),OSF.OUtil.augmentList(Microsoft.Office.WebExtension.EventType,{DialogMessageReceived:"dialogMessageReceived",DialogEventReceived:"dialogEventReceived"}),OSF.OUtil.augmentList(OSF.DDA.PropertyDescriptors,{MessageType:"messageType",MessageContent:"messageContent",MessageOrigin:"messageOrigin"}),OSF.DDA.DialogEventType={},OSF.OUtil.augmentList(OSF.DDA.DialogEventType,{DialogClosed:"dialogClosed",NavigationFailed:"naviationFailed"}),OSF.DDA.AsyncMethodNames.addNames({DisplayDialogAsync:"displayDialogAsync",DisplayModalDialogAsync:"displayModalDialogAsync",CloseAsync:"close"}),OSF.DDA.SyncMethodNames.addNames({MessageParent:"messageParent",MessageChild:"messageChild",SendMessage:"sendMessage",AddMessageHandler:"addEventHandler"}),OSF.DDA.UI.ParentUI=function(){var e;if(e=null!=Microsoft.Office.WebExtension.EventType.DialogParentMessageReceived?new OSF.EventDispatch([Microsoft.Office.WebExtension.EventType.DialogMessageReceived,Microsoft.Office.WebExtension.EventType.DialogEventReceived,Microsoft.Office.WebExtension.EventType.DialogParentMessageReceived]):new OSF.EventDispatch([Microsoft.Office.WebExtension.EventType.DialogMessageReceived,Microsoft.Office.WebExtension.EventType.DialogEventReceived]),Microsoft.Office.WebExtension.FeatureGates&&Microsoft.Office.WebExtension.FeatureGates.OfficeJsDialogRefactor){var t=this,n=function(n,r){t[n]||OSF.OUtil.defineEnumerableProperty(t,n,{value:function(){var n=OSF._OfficeAppFactory.getHostFacade()[OSF.DDA.DispIdHost.Methods.OpenDialog];n(arguments,e,t,r)}})};n(OSF.DDA.AsyncMethodNames.DisplayDialogAsync.displayName,!1),Microsoft.Office.WebExtension.FeatureGates.ModalWebDialogAPI&&n(OSF.DDA.AsyncMethodNames.DisplayModalDialogAsync.displayName,!0)}else{var r=OSF.DDA.AsyncMethodNames.DisplayDialogAsync.displayName,o=this;o[r]||OSF.OUtil.defineEnumerableProperty(o,r,{value:function(){var t=OSF._OfficeAppFactory.getHostFacade()[OSF.DDA.DispIdHost.Methods.OpenDialog];t(arguments,e,o,!1)}})}OSF.OUtil.finalizeProperties(this)},OSF.DDA.UI.ChildUI=function(e){var t=OSF.DDA.SyncMethodNames.MessageParent.displayName,n=this;n[t]||OSF.OUtil.defineEnumerableProperty(n,t,{value:function(){var e=OSF._OfficeAppFactory.getHostFacade()[OSF.DDA.DispIdHost.Methods.MessageParent];return e(arguments,n)}});var r=OSF.DDA.SyncMethodNames.AddMessageHandler.displayName;n[r]||void 0===OSF.DialogParentMessageEventDispatch||OSF.DDA.DispIdHost.addEventSupport(n,OSF.DialogParentMessageEventDispatch,e),OSF.OUtil.finalizeProperties(this)},OSF.DialogHandler=function(){},OSF.DDA.DialogEventArgs=function(e){e[OSF.DDA.PropertyDescriptors.MessageType]==OSF.DialogMessageType.DialogMessageReceived?OSF.OUtil.defineEnumerableProperties(this,{type:{value:Microsoft.Office.WebExtension.EventType.DialogMessageReceived},message:{value:e[OSF.DDA.PropertyDescriptors.MessageContent]},origin:{value:e[OSF.DDA.PropertyDescriptors.MessageOrigin]}}):OSF.OUtil.defineEnumerableProperties(this,{type:{value:Microsoft.Office.WebExtension.EventType.DialogEventReceived},error:{value:e[OSF.DDA.PropertyDescriptors.MessageType]}})},OSF.DDA.DialogParentEventArgs=function(e){OSF.OUtil.defineEnumerableProperties(this,{type:{value:Microsoft.Office.WebExtension.EventType.DialogParentMessageReceived},message:{value:e[OSF.DDA.PropertyDescriptors.MessageContent]},origin:{value:e[OSF.DDA.PropertyDescriptors.MessageOrigin]}})};var DialogApiManager=function(){function e(){}return e.defineApi=function(e,t){OSF.DDA.AsyncMethodCalls.define({method:e,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Url,types:["string"]}],supportedOptions:t,privateStateCallbacks:[],onSucceeded:function(e,t,n){var r=e[Microsoft.Office.WebExtension.Parameters.Id],o=e[Microsoft.Office.WebExtension.Parameters.Data],i=new OSF.DialogHandler,a=OSF.DDA.AsyncMethodNames.CloseAsync.displayName;OSF.OUtil.defineEnumerableProperty(i,a,{value:function(){var e=OSF._OfficeAppFactory.getHostFacade()[OSF.DDA.DispIdHost.Methods.CloseDialog];e(arguments,r,o,i)}});var s=OSF.DDA.SyncMethodNames.AddMessageHandler.displayName;if(OSF.OUtil.defineEnumerableProperty(i,s,{value:function(){var e=OSF.DDA.SyncMethodCalls[OSF.DDA.SyncMethodNames.AddMessageHandler.id],t=e.verifyAndExtractCall(arguments,i,o),n=t[Microsoft.Office.WebExtension.Parameters.EventType],r=t[Microsoft.Office.WebExtension.Parameters.Handler];return o.addEventHandlerAndFireQueuedEvent(n,r)}}),!0===OSF.DDA.UI.EnableSendMessageDialogAPI){var l=OSF.DDA.SyncMethodNames.SendMessage.displayName;OSF.OUtil.defineEnumerableProperty(i,l,{value:function(){var e=OSF._OfficeAppFactory.getHostFacade()[OSF.DDA.DispIdHost.Methods.SendMessage];return e(arguments,o,i)}})}if(!0===OSF.DDA.UI.EnableMessageChildDialogAPI){var c=OSF.DDA.SyncMethodNames.MessageChild.displayName;OSF.OUtil.defineEnumerableProperty(i,c,{value:function(){var e=OSF._OfficeAppFactory.getHostFacade()[OSF.DDA.DispIdHost.Methods.SendMessage];return e(arguments,o,i)}})}return i},checkCallArgs:function(e,t,n){return e[Microsoft.Office.WebExtension.Parameters.Width]<=0&&(e[Microsoft.Office.WebExtension.Parameters.Width]=1),!e[Microsoft.Office.WebExtension.Parameters.UseDeviceIndependentPixels]&&e[Microsoft.Office.WebExtension.Parameters.Width]>100&&(e[Microsoft.Office.WebExtension.Parameters.Width]=99),e[Microsoft.Office.WebExtension.Parameters.Height]<=0&&(e[Microsoft.Office.WebExtension.Parameters.Height]=1),!e[Microsoft.Office.WebExtension.Parameters.UseDeviceIndependentPixels]&&e[Microsoft.Office.WebExtension.Parameters.Height]>100&&(e[Microsoft.Office.WebExtension.Parameters.Height]=99),e[Microsoft.Office.WebExtension.Parameters.RequireHTTPs]||(e[Microsoft.Office.WebExtension.Parameters.RequireHTTPs]=!0),e}})},e.messageChildRichApiBridge=function(){if(!0===OSF.DDA.UI.EnableMessageChildDialogAPI){var e=OSF._OfficeAppFactory.getHostFacade()[OSF.DDA.DispIdHost.Methods.SendMessage];return e(arguments,null,null)}},e.initOnce=function(){e.defineApi(OSF.DDA.AsyncMethodNames.DisplayDialogAsync,e.displayDialogAsyncApiSupportedOptions),e.defineApi(OSF.DDA.AsyncMethodNames.DisplayModalDialogAsync,e.displayModalDialogAsyncApiSupportedOptions)},e.displayDialogAsyncApiSupportedOptions=[{name:Microsoft.Office.WebExtension.Parameters.Width,value:{types:["number"],defaultValue:99}},{name:Microsoft.Office.WebExtension.Parameters.Height,value:{types:["number"],defaultValue:99}},{name:Microsoft.Office.WebExtension.Parameters.RequireHTTPs,value:{types:["boolean"],defaultValue:!0}},{name:Microsoft.Office.WebExtension.Parameters.DisplayInIframe,value:{types:["boolean"],defaultValue:!1}},{name:Microsoft.Office.WebExtension.Parameters.HideTitle,value:{types:["boolean"],defaultValue:!1}},{name:Microsoft.Office.WebExtension.Parameters.UseDeviceIndependentPixels,value:{types:["boolean"],defaultValue:!1}},{name:Microsoft.Office.WebExtension.Parameters.PromptBeforeOpen,value:{types:["boolean"],defaultValue:!0}},{name:Microsoft.Office.WebExtension.Parameters.EnforceAppDomain,value:{types:["boolean"],defaultValue:!0}},{name:Microsoft.Office.WebExtension.Parameters.UrlNoHostInfo,value:{types:["boolean"],defaultValue:!1}}],e.displayModalDialogAsyncApiSupportedOptions=e.displayDialogAsyncApiSupportedOptions.concat([{name:"abortWhenParentIsMinimized",value:{types:["boolean"],defaultValue:!1}},{name:"abortWhenDocIsInactive",value:{types:["boolean"],defaultValue:!1}}]),e}();DialogApiManager.initOnce(),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.CloseAsync,requiredArguments:[],supportedOptions:[],privateStateCallbacks:[]}),OSF.DDA.SyncMethodCalls.define({method:OSF.DDA.SyncMethodNames.MessageParent,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.MessageToParent,types:["string","number","boolean"]}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.TargetOrigin,value:{types:["string"],defaultValue:""}}]}),OSF.DDA.SyncMethodCalls.define({method:OSF.DDA.SyncMethodNames.AddMessageHandler,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.EventType,enum:Microsoft.Office.WebExtension.EventType,verify:function(e,t,n){return n.supportsEvent(e)}},{name:Microsoft.Office.WebExtension.Parameters.Handler,types:["function"]}],supportedOptions:[]}),OSF.DDA.SyncMethodCalls.define({method:OSF.DDA.SyncMethodNames.SendMessage,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.MessageContent,types:["string"]}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.TargetOrigin,value:{types:["string"],defaultValue:""}}],privateStateCallbacks:[]}),OSF.DDA.SafeArray.Delegate.openDialog=function(e){try{e.onCalling&&e.onCalling();var t=OSF.DDA.SafeArray.Delegate._getOnAfterRegisterEvent(!0,e);OSF.ClientHostController.openDialog(e.dispId,e.targetId,(function(t,n){e.onEvent&&e.onEvent(n),OSF.AppTelemetry&&OSF.AppTelemetry.onEventDone(e.dispId)}),t)}catch(t){OSF.DDA.SafeArray.Delegate._onException(t,e)}},OSF.DDA.SafeArray.Delegate.closeDialog=function(e){e.onCalling&&e.onCalling();var t=OSF.DDA.SafeArray.Delegate._getOnAfterRegisterEvent(!1,e);try{OSF.ClientHostController.closeDialog(e.dispId,e.targetId,t)}catch(t){OSF.DDA.SafeArray.Delegate._onException(t,e)}},OSF.DDA.SafeArray.Delegate.messageParent=function(e){try{e.onCalling&&e.onCalling();var t=(new Date).getTime(),n=OSF.ClientHostController.messageParent(e.hostCallArgs);return e.onReceiving&&e.onReceiving(),OSF.AppTelemetry&&OSF.AppTelemetry.onMethodDone(e.dispId,e.hostCallArgs,Math.abs((new Date).getTime()-t),n),n}catch(e){return OSF.DDA.SafeArray.Delegate._onExceptionSyncMethod(e)}},OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.EventDispId.dispidDialogMessageReceivedEvent,fromHost:[{name:OSF.DDA.EventDescriptors.DialogMessageReceivedEvent,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}],isComplexType:!0}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.EventDescriptors.DialogMessageReceivedEvent,fromHost:[{name:OSF.DDA.PropertyDescriptors.MessageType,value:0},{name:OSF.DDA.PropertyDescriptors.MessageContent,value:1},{name:OSF.DDA.PropertyDescriptors.MessageOrigin,value:2}],isComplexType:!0}),OSF.DDA.SafeArray.Delegate.sendMessage=function(e){try{e.onCalling&&e.onCalling();(new Date).getTime();var t=OSF.ClientHostController.sendMessage(e.hostCallArgs);return e.onReceiving&&e.onReceiving(),t}catch(e){return OSF.DDA.SafeArray.Delegate._onExceptionSyncMethod(e)}},function(e){var t={};function n(r){if(t[r])return t[r].exports;var o=t[r]={i:r,l:!1,exports:{}};return e[r].call(o.exports,o,o.exports,n),o.l=!0,o.exports}n.m=e,n.c=t,n.d=function(e,t,r){n.o(e,t)||Object.defineProperty(e,t,{enumerable:!0,get:r})},n.r=function(e){"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},n.t=function(e,t){if(1&t&&(e=n(e)),8&t)return e;if(4&t&&"object"==typeof e&&e&&e.__esModule)return e;var r=Object.create(null);if(n.r(r),Object.defineProperty(r,"default",{enumerable:!0,value:e}),2&t&&"string"!=typeof e)for(var o in e)n.d(r,o,function(t){return e[t]}.bind(null,o));return r},n.n=function(e){var t=e&&e.__esModule?function(){return e.default}:function(){return e};return n.d(t,"a",t),t},n.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},n.p="",n(n.s=1)}([function(e,t){e.exports=OfficeExtensionBatch},function(e,t,n){"use strict";var r=this&&this.__assign||function(){return(r=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var o in t=arguments[n])Object.prototype.hasOwnProperty.call(t,o)&&(e[o]=t[o]);return e}).apply(this,arguments)};Object.defineProperty(t,"__esModule",{value:!0});var o=n(2),i=n(3),a=n(5),s=n(6),l=n(7),c=n(8),d=n(9),u=n(10),f=n(13),p=n(14);window._useSDXSandbox?window._OfficeRuntimeNative=r(r({},window.OfficeRuntime),{displayWebDialog:s.displayWebDialog,AsyncStorage:l.AsyncStorage,storage:l.storage,experimentation:d.experimentation,apiInformation:i.apiInformation,message:u.Message.instance,ui:f.ui,auth:p.Auth.instance}):window._OfficeRuntimeNative=r(r({},window.OfficeRuntime),{displayWebDialog:s.displayWebDialog,AsyncStorage:a,storage:c.storage,experimentation:d.experimentation,apiInformation:o.apiInformation,message:u.Message.instance,ui:f.ui,auth:p.Auth.instance}),Object.freeze(window._OfficeRuntimeNative)},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.apiInformation=void 0,t.apiInformation={isSetSupported:function(e,t){try{return!!(Office&&Office.context&&Office.context.requirements)&&Office.context.requirements.isSetSupported(e,t)}catch(e){return!1}}}},function(e,t,n){"use strict";(function(e){function n(e){var t,n=e.split("."),r=0,o=0;if(n.length<2&&isNaN(Number(e)))throw"version format incorrect";if(t=Number(n[0]),n.length>=2&&(r=Number(n[1])),n.length>=3&&(o=Number(n[2])),isNaN(t)||isNaN(r)||isNaN(o))throw"version format incorrect";return{major:t,minor:r,patch:o}}Object.defineProperty(t,"__esModule",{value:!0}),t.apiInformation=void 0,t.apiInformation={isSetSupported:function(t,r){if("string"!=typeof t)return!1;if(null==r&&(r="0.0.0"),void 0===e.__apiSets)return!1;var o=e.__apiSets,i=Object.keys(o).map((function(e){return e.toLowerCase()})).indexOf(t.toLowerCase());if(i>-1){var a=o[Object.keys(o)[i]];try{var s,l=n(r);if((s=Array.isArray(a)?function(e,t){for(var r=0;r<e.length;r++){var o=n(e[r]);if(o.major==t)return o}return{major:0,minor:0,patch:0}}(a,l.major):n(a)).major>0){if(s.major>l.major)return!0;if(s.major==l.major&&s.minor>l.minor)return!0;if(s.major==l.major&&s.minor==l.minor&&s.patch>=l.patch)return!0}}catch(e){return!1}}return!1}}}).call(this,n(4))},function(e,t){var n;n=function(){return this}();try{n=n||new Function("return this")()}catch(e){"object"==typeof window&&(n=window)}e.exports=n},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.multiGet=t.multiRemove=t.multiSet=t.getAllKeys=t.clear=t.removeItem=t.setItem=t.getItem=void 0;var r="_Office_AsyncStorage_";function o(){window.localStorage.setItem("_Office_AsyncStorage_|_unusedKey_",null),window.localStorage.removeItem("_Office_AsyncStorage_|_unusedKey_")}function i(e,t){return void 0===t&&(t=function(){}),new Promise((function(n,r){try{o(),e(),t(null),n()}catch(e){t(e),r(e)}}))}function a(e,t){return void 0===t&&(t=function(){}),new Promise((function(n,r){try{o();var i=e();t(null,i),n(i)}catch(e){t(e,null),r(e)}}))}function s(e,t,n){return void 0===n&&(n=function(){}),new Promise((function(r,i){var a=[];try{o()}catch(e){a.push(e)}e.forEach((function(e){try{t(e)}catch(e){a.push(e)}})),n(a),a.length>0?i(a):r()}))}t.getItem=function(e,t){return a((function(){return window.localStorage.getItem(r+e)}),t)},t.setItem=function(e,t,n){return i((function(){return window.localStorage.setItem(r+e,t)}),n)},t.removeItem=function(e,t){return i((function(){return window.localStorage.removeItem(r+e)}),t)},t.clear=function(e){return i((function(){Object.keys(window.localStorage).filter((function(e){return 0===e.indexOf(r)})).forEach((function(e){return window.localStorage.removeItem(e)}))}),e)},t.getAllKeys=function(e){return a((function(){return Object.keys(window.localStorage).filter((function(e){return 0===e.indexOf(r)})).map((function(e){return e.substr(r.length)}))}),e)},t.multiSet=function(e,t){return s(e,(function(e){var t=e[0],n=e[1];return window.localStorage.setItem(r+t,n)}),t)},t.multiRemove=function(e,t){return s(e,(function(e){return window.localStorage.removeItem(r+e)}),t)},t.multiGet=function(e,t){return new Promise((function(n,o){t||(t=function(){});var i=[],a=e.map((function(e){try{return[e,window.localStorage.getItem(r+e)]}catch(e){i.push(e)}})).filter((function(e){return e}));i.length>0?(t(i,a),o(i)):(t(null,a),n(a))}))}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.displayWebDialog=t.Dialog=void 0;var r=n(0),o=function(){function e(e){this._dialog=e}return e.prototype.close=function(){return this._dialog.close(),r.CoreUtility.Promise.resolve()},e}();t.Dialog=o,t.displayWebDialog=function(e,t){return new r.CoreUtility.Promise((function(n,i){if(t.width&&t.height&&(!d(t.width)||!d(t.height)))throw new r.Error({code:"InvalidArgument",message:'Dimensions must be "number%" or number.'});var a,s={width:t.width?parseInt(t.width,10):50,height:t.height?parseInt(t.height,10):50,displayInIframe:t.displayInIFrame||!1};function l(e){t.onMessage&&t.onMessage(e.message,a,e.origin)}function c(e){12006===e.error?t.onClose&&t.onClose():t.onRuntimeError&&t.onRuntimeError(new r.Error(u(e.error)),a)}function d(e){return/^(\-|\+)?([0-9]+)%?$/.test(e)}function u(e){var t,n=((t={})[12002]={code:"InvalidUrl",message:"Cannot load URL, no such page or bad URL syntax."},t[12003]={code:"InvalidUrl",message:"HTTPS is required."},t[12004]={code:"Untrusted",message:"Domain is not trusted."},t[12005]={code:"InvalidUrl",message:"HTTPS is required."},t[12007]={code:"FailedToOpen",message:"Another dialog is already opened."},t);return n[e]?n[e]:{code:"Unknown",message:"An unknown error has occured"}}Office.context.ui.displayDialogAsync(e,s,(function(e){"failed"===e.status?i(new r.Error(u(e.error.code))):((a=e.value).addEventHandler(Office.EventType.DialogMessageReceived,l),a.addEventHandler(Office.EventType.DialogEventReceived,c),n(new o(a)))}))}))}},function(e,t,n){"use strict";var r,o=this&&this.__extends||(r=function(e,t){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(e,t)},function(e,t){function n(){this.constructor=e}r(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)});Object.defineProperty(t,"__esModule",{value:!0}),t.ErrorCodes=t.PersistentKvStorageService=t.storage=t.AsyncStorage=void 0;var i=n(0),a=(i.BatchApiHelper.createPropertyObject,i.BatchApiHelper.createMethodObject,i.BatchApiHelper.createIndexerObject,i.BatchApiHelper.createRootServiceObject,i.BatchApiHelper.createTopLevelServiceObject),s=(i.BatchApiHelper.createChildItemObject,i.BatchApiHelper.invokeMethod),l=(i.BatchApiHelper.invokeEnsureUnchanged,i.BatchApiHelper.invokeSetProperty,i.Utility.isNullOrUndefined),c=(i.Utility.isUndefined,i.Utility.throwIfNotLoaded,i.Utility.throwIfApiNotSupported,i.Utility.load,i.Utility.retrieve,i.Utility.toJson),d=i.Utility.fixObjectPathIfNecessary,u=(i.Utility._handleNavigationPropertyResults,i.Utility.adjustToDateTime,i.Utility.processRetrieveResult);function f(e,t){return new i.CoreUtility.Promise((function(n,r){var o=g.getInstance(),i=o.setCallBack((function(e,o){if(o)r(o);else{var i=t(e);n(i)}}));o.ctx.sync().then((function(){var t=o.getPersistentKvStorageService();return e(t,i),o.ctx.sync()})).catch((function(e){r(e)}))}))}function p(e,t,n){return new i.CoreUtility.Promise((function(r,o){var i=g.getInstance(),a=i.setCallBack((function(e,i){if(i)return n&&n(i),void o(i);var a=t(e);n&&n(null,a),r(a)}));i.ctx.sync().then((function(){var t=i.getPersistentKvStorageService();return e(t,a),i.ctx.sync()})).catch((function(e){o(e)}))}))}i.Utility.setMockData,i.CommonUtility.calculateApiFlags,t.AsyncStorage={getItem:function(e,t){return p((function(t,n){return t.multiGet(n,JSON.stringify([e]))}),(function(e){var t=JSON.parse(e);return t&&t[0]&&t[0][1]?t[0][1]:null}),t)},setItem:function(e,t,n){return p((function(n,r){return n.multiSet(r,JSON.stringify([[e,t]]))}),(function(){return null}),n)},removeItem:function(e,t){return p((function(t,n){return t.multiRemove(n,JSON.stringify([e]))}),(function(){return null}),t)},multiGet:function(e,t){return p((function(t,n){return t.multiGet(n,JSON.stringify(e))}),(function(t){var n=JSON.parse(t),r={};return n&&n.forEach((function(e){var t=e[0],n=e[1];return r[t]=n,n})),e.map((function(e){return[e,r[e]?r[e]:null]}))}),t)},multiSet:function(e,t){return p((function(t,n){return t.multiSet(n,JSON.stringify(e))}),(function(){return null}),t)},multiRemove:function(e,t){return p((function(t,n){return t.multiRemove(n,JSON.stringify(e))}),(function(){return null}),t)},getAllKeys:function(e){return p((function(e,t){return e.getAllKeys(t)}),(function(e){return JSON.parse(e)}),e)},clear:function(e){return p((function(e,t){return e.clear(t)}),(function(){return null}),e)}},t.storage={getItem:function(e){return f((function(t,n){return t.multiGet(n,JSON.stringify([e]))}),(function(e){var t=JSON.parse(e);return t&&t[0]&&t[0][1]?t[0][1]:null}))},setItem:function(e,t){return f((function(n,r){return n.multiSet(r,JSON.stringify([[e,t]]))}),(function(){return null}))},removeItem:function(e){return f((function(t,n){return t.multiRemove(n,JSON.stringify([e]))}),(function(){return null}))},getItems:function(e){return f((function(t,n){return t.multiGet(n,JSON.stringify(e))}),(function(t){var n=JSON.parse(t),r={};return e.forEach((function(e){r[e]=null})),n&&n.forEach((function(e){var t=e[0],n=e[1];return r[t]=n,n})),r}))},setItems:function(e){var t=[];for(var n in e)e.hasOwnProperty(n)&&t.push([n,e[n]]);return f((function(e,n){return e.multiSet(n,JSON.stringify(t))}),(function(){return null}))},removeItems:function(e){return f((function(t,n){return t.multiRemove(n,JSON.stringify(e))}),(function(){return null}))},getKeys:function(){return f((function(e,t){return e.getAllKeys(t)}),(function(e){return JSON.parse(e)}))}};var g=function(){function e(){var e=this;this._invokeId=0,this._callDict={},this.ctx=new i.ClientRequestContext,this._perkvstorService=m.newObject(this.ctx),this._eventResult=this._perkvstorService.onPersistentStorageMessage.add((function(t){i.Utility.log("persistentKvStoragegMessageHandler:"+JSON.stringify(t));var n=e._callDict[t.invokeId];n&&(n(t.message,t.error),delete e._callDict[t.invokeId])}))}return e.getInstance=function(){return void 0===e.instance?e.instance=new e:e.instance._perkvstorService=m.newObject(e.instance.ctx),e.instance},e.prototype.getPersistentKvStorageService=function(){return this._perkvstorService},e.prototype.getCallBack=function(e){return this._callDict[e]},e.prototype.setCallBack=function(e){var t=this._invokeId;return this._callDict[this._invokeId++]=e,t},e}(),m=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return o(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"PersistentKvStorageService"},enumerable:!1,configurable:!0}),t.prototype.clear=function(e){s(this,"Clear",1,[e],4,0)},t.prototype.getAllKeys=function(e){s(this,"GetAllKeys",1,[e],4,0)},t.prototype.multiGet=function(e,t){s(this,"MultiGet",1,[e,t],4,0)},t.prototype.multiRemove=function(e,t){s(this,"MultiRemove",1,[e,t],4,0)},t.prototype.multiSet=function(e,t){s(this,"MultiSet",1,[e,t],4,0)},t.prototype._handleResult=function(t){e.prototype._handleResult.call(this,t),l(t)||d(this,t)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),u(this,t,n)},t.newObject=function(e){return a(t,e,"Microsoft.PersistentKvStorage.PersistentKvStorageService",!1,4)},Object.defineProperty(t.prototype,"onPersistentStorageMessage",{get:function(){return this.m_persistentStorageMessage||(this.m_persistentStorageMessage=new i.GenericEventHandlers(this.context,this,"PersistentStorageMessage",{eventType:65537,registerFunc:function(){},unregisterFunc:function(){},getTargetIdFunc:function(){return null},eventArgsTransformFunc:function(e){var t,n,r,o;try{var a=JSON.parse(e.message),s=parseInt(a.errorCode),l=0!=s?new i.Error((n=s,(r={})[16389]={code:"GenericException",message:"Unknown error."},r[65535]={code:"Unexcepted",message:"Catastrophic failure."},r[14]={code:"OutOfMemory",message:"Ran out of memory."},r[87]={code:"InvalidArg",message:"One or more arguments are invalid."},r[16385]={code:"NotImplemented",message:"Not implemented."},r[6]={code:"BadHandle",message:"File Handle is not Set."},r[5]={code:"AccessDenied",message:"Can't read the Storage File."},(o=r)[n]?o[n]:{code:"Unknown",message:"An unknown error has occured"})):null;t={invokeId:a.invokeId,message:a.message,error:l}}catch(e){t={invokeId:-1,message:e.message,error:new i.Error({code:"GenericException",message:"Unknown error"})}}return i.Utility._createPromiseFromResult(t)}})),this.m_persistentStorageMessage},enumerable:!1,configurable:!0}),t.prototype.toJSON=function(){return c(this,{},{})},t}(i.ClientObject);t.PersistentKvStorageService=m,(t.ErrorCodes||(t.ErrorCodes={})).generalException="GeneralException"},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.storage=void 0;var r="_OfficeRuntime_Storage_";function o(){window.localStorage.setItem("_OfficeRuntime_Storage_|_unusedKey_",null),window.localStorage.removeItem("_OfficeRuntime_Storage_|_unusedKey_")}function i(e){return new Promise((function(t,n){try{o(),e(),t()}catch(e){n(e)}}))}function a(e){return new Promise((function(t,n){try{o(),t(e())}catch(e){n(e)}}))}function s(e,t){return new Promise((function(n,r){var i=[];try{o()}catch(e){i.push(e)}for(var a in e)if(e.hasOwnProperty(a)||Array.isArray(e))try{Array.isArray(e)?t(e[a]):t(a)}catch(e){i.push(e)}i.length>0?r(new Error("Unknown error.")):n()}))}t.storage={getItem:function(e){return a((function(){return window.localStorage.getItem(r+e)}))},setItem:function(e,t){return i((function(){return window.localStorage.setItem(r+e,t)}))},removeItem:function(e){return i((function(){return window.localStorage.removeItem(r+e)}))},getItems:function(e){return new Promise((function(t,n){var i={},a=[];try{o()}catch(e){n(e)}e.forEach((function(e){try{var t=window.localStorage.getItem(r+e);i[e]=t||null}catch(e){a.push(e)}})),a.length>0?n(new Error("Unknown error.")):t(i)}))},setItems:function(e){return s(e,(function(t){return window.localStorage.setItem(r+t,e[t])}))},removeItems:function(e){return s(e,(function(e){window.localStorage.removeItem(r+e)}))},getKeys:function(){return a((function(){return Object.keys(window.localStorage).filter((function(e){return 0===e.indexOf(r)})).map((function(e){return e.substr(r.length)}))}))}}},function(e,t,n){"use strict";var r=this&&this.__awaiter||function(e,t,n,r){return new(n||(n=Promise))((function(o,i){function a(e){try{l(r.next(e))}catch(e){i(e)}}function s(e){try{l(r.throw(e))}catch(e){i(e)}}function l(e){var t;e.done?o(e.value):(t=e.value,t instanceof n?t:new n((function(e){e(t)}))).then(a,s)}l((r=r.apply(e,t||[])).next())}))},o=this&&this.__generator||function(e,t){var n,r,o,i,a={label:0,sent:function(){if(1&o[0])throw o[1];return o[1]},trys:[],ops:[]};return i={next:s(0),throw:s(1),return:s(2)},"function"==typeof Symbol&&(i[Symbol.iterator]=function(){return this}),i;function s(i){return function(s){return function(i){if(n)throw new TypeError("Generator is already executing.");for(;a;)try{if(n=1,r&&(o=2&i[0]?r.return:i[0]?r.throw||((o=r.return)&&o.call(r),0):r.next)&&!(o=o.call(r,i[1])).done)return o;switch(r=0,o&&(i=[2&i[0],o.value]),i[0]){case 0:case 1:o=i;break;case 4:return a.label++,{value:i[1],done:!1};case 5:a.label++,r=i[1],i=[0];continue;case 7:i=a.ops.pop(),a.trys.pop();continue;default:if(!((o=(o=a.trys).length>0&&o[o.length-1])||6!==i[0]&&2!==i[0])){a=0;continue}if(3===i[0]&&(!o||i[1]>o[0]&&i[1]<o[3])){a.label=i[1];break}if(6===i[0]&&a.label<o[1]){a.label=o[1],o=i;break}if(o&&a.label<o[2]){a.label=o[2],a.ops.push(i);break}o[2]&&a.ops.pop(),a.trys.pop();continue}i=t.call(e,a)}catch(e){i=[6,e],r=0}finally{n=o=0}if(5&i[0])throw i[1];return{value:i[0]?i[1]:void 0,done:!0}}([i,s])}}};Object.defineProperty(t,"__esModule",{value:!0}),t.experimentation=void 0;var i=function(){function e(){}return e.prototype.getBooleanFeatureGate=function(e,t){try{var n=Microsoft.Office.WebExtension.FeatureGates[e];return"true"===n.toString().toLowerCase()||!0===n}catch(e){return t}},e.prototype.getIntFeatureGate=function(e,t){try{var n=parseInt(Microsoft.Office.WebExtension.FeatureGates[e]);return isNaN(n)?t:n}catch(e){return t}},e.prototype.getStringFeatureGate=function(e,t){try{var n=Microsoft.Office.WebExtension.FeatureGates[e];return null==n?t:n}catch(e){return t}},e.prototype.getChangeGate=function(e){return this.getBooleanFeatureGate(e,!0)},e.prototype.getBooleanFeatureGateAsync=function(e,t){return r(this,void 0,void 0,(function(){var n,r;return o(this,(function(o){switch(o.label){case 0:return o.trys.push([0,2,,3]),n=this.getRequestContext(),r=n.flighting.getFeature(e,"Boolean",t),n.load(r),[4,n.sync()];case 1:return o.sent(),[2,r.value];case 2:return o.sent(),[2,Promise.resolve(t)];case 3:return[2]}}))}))},e.prototype.getStringFeatureGateAsync=function(e,t){return r(this,void 0,void 0,(function(){var n,r;return o(this,(function(o){switch(o.label){case 0:return o.trys.push([0,2,,3]),n=this.getRequestContext(),r=n.flighting.getFeature(e,"String",t),n.load(r),[4,n.sync()];case 1:return o.sent(),[2,r.value];case 2:return o.sent(),[2,Promise.resolve(t)];case 3:return[2]}}))}))},e.prototype.getIntFeatureGateAsync=function(e,t){return r(this,void 0,void 0,(function(){var n,r;return o(this,(function(o){switch(o.label){case 0:return o.trys.push([0,2,,3]),n=this.getRequestContext(),r=n.flighting.getFeature(e,"Integer",t),n.load(r),[4,n.sync()];case 1:return o.sent(),[2,r.value];case 2:return o.sent(),[2,Promise.resolve(t)];case 3:return[2]}}))}))},e.prototype.getRequestContext=function(){var e;if(Office.context.platform!==Office.PlatformType.OfficeOnline){switch(Office.context.host){case Office.HostType.Excel:e=new Excel.RequestContext;break;case Office.HostType.OneNote:e=new OneNote.RequestContext;break;case Office.HostType.PowerPoint:e=new PowerPoint.RequestContext;break;case Office.HostType.Word:e=new Word.RequestContext;break;default:throw new Error("Unknown Application "+Office.context.host)}return e}},e}();t.experimentation=new i},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Message=void 0;var r=n(11);Object.defineProperty(t,"Message",{enumerable:!0,get:function(){return r.Message}})},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Message=void 0;var r=n(12),o=function(){function e(){this.listeners={}}return e.prototype.on=function(e,t){return this.add(e,t),new Promise((function(e){e()}))},e.prototype.off=function(e,t){return this.remove(e,t),new Promise((function(e){e()}))},e.prototype.emit=function(e,t){return this.send(e,t),new Promise((function(e){e()}))},Object.defineProperty(e,"instance",{get:function(){return e.singleton||(e.singleton=new e),this.singleton},enumerable:!1,configurable:!0}),e.prototype.setupReceive=function(){Office&&Office.context&&Office.context.messaging&&!Office.context.messaging.onMessage&&(Office.context.messaging.onMessage=this.receiveMessage.bind(this))},e.prototype.add=function(e,t){this.listeners.hasOwnProperty(e)||(this.listeners[e]=new r.ListenerManager,this.setupReceive()),this.listeners[e].add(t)},e.prototype.remove=function(e,t){this.listeners.hasOwnProperty(e)&&(t?this.listeners[e].remove(t):delete this.listeners[e])},e.prototype.send=function(e,t){var n={eventName:e,message:t};Office&&Office.context&&Office.context.messaging&&Office.context.messaging.sendMessage&&Office.context.messaging.sendMessage(n)},e.prototype.receiveMessage=function(e){var t=e.eventName,n=e.message;this.listeners.hasOwnProperty(t)&&this.listeners[t].call(n)},e}();t.Message=o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.ListenerManager=void 0;var r=function(){function e(){this.listeners=[]}return e.prototype.add=function(e){this.listeners.push(e)},e.prototype.remove=function(e){var t=this.listeners.lastIndexOf(e);-1!==t&&this.listeners.splice(t,1)},e.prototype.call=function(e){this.listeners.forEach((function(t){return t(e)}))},e}();t.ListenerManager=r},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Ribbon=t.ui=void 0,t.ui={getRibbon:function(){return new Promise((function(e,t){e(new r)}))}};var r=function(){function e(){this.requestContext=new OfficeCore.RequestContext,OSF.WebAuth&&"web"==OSF._OfficeAppFactory.getHostInfo().hostPlatform&&(this.requestContext._customData="WacPartition")}return e.prototype.requestUpdate=function(e){var t=this.requestContext.ribbon;return e.tabs.filter((function(e){return!!e.id})).forEach((function(e){var n=t.getTab(e.id);void 0!==e.visible&&null!==e.visible&&n.setVisibility(e.visible),e.controls.filter((function(e){return!!e.id})).forEach((function(e){var n=t.getButton(e.id);void 0!==e.enabled&&null!==e.enabled&&(n.enabled=e.enabled)}))})),this.requestContext.sync()},e}();t.Ribbon=r},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Auth=void 0;var r=n(0),o=function(){function e(){}return e.prototype.getAccessToken=function(e){return new r.CoreUtility.Promise((function(t,n){try{Office.context.auth.getAccessTokenAsync(e||{},(function(e){"succeeded"===e.status?t(e.value):n(e.error)}))}catch(e){n(e)}}))},Object.defineProperty(e,"instance",{get:function(){return e.singleton||(e.singleton=new e),e.singleton},enumerable:!1,configurable:!0}),e}();t.Auth=o}]);